/*
 * Arduino compressed audio output
 
 BTc Sound Compression Algorithm. Roman Black.
 http://www.romanblack.com/picsound.htm
 */

#include <SPI.h>
#include <GD.h>
#include <digitalWriteFast.h>
#include <TimerOne.h>

#include <avr/pgmspace.h>


extern const uint32_t battle01,battle01Size;
extern const uint32_t battle02,battle02Size;
extern const uint32_t battle03,battle03Size;
extern const uint32_t battle04,battle04Size;


uint32_t addr;

#include "sounddata.h"
//#include "morepgmspace.h"
//#include "battle01.h"
//#include "battle02.h"
//#include "battle03.h"
//#include "battle04.h"

static uint8_t nextPwm=1;
byte writepointer = 0;
#define SOUNDBUFFER 0x3f00


char im[128], data[128], lastpass[64];


#define RENDERWIDTH 50
#define RENDERHEIGHT 37

#define SCREEN_WIDTH 400
#define SCREEN_HEIGHT 224   // 296
static byte flip=0; // sprite double buffer page

byte fire_buffer[(RENDERWIDTH*RENDERHEIGHT)+RENDERWIDTH+1];



void fillSampleBuffer(){
  byte readpointer = GD.rd(COMM+0);
  byte fullness = writepointer - readpointer;

  while (fullness < 254  ) {
    BTC();
    GD.wr(SOUNDBUFFER  + writepointer,(nextPwm));                 
    writepointer++;
    fullness++;
    
    
      data[fullness&127] = nextPwm/4 ;
      im[fullness&127] = 0; 
    
  }
 
 
}

int pos=0;
byte currentbit =7;
#define OUTPUT_AMPLITUDE  100 //# Directly propertional to the "loudness" of the wave (the output wave amplitude)
int DV = ( OUTPUT_AMPLITUDE / 2 ); //# Beginning amplitude
byte c;

//# Perfectly simulates the BTC waveform just as shown by RomanBlack's BTc encoder application
inline int V (boolean BIT_TO_BE_SENT, int V_INITIAL){
  if (BIT_TO_BE_SENT)
    return ( V_INITIAL + (( OUTPUT_AMPLITUDE - V_INITIAL) >> 2) );
  else
    return V_INITIAL - (V_INITIAL >> 2);
}

int bank =1;

byte getByte(uint32_t src){
  return src<65536 ? pgm_read_byte_near(src) : pgm_read_byte_far(src);
}

void BTC(){
  DV = V( bitRead(c, --currentbit), DV);

  // get a new byte every 8th bit read
  if (currentbit==0){ 
    c = getByte(addr + pos++);
    currentbit=8;
  }

  switchBank();
  nextPwm =  DV;
}

inline void switchBank(){
  switch (bank)
  {
  case 1:
    if (pos > 32000) {
      pos=0; 
      bank=2;
      addr = GET_FAR_ADDRESS(battle01);
    }; 
    break;

/*
  case 2:
    if (pos > 32000) {
      pos=0; 
      bank=3;
      addr = GET_FAR_ADDRESS(battle03);
    }; 
    break;


  case 3:
    if (pos > 32000) {
      pos=0; 
      bank=4;
      addr = GET_FAR_ADDRESS(battle01);
    }; 
    break;

  case 4:
    if (pos > 23176) {
      pos=0; 
      bank=1;
      addr = GET_FAR_ADDRESS(battle01); // loop
    }; 
    break;
    */
  }
}


void setup()
{
  GD.begin();
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 0x0FFF); // second number is color 0-3        

  GD.microcode(soundbuffer_code, sizeof(soundbuffer_code));
  Timer1.initialize(17000);         // initialize timer1, and set the period in microseconds.     

  generate_332_palette();
  showPalette();

  //  flashBTC=battle01;
  addr = GET_FAR_ADDRESS(battle01);

  Timer1.attachInterrupt(fillSampleBuffer);  // attaches callback() as a timer overflow interrupt  
  
  
 for (int z=0; z<64; z++) {lastpass[z]=0;};       //  fill the lastpass[] array with dummy data
      
  
}


char x=32, ylim=37;
int i, val;


// http://blurtime.blogspot.com/2010/11/arduino-realtime-audio-spectrum.html
void loop()
{

spec();

  
  
}



void spec(){
  int static i = 0;
  static long tt;
  int val;
   
//   if (millis() > tt){


        //this could be done with the fix_fftr function without the im array.
        fix_fft(data,im,7,0);
        // I am only interessted in the absolute value of the transformation
        for (i=1; i< 50;i++){
           data[i] = sqrt(data[i] * data[i] + im[i] * im[i]); 
            RenderLine(i+x,lastpass[i],i+x,ylim,color(0,0,0));          // and code may fix this, but for now, I
            RenderLine(i+x,ylim,i+x,ylim-data[i],color(255,0,0));        // skip displaying the 0-500hz band completely.
            lastpass[i]=ylim-data[i];                                   // if you insist, initialize the loop with 0      rather than 1.   
        }
//      }
      
    fb();
    GD.waitvblank();
//    tt = millis();

}












// lets see our palette on screen
void showPalette(){
  // show color palette
  int buffer=0;
  int counter = 0;
  for (int y=0;y<16;y++) {
    for (int x=0;x<16;x++) {
      GD.wr(RAM_PIC+(x+buffer), counter++);   // index here
    }
    buffer+=64;
  }  
}

// create 256 colors
void generate_332_palette()
{
  for (int c=0; c < 256; c++) {
    int r = (c&7) * 0xff/7;
    int g = ((c>>3)&7) * 0xff/7;
    int b = ((c>>6)&3) * 0xff/3;
    GD.setpal(c*4, RGB(r,g,b));
  }
} 





void RenderLine(int x0, int y0, int x1, int y1, char lineChar)
{
  int dx = abs(x1-x0);
  int dy = abs(y1-y0);

  int sx, sy;

  if(x0 < x1)
    sx = 1;
  else
    sx = -1;
  if(y0 < y1)
    sy = 1;
  else
    sy = -1;

  int err = dx - dy;

  while(true)
  {
    DrawChar(x0, y0, lineChar);
    if(x0 == x1 && y0 == y1)
      break;
    int e2 = 2 * err;
    if(e2 > -dy)
    {
      err = err - dy;
      x0 = x0 + sx;
    }
    if(e2 < dx)
    {
      err = err + dx;
      y0 = y0 + sy;
    }
  }
}



void DrawChar(int x, int y, byte newChar)
{
  /*
    char msg[2];
   msg[0] = newChar;
   msg[1] = 0;
   GD.putstr(x, y, msg);
   */

  //    pD[x+y*RENDERWIDTH]=newChar;


  fire_buffer[x+y*RENDERWIDTH]=newChar;    


}




/* write framebuffer array to gameduino character memory */
#define LRBLOCK (RENDERHEIGHT*64)+RAM_PIC // last visible  address on bottom right of screen

// fast version of fill buffer.   frame buffer as array. copy to screen characters
inline void fb(){
  unsigned int pixelCount =0;
  // character map is 64x64 but we only need to updated 50x37 so we skip every 64 Y blocks 
  for (unsigned int y = RAM_PIC; y < LRBLOCK;y+=64) { 
    byte fillWidth = RENDERWIDTH;
    GD.__wstart(y);
    while (fillWidth--)
    {
      SPI.transfer(fire_buffer[pixelCount++]);
      //      SPI.transfer(color(0,255,0));      
    }
    GD.__end();
  }  
}

// 256 color indexed palette RGB color conversion 
byte color(byte r, byte g, byte b){
  return ((b&255 & 0xC0)+((g&255 & 0xE0) >> 2)+((r & 0xE0) >> 5))&0xFF;
}

