/*

 Name      :  2D Starfield
 Notes     :  sprite based stars scrollinig horizontally
 
 */

#define NUM_STARS_2D 40
#define VERTICAL_VELOCITY 0  

class StarField2D
{
public:
  StarField2D ()
  {
    min_brightness = 50;
    top_star_speed = 8;
    init();

    GD.copy(RAM_SPRPAL, sm_logo_sprpal, sizeof(sm_logo_sprpal));
    GD.uncompress(RAM_SPRIMG, sm_logo_sprimg);

  }

  void init()
  {
    int brightness;

    //Initialise each star
    for(int i = 0; i < NUM_STARS_2D ; i++)
    {
      //Inialise velocities
      vel_x[i] = -(int)floor( (random(0,top_star_speed)  )+1 );
      vel_y[i] = VERTICAL_VELOCITY;

      //Initialise positions randomly
      pos_x[i] = (int)floor((random(0,SCREEN_WIDTH)  ));
      pos_y[i] = (int)floor((random(0,SCREEN_HEIGHT)  ));

      //The Faster it goes, the Dimmer it is
      if (vel_x[i] != 0)
      {
        brightness = (int)(255 / fabs(vel_x[i]) );
        if (brightness < min_brightness)
          brightness = min_brightness;
      }
      else
        brightness = 255;

      star_color[i] = brightness;
    }
  }

  void move()
  {
    for(int i = 0; i < NUM_STARS_2D ; i++)
    {
      //Check speed limits x
      if (vel_x[i] > top_star_speed)
        vel_x[i] = top_star_speed;
      else if (vel_x[i] < -top_star_speed)
        vel_x[i] = -top_star_speed;

      //Check speed limits y
      if (vel_y[i] > top_star_speed)
        vel_y[i] = top_star_speed;
      else if (vel_y[i] < -top_star_speed)
        vel_y[i] = -top_star_speed;

      //Move Star
      pos_x[i] += vel_x[i];
      pos_y[i] += vel_y[i];

      if (pos_x[i] < 0)
        pos_x[i] = pos_x[i] + SCREEN_WIDTH;

      if (pos_x[i] > SCREEN_WIDTH)
        pos_x[i] = pos_x[i] - SCREEN_WIDTH;

      if (pos_y[i] < 0)
        pos_y[i] = pos_y[i] + SCREEN_HEIGHT;

      if (pos_y[i] > SCREEN_HEIGHT)
        pos_y[i] = pos_y[i] - SCREEN_HEIGHT;
    }
  }

  void draw()
  {
    move();

    GD.__wstartspr(flip ? 256 : 0);      

    for(int i = 0; i < NUM_STARS_2D; i++)
    {
      int darkness=  map(star_color[i], min_brightness, 255, 1, 10);// 10 is number of shaded stars
      draw_sm_star_shaded(pos_x[i], pos_y[i], darkness, 0);
    }

    draw_sm_logo((SCREEN_WIDTH/2)+8, (SCREEN_HEIGHT/2)+8, 0, 0);    

    GD.__end();
    GD.waitvblank(); 
    GD.wr(SPR_PAGE, flip);
    flip = !flip;
  }

private:
  int star_color[NUM_STARS_2D];
  int vel_x[NUM_STARS_2D];
  int vel_y[NUM_STARS_2D];
  int pos_x[NUM_STARS_2D];
  int pos_y[NUM_STARS_2D];
  int min_brightness;
  int top_star_speed;
};


