/*

 Name      :  Common
 Notes     :  global functions used by all effects
 
 reusable functions to set up palettes, clear the screen etc
 
 */

#include <SPI.h>
#include <GD.h>

#include <stdlib.h> 

/* memory manangement functions */
//http://arduino.cc/forum/index.php?PHPSESSID=05d8c47eb8378217198adf881d939657&/topic,95914.msg1036128.html#msg1036128 
// https://github.com/arduino/Arduino/blob/d457332664730fde14146649169b1fdfe2209514/hardware/arduino/cores/arduino/malloc.c


/*
// uncomment this section if for older versions of AVR toolchain
 
 void* operator new(size_t size)
 {
 void* mem = malloc(size);
 if (!mem) {
 //throw std::bad_alloc();
 }
 return mem;
 }
 
 void operator delete(void* ptr)
 {
 free(ptr);
 }
 
 void* operator new[] (size_t size)
 {
 return (operator new)(size);
 }
 
 void operator delete[](void* ptr)
 {
 return (operator delete)(ptr);
 }
 */


#define SQ(v) ((v) * (v))
#define RANDOM_RGB() RGB(random(256),random(256),random(256))

#define RENDERWIDTH 50
#define RENDERHEIGHT 37

#define SCREEN_WIDTH 400
#define SCREEN_HEIGHT 296 // 224   // 296
static byte flip=0; // sprite double buffer page

#define bufferY (RENDERHEIGHT*64)+RAM_PIC

static byte fire_buffer[(RENDERWIDTH*RENDERHEIGHT)+RENDERWIDTH+1];

static int frameCounter = 0;

// grab a screenshot over serial port to assemble video capture
void screenshot(){
  //       GD.screenshot(frameCounter++);  
}

/* color functions */

// 256 color indexed palette RGB color conversion 
byte color(byte r, byte g, byte b){
  return ((b&255 & 0xC0)+((g&255 & 0xE0) >> 2)+((r & 0xE0) >> 5))&0xFF;
}

// lets see our palette on screen
void showPalette(){
  // show color palette
  int buffer=0;
  int counter = 0;
  for (int y=0;y<16;y++) {
    for (int x=0;x<16;x++) {
      GD.wr(RAM_PIC+(x+buffer), counter++);   // index here
    }
    buffer+=64;
  }  
}

// create 256 colors
void generate_332_palette()
{

  // clear screen of character data  
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 4096); // second number is color 0-3  
  
//    GD.fill(RAM_PAL, 0, 0x0FF); // second number is color 0-3  
  
  for (int c=0; c < 256; c++) {
    int r = (c&7) * 0xff/7;
    int g = ((c>>3)&7) * 0xff/7;
    int b = ((c>>6)&3) * 0xff/3;
    GD.setpal(c*4, RGB(r,g,b));
  }
} 

// fade to black
void fade(){
  for (byte i = 0; i < 32; i++) {
    for (int j = RAM_PAL; j < (RAM_PAL + 2048); j += 2) {
      uint16_t pal = GD.rd16(j);
      byte r = 31 & (pal >> 10);
      byte g = 31 & (pal >> 5);
      byte b = 31 & pal;
      if (r) r--;
      if (g) g--;
      if (b) b--;
      pal = (r << 10) | (g << 5) | b;
      GD.wr16(j, pal);
    }
//    GD.waitvblank();
    GD.waitvblank();
  }  
}

void bgfade(byte t)
{
  for (byte i = 0; i < 64; i++) {
    byte l = ((t == 31) ? 8 : 0) + ((i * t) >> 4);
    GD.wr16(0x3e80 + 2 * i, RGB(l, l, l));
  }
}

static void fadeSprite(uint16_t addr)
{
  uint16_t col = GD.rd16(addr);
  if (col & (31 << 10)) col -= (1 << 10);
  if (col & (31 << 5))  col -= (1 << 5);
  if (col & 31)         col -= 1;
  GD.wr16(addr, col);
}


// Fade 16 palette entries starting at addr
static void fade16(uint16_t addr)
{
  uint16_t cols[16];
  GD.__start(addr);
  for (byte i = 0; i < 16; i++) {
    byte lo = SPI.transfer(0);
    byte hi = SPI.transfer(0);
    uint16_t col = (hi << 8) | lo;
    if (col & (31 << 10)) col -= (1 << 10);
    if (col & (31 << 5))  col -= (1 << 5);
    if (col & 31)         col -= 1;
    cols[i] = col;
  }
  GD.__end();

  GD.__wstart(addr);
  for (byte i = 0; i < 16; i++) {
    uint16_t col = cols[i];
    SPI.transfer(lowByte(col));
    SPI.transfer(highByte(col));
  }
  GD.__end();
}

void fadeAll(){
  for (int t = 0; t < 8 * 72; t++) {
    byte frame = (t & 1);

    if (t < 32)
      bgfade(t);
    if (((8 * 72) - 32) < t) {
      bgfade(8 * 72 - t);
      for (uint16_t a = RAM_PAL; a < RAM_PAL + (256 * 4 * 2); a += 32)
        fade16(a);
      fade16(PALETTE16A);
    }
  }
}

/* draw primatives */
void DrawChar(int x, int y, byte newChar)
{
  /*
    char msg[2];
   msg[0] = newChar;
   msg[1] = 0;
   GD.putstr(x, y, msg);
   */

  //    pD[x+y*RENDERWIDTH]=newChar;


  fire_buffer[x+y*RENDERWIDTH]=newChar;    


}

void RenderLine(int x0, int y0, int x1, int y1, char lineChar)
{
  int dx = abs(x1-x0);
  int dy = abs(y1-y0);

  int sx, sy;

  if(x0 < x1)
    sx = 1;
  else
    sx = -1;
  if(y0 < y1)
    sy = 1;
  else
    sy = -1;

  int err = dx - dy;

  while(true)
  {
    DrawChar(x0, y0, lineChar);
    if(x0 == x1 && y0 == y1)
      break;
    int e2 = 2 * err;
    if(e2 > -dy)
    {
      err = err - dy;
      x0 = x0 + sx;
    }
    if(e2 < dx)
    {
      err = err + dx;
      y0 = y0 + sy;
    }
  }
}


void crossFade(int amount)
{  
  // All event rows start from the left
  for(int y = 0; y < RENDERHEIGHT-1; y += 2)
  {
    int dX = (RENDERWIDTH-amount)+1;
    if (dX<0)dX=0;
    RenderLine(0, y, dX, y, 0);        
  }
  for(int y = RENDERHEIGHT-1; y >= 0; y -= 2)
  {
    int dX = amount-2;
    if (dX<0)dX=0;
    RenderLine(RENDERWIDTH-1, y, dX, y, 0);
  }
}

/* write framebuffer array to gameduino character memory */
#define LRBLOCK (RENDERHEIGHT*64)+RAM_PIC // last visible  address on bottom right of screen

// fast version of fill buffer.   frame buffer as array. copy to screen characters
inline void fb(){
  unsigned int pixelCount =0;
  // character map is 64x64 but we only need to updated 50x37 so we skip every 64 Y blocks 
  for (unsigned int y = RAM_PIC; y < LRBLOCK;y+=64) { 
    byte fillWidth = RENDERWIDTH;
    GD.__wstart(y);
    while (fillWidth--)
    {
      SPI.transfer(fire_buffer[pixelCount++]);
      //      SPI.transfer(color(0,255,0));      
    }
    GD.__end();
  }  
}


byte getByte(uint32_t src){
  return src<65536 ? pgm_read_byte_near(src) : pgm_read_byte_far(src);
}

void clearSprites(){
  for (int page=0; page<2; page++)
  {  
    GD.__wstartspr(page ? 256 : 0);      
    for (int i=0; i<256; i++)
    {
      GD.xsprite(400, 0, 0, 0, i, 0, 0, 0);
    }
    GD.__end();

    GD.wr(SPR_PAGE, page);
    //    flip = !flip;
  }
}

void clearScreen()
{
  // reset screen to top 0,0 position
  GD.wr16(SCROLL_Y, 0); 

  // Clear the screen of sprite data
  //  GD.fill(RAM_SPRIMG, 0, 16384);

  clearSprites();

  // clear screen of character data
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 0x0FFF); // second number is color 0-3  

  // clear internal frame buffer
  memset(fire_buffer, 0, sizeof(fire_buffer) );
  //fb();
}

int availableMemory() {
  // Use 1024 with ATmega168
  int size = 2048;
  byte *buf;
  while ((buf = (byte *) malloc(--size)) == NULL);
  free(buf);
  return size;
}


int pixelOffset(int x, int y)
{
  return (y * RENDERWIDTH) + x;
}


// report frame rate in top-right
void getFPS(){
  { 
    static byte every;
    if (++every == 4) {
      static long tprev;
      long t = micros();
      every = 0;

      char msg[30];
      int fps10 = int(4 * 10000000UL / (t - tprev));

      //      int fps10 = div16u(4 * 10000000UL,(t - tprev));

      sprintf(msg, "%3d.%d fps  ", fps10 / 10, fps10 % 10);
      GD.putstr(41, 0, msg);
      tprev = t;
    }
  }
}




static PROGMEM prog_uchar space_pic[] = {

0x00,  0x01,  0x02,  0x03,  0x03,  0x04,  0x02,  0x02,  0x05,  0x06,  0x07,  0x07,  0x07,  0x06,  0x03,  0x06, 
0x03,  0x08,  0x08,  0x09,  0x08,  0x08,  0x09,  0x01,  0x09,  0x07,  0x06,  0x0a,  0x04,  0x0b,  0x0c,  0x0d, 
0x0e,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x03,  0x06,  0x06,  0x06,  0x03,  0x02,  0x03,  0x07, 
0x06,  0x07,  0x06,  0x06,  0x02,  0x03,  0x12,  0x0e,  0x12,  0x03,  0x02,  0x13,  0x13,  0x07,  0x03,  0x13, 
0x0a,  0x04,  0x14,  0x0a,  0x06,  0x15,  0x03,  0x16,  0x17,  0x06,  0x07,  0x07,  0x07,  0x03,  0x02,  0x03, 
0x02,  0x01,  0x16,  0x08,  0x01,  0x16,  0x09,  0x00,  0x18,  0x07,  0x03,  0x17,  0x16,  0x01,  0x15,  0x03, 
0x06,  0x0e,  0x0f,  0x10,  0x11,  0x10,  0x0f,  0x02,  0x03,  0x03,  0x06,  0x03,  0x02,  0x0d,  0x02,  0x03, 
0x07,  0x06,  0x07,  0x06,  0x07,  0x02,  0x03,  0x02,  0x03,  0x02,  0x03,  0x02,  0x07,  0x07,  0x07,  0x03, 
0x06,  0x14,  0x19,  0x09,  0x00,  0x16,  0x16,  0x01,  0x01,  0x03,  0x17,  0x07,  0x06,  0x06,  0x03,  0x02, 
0x03,  0x00,  0x01,  0x18,  0x00,  0x03,  0x0a,  0x15,  0x07,  0x03,  0x02,  0x09,  0x18,  0x05,  0x15,  0x15, 
0x03,  0x03,  0x0d,  0x0f,  0x11,  0x0f,  0x03,  0x03,  0x02,  0x02,  0x03,  0x06,  0x03,  0x02,  0x03,  0x07, 
0x03,  0x03,  0x06,  0x07,  0x07,  0x06,  0x07,  0x06,  0x07,  0x06,  0x07,  0x07,  0x06,  0x07,  0x07,  0x07, 
0x17,  0x03,  0x00,  0x06,  0x00,  0x18,  0x04,  0x00,  0x03,  0x05,  0x18,  0x07,  0x07,  0x06,  0x06,  0x03, 
0x02,  0x09,  0x17,  0x06,  0x06,  0x18,  0x15,  0x17,  0x00,  0x02,  0x12,  0x02,  0x03,  0x06,  0x06,  0x06, 
0x06,  0x03,  0x03,  0x02,  0x10,  0x02,  0x03,  0x02,  0x03,  0x02,  0x03,  0x03,  0x06,  0x03,  0x06,  0x03, 
0x02,  0x13,  0x03,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x07, 

0x06,  0x07,  0x18,  0x18,  0x0a,  0x0a,  0x18,  0x00,  0x17,  0x07,  0x06,  0x07,  0x07,  0x07,  0x07,  0x06, 
0x03,  0x07,  0x00,  0x0a,  0x17,  0x17,  0x05,  0x05,  0x17,  0x03,  0x02,  0x03,  0x06,  0x07,  0x06,  0x15, 
0x09,  0x03,  0x02,  0x03,  0x0f,  0x03,  0x03,  0x02,  0x02,  0x06,  0x06,  0x06,  0x03,  0x07,  0x06,  0x03, 
0x03,  0x03,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x07, 
0x07,  0x06,  0x0a,  0x0a,  0x0a,  0x06,  0x07,  0x18,  0x07,  0x07,  0x06,  0x06,  0x07,  0x06,  0x07,  0x06, 
0x07,  0x15,  0x17,  0x18,  0x00,  0x09,  0x09,  0x18,  0x05,  0x03,  0x03,  0x02,  0x03,  0x06,  0x03,  0x02, 
0x03,  0x02,  0x02,  0x02,  0x0f,  0x02,  0x06,  0x03,  0x02,  0x03,  0x06,  0x03,  0x06,  0x06,  0x06,  0x06, 
0x06,  0x06,  0x06,  0x03,  0x07,  0x07,  0x07,  0x06,  0x06,  0x07,  0x07,  0x06,  0x06,  0x06,  0x07,  0x07, 
0x18,  0x05,  0x00,  0x00,  0x15,  0x17,  0x07,  0x17,  0x07,  0x0a,  0x07,  0x07,  0x07,  0x06,  0x06,  0x07, 
0x17,  0x0a,  0x00,  0x15,  0x06,  0x18,  0x09,  0x17,  0x06,  0x06,  0x03,  0x03,  0x02,  0x06,  0x03,  0x02, 
0x02,  0x03,  0x12,  0x02,  0x02,  0x03,  0x03,  0x03,  0x02,  0x06,  0x06,  0x06,  0x03,  0x12,  0x03,  0x06, 
0x06,  0x06,  0x03,  0x03,  0x03,  0x06,  0x06,  0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x07, 
0x0a,  0x00,  0x02,  0x00,  0x06,  0x17,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x02,  0x06,  0x07,  0x15, 
0x00,  0x06,  0x00,  0x0a,  0x06,  0x0a,  0x0a,  0x0a,  0x06,  0x03,  0x06,  0x02,  0x12,  0x02,  0x06,  0x02, 
0x02,  0x02,  0x12,  0x12,  0x02,  0x02,  0x03,  0x02,  0x03,  0x03,  0x07,  0x06,  0x12,  0x0f,  0x12,  0x03, 
0x06,  0x03,  0x06,  0x03,  0x06,  0x02,  0x03,  0x03,  0x06,  0x06,  0x06,  0x06,  0x07,  0x06,  0x07,  0x06, 

0x01,  0x08,  0x08,  0x16,  0x00,  0x17,  0x0a,  0x0a,  0x18,  0x07,  0x07,  0x08,  0x13,  0x02,  0x07,  0x00, 
0x18,  0x06,  0x17,  0x18,  0x0a,  0x15,  0x06,  0x07,  0x06,  0x06,  0x03,  0x03,  0x02,  0x06,  0x03,  0x03, 
0x02,  0x02,  0x0d,  0x12,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x06,  0x06,  0x03,  0x12,  0x03,  0x06, 
0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x06,  0x06,  0x07,  0x07,  0x03,  0x03,  0x07,  0x07,  0x00, 
0x14,  0x04,  0x16,  0x08,  0x03,  0x00,  0x15,  0x0a,  0x18,  0x18,  0x07,  0x07,  0x02,  0x07,  0x07,  0x0a, 
0x07,  0x07,  0x0a,  0x0a,  0x07,  0x07,  0x07,  0x07,  0x07,  0x03,  0x03,  0x03,  0x06,  0x03,  0x03,  0x02, 
0x02,  0x12,  0x0d,  0x0d,  0x02,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06,  0x03,  0x03,  0x06,  0x06,  0x03, 
0x03,  0x02,  0x03,  0x12,  0x02,  0x02,  0x03,  0x03,  0x06,  0x06,  0x03,  0x02,  0x03,  0x03,  0x03,  0x08, 
0x1a,  0x14,  0x02,  0x08,  0x18,  0x15,  0x17,  0x17,  0x00,  0x06,  0x06,  0x07,  0x07,  0x07,  0x06,  0x07, 
0x07,  0x18,  0x17,  0x0a,  0x17,  0x07,  0x07,  0x07,  0x06,  0x06,  0x03,  0x06,  0x03,  0x06,  0x03,  0x03, 
0x02,  0x02,  0x12,  0x0d,  0x12,  0x02,  0x03,  0x06,  0x03,  0x03,  0x03,  0x03,  0x02,  0x06,  0x06,  0x03, 
0x02,  0x02,  0x02,  0x12,  0x12,  0x02,  0x02,  0x03,  0x03,  0x03,  0x06,  0x03,  0x12,  0x02,  0x12,  0x1b, 
0x02,  0x12,  0x16,  0x12,  0x03,  0x06,  0x07,  0x06,  0x06,  0x06,  0x07,  0x06,  0x07,  0x03,  0x06,  0x06, 
0x07,  0x07,  0x06,  0x17,  0x07,  0x07,  0x07,  0x06,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03,  0x02,  0x03, 
0x02,  0x03,  0x02,  0x12,  0x02,  0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x06,  0x06,  0x03,  0x03,  0x12, 
0x02,  0x02,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x02,  0x02,  0x06,  0x03,  0x02,  0x12,  0x12,  0x12, 

0x02,  0x14,  0x08,  0x09,  0x0a,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x07,  0x06,  0x06,  0x03,  0x06, 
0x03,  0x06,  0x0a,  0x17,  0x07,  0x07,  0x06,  0x06,  0x03,  0x07,  0x06,  0x06,  0x03,  0x07,  0x03,  0x06, 
0x03,  0x06,  0x03,  0x02,  0x02,  0x06,  0x03,  0x03,  0x12,  0x02,  0x03,  0x07,  0x06,  0x06,  0x02,  0x02, 
0x03,  0x03,  0x03,  0x03,  0x03,  0x06,  0x02,  0x03,  0x02,  0x03,  0x03,  0x03,  0x12,  0x0d,  0x13,  0x02, 
0x04,  0x0b,  0x16,  0x05,  0x0a,  0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x03,  0x06,  0x06, 
0x03,  0x03,  0x06,  0x06,  0x07,  0x07,  0x06,  0x06,  0x06,  0x06,  0x07,  0x03,  0x02,  0x03,  0x06,  0x06, 
0x07,  0x03,  0x03,  0x03,  0x03,  0x03,  0x02,  0x02,  0x03,  0x02,  0x02,  0x03,  0x06,  0x06,  0x06,  0x06, 
0x07,  0x03,  0x03,  0x03,  0x06,  0x06,  0x06,  0x03,  0x03,  0x03,  0x12,  0x12,  0x12,  0x0d,  0x0d,  0x12, 
0x08,  0x02,  0x1a,  0x09,  0x06,  0x07,  0x06,  0x06,  0x06,  0x07,  0x06,  0x06,  0x02,  0x03,  0x02,  0x06, 
0x02,  0x06,  0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x06,  0x07, 
0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x02,  0x03,  0x0d,  0x03,  0x02,  0x06,  0x06,  0x07, 
0x07,  0x06,  0x06,  0x06,  0x06,  0x07,  0x06,  0x03,  0x03,  0x12,  0x0e,  0x12,  0x12,  0x12,  0x12,  0x19, 
0x16,  0x16,  0x14,  0x01,  0x06,  0x06,  0x07,  0x06,  0x07,  0x06,  0x03,  0x02,  0x03,  0x02,  0x12,  0x02, 
0x03,  0x03,  0x06,  0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x06,  0x03,  0x06,  0x06, 
0x07,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x02,  0x03,  0x02,  0x02,  0x13,  0x03,  0x03, 
0x07,  0x06,  0x07,  0x03,  0x07,  0x06,  0x06,  0x02,  0x03,  0x02,  0x12,  0x12,  0x02,  0x12,  0x12,  0x0d, 

0x1a,  0x04,  0x03,  0x06,  0x06,  0x06,  0x07,  0x07,  0x06,  0x03,  0x02,  0x02,  0x02,  0x02,  0x12,  0x03, 
0x06,  0x06,  0x06,  0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x06,  0x06,  0x07,  0x03,  0x03,  0x02,  0x06, 
0x06,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x13,  0x0d,  0x13,  0x03, 
0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x02,  0x02,  0x12,  0x12,  0x0d,  0x12,  0x02,  0x0d,  0x12, 
0x14,  0x04,  0x06,  0x06,  0x06,  0x03,  0x06,  0x06,  0x06,  0x02,  0x02,  0x12,  0x02,  0x0d,  0x02,  0x03, 
0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x06,  0x03,  0x03,  0x03,  0x03,  0x02,  0x02,  0x02, 
0x06,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x06,  0x03,  0x06,  0x07,  0x06,  0x03,  0x13,  0x06,  0x03, 
0x07,  0x06,  0x06,  0x06,  0x07,  0x06,  0x03,  0x02,  0x03,  0x02,  0x12,  0x0d,  0x0d,  0x02,  0x02,  0x12, 
0x03,  0x09,  0x02,  0x16,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x12,  0x02,  0x12,  0x0e,  0x12,  0x03, 
0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x06,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06, 
0x03,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x06,  0x07,  0x03,  0x03,  0x06,  0x06, 
0x03,  0x07,  0x07,  0x07,  0x06,  0x06,  0x02,  0x03,  0x02,  0x02,  0x12,  0x12,  0x0d,  0x02,  0x12,  0x02, 
0x02,  0x12,  0x02,  0x02,  0x02,  0x02,  0x02,  0x02,  0x03,  0x02,  0x0e,  0x12,  0x03,  0x12,  0x02,  0x06, 
0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x06,  0x03,  0x06,  0x06,  0x07,  0x06, 
0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x03,  0x06,  0x03,  0x03,  0x06,  0x06,  0x03,  0x06,  0x03, 
0x02,  0x03,  0x07,  0x06,  0x07,  0x06,  0x06,  0x02,  0x03,  0x02,  0x12,  0x0d,  0x12,  0x12,  0x02,  0x12, 

0x12,  0x12,  0x02,  0x03,  0x12,  0x02,  0x0f,  0x02,  0x03,  0x02,  0x0d,  0x02,  0x03,  0x06,  0x03,  0x07, 
0x06,  0x07,  0x07,  0x07,  0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x06,  0x07,  0x07,  0x07, 
0x07,  0x07,  0x07,  0x07,  0x07,  0x07,  0x03,  0x03,  0x03,  0x03,  0x03,  0x06,  0x06,  0x02,  0x02,  0x06, 
0x03,  0x02,  0x02,  0x03,  0x03,  0x06,  0x0e,  0x02,  0x02,  0x02,  0x0d,  0x12,  0x12,  0x02,  0x02,  0x02, 
0x02,  0x12,  0x02,  0x12,  0x12,  0x0f,  0x11,  0x0f,  0x03,  0x03,  0x03,  0x02,  0x06,  0x06,  0x06,  0x03, 
0x07,  0x07,  0x07,  0x03,  0x06,  0x07,  0x07,  0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x06,  0x06, 
0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x02,  0x12,  0x02,  0x02, 
0x02,  0x0e,  0x12,  0x03,  0x06,  0x06,  0x0f,  0x12,  0x12,  0x12,  0x02,  0x02,  0x12,  0x12,  0x02,  0x12, 
0x12,  0x12,  0x0d,  0x12,  0x0d,  0x0d,  0x0f,  0x12,  0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03, 
0x06,  0x07,  0x07,  0x07,  0x07,  0x07,  0x06,  0x07,  0x06,  0x03,  0x06,  0x07,  0x07,  0x07,  0x06,  0x06, 
0x06,  0x07,  0x07,  0x07,  0x06,  0x06,  0x03,  0x03,  0x12,  0x06,  0x03,  0x02,  0x03,  0x02,  0x02,  0x02, 
0x02,  0x02,  0x02,  0x06,  0x02,  0x12,  0x11,  0x0d,  0x0d,  0x0d,  0x02,  0x03,  0x12,  0x12,  0x12,  0x02, 
0x12,  0x1b,  0x0d,  0x0d,  0x12,  0x12,  0x02,  0x02,  0x02,  0x02,  0x03,  0x02,  0x03,  0x03,  0x03,  0x06, 
0x07,  0x03,  0x07,  0x07,  0x07,  0x03,  0x06,  0x06,  0x06,  0x02,  0x06,  0x06,  0x07,  0x07,  0x07,  0x07, 
0x07,  0x06,  0x07,  0x07,  0x06,  0x02,  0x03,  0x03,  0x03,  0x06,  0x03,  0x02,  0x03,  0x03,  0x02,  0x12, 
0x02,  0x02,  0x12,  0x0e,  0x0f,  0x11,  0x11,  0x11,  0x0f,  0x0e,  0x03,  0x02,  0x02,  0x0d,  0x12,  0x02, 

0x02,  0x12,  0x1b,  0x12,  0x04,  0x02,  0x02,  0x02,  0x0d,  0x12,  0x02,  0x03,  0x03,  0x03,  0x06,  0x07, 
0x07,  0x07,  0x07,  0x07,  0x06,  0x06,  0x03,  0x03,  0x02,  0x12,  0x02,  0x03,  0x07,  0x07,  0x07,  0x07, 
0x07,  0x07,  0x07,  0x07,  0x06,  0x03,  0x02,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x02,  0x02, 
0x12,  0x0d,  0x0d,  0x12,  0x0e,  0x0f,  0x11,  0x0e,  0x02,  0x03,  0x06,  0x03,  0x12,  0x0d,  0x02,  0x03, 
0x02,  0x12,  0x1a,  0x19,  0x02,  0x02,  0x02,  0x0d,  0x12,  0x12,  0x02,  0x03,  0x03,  0x03,  0x07,  0x06, 
0x06,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x06,  0x06,  0x02,  0x06,  0x07,  0x07,  0x07,  0x07,  0x06, 
0x07,  0x07,  0x07,  0x06,  0x06,  0x03,  0x03,  0x02,  0x02,  0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x12, 
0x12,  0x0d,  0x0d,  0x12,  0x12,  0x12,  0x0f,  0x12,  0x02,  0x02,  0x06,  0x03,  0x03,  0x12,  0x02,  0x02, 
0x12,  0x12,  0x12,  0x02,  0x12,  0x02,  0x12,  0x12,  0x0d,  0x12,  0x02,  0x03,  0x03,  0x06,  0x06,  0x06, 
0x03,  0x07,  0x06,  0x07,  0x07,  0x06,  0x03,  0x06,  0x07,  0x03,  0x07,  0x07,  0x07,  0x07,  0x07,  0x07, 
0x06,  0x07,  0x07,  0x06,  0x03,  0x03,  0x02,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x02,  0x12, 
0x0d,  0x0d,  0x0d,  0x02,  0x02,  0x03,  0x0e,  0x02,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x12,  0x1c, 
0x1c,  0x12,  0x12,  0x12,  0x02,  0x12,  0x02,  0x12,  0x12,  0x0d,  0x12,  0x02,  0x03,  0x03,  0x06,  0x03, 
0x03,  0x06,  0x07,  0x07,  0x07,  0x06,  0x06,  0x07,  0x07,  0x06,  0x06,  0x06,  0x07,  0x06,  0x07,  0x06, 
0x06,  0x06,  0x07,  0x03,  0x03,  0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x12,  0x12, 
0x0d,  0x0d,  0x0e,  0x12,  0x12,  0x02,  0x06,  0x03,  0x07,  0x06,  0x02,  0x02,  0x03,  0x02,  0x1c,  0x11, 

0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x02,  0x02,  0x03,  0x02,  0x03, 
0x02,  0x03,  0x07,  0x07,  0x06,  0x06,  0x06,  0x07,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x03,  0x06, 
0x06,  0x06,  0x06,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x03,  0x03,  0x12,  0x02, 
0x12,  0x0e,  0x0f,  0x0e,  0x12,  0x03,  0x03,  0x03,  0x06,  0x07,  0x03,  0x03,  0x02,  0x02,  0x02,  0x1c, 
0x0d,  0x12,  0x02,  0x12,  0x12,  0x12,  0x02,  0x12,  0x12,  0x12,  0x12,  0x12,  0x0d,  0x0d,  0x02,  0x03, 
0x03,  0x06,  0x07,  0x07,  0x07,  0x06,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03,  0x02,  0x02,  0x02,  0x03, 
0x06,  0x06,  0x06,  0x02,  0x02,  0x02,  0x06,  0x03,  0x02,  0x02,  0x03,  0x03,  0x06,  0x03,  0x02,  0x02, 
0x02,  0x0d,  0x0e,  0x12,  0x02,  0x02,  0x03,  0x03,  0x03,  0x12,  0x12,  0x0d,  0x02,  0x02,  0x02,  0x0d, 
0x12,  0x03,  0x12,  0x12,  0x0d,  0x12,  0x12,  0x0d,  0x12,  0x0d,  0x0d,  0x12,  0x0e,  0x0d,  0x02,  0x03, 
0x03,  0x06,  0x06,  0x02,  0x06,  0x06,  0x06,  0x06,  0x06,  0x06,  0x03,  0x03,  0x02,  0x03,  0x02,  0x06, 
0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x12,  0x12,  0x02,  0x03,  0x03,  0x03,  0x12,  0x02, 
0x02,  0x02,  0x12,  0x02,  0x12,  0x03,  0x02,  0x06,  0x03,  0x03,  0x12,  0x12,  0x12,  0x02,  0x02,  0x12, 
0x03,  0x03,  0x02,  0x12,  0x0d,  0x0d,  0x12,  0x0e,  0x0d,  0x0d,  0x12,  0x12,  0x12,  0x02,  0x03,  0x03, 
0x03,  0x06,  0x02,  0x12,  0x02,  0x06,  0x07,  0x06,  0x03,  0x03,  0x03,  0x02,  0x02,  0x02,  0x03,  0x03, 
0x03,  0x03,  0x03,  0x03,  0x06,  0x03,  0x06,  0x02,  0x02,  0x02,  0x03,  0x02,  0x03,  0x02,  0x02,  0x12, 
0x02,  0x12,  0x12,  0x02,  0x03,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x12,  0x12,  0x12,  0x02,  0x12, 

0x03,  0x06,  0x0d,  0x12,  0x0d,  0x0e,  0x0d,  0x0f,  0x0d,  0x0d,  0x0d,  0x12,  0x12,  0x03,  0x03,  0x06, 
0x03,  0x03,  0x03,  0x02,  0x03,  0x07,  0x07,  0x07,  0x02,  0x03,  0x02,  0x02,  0x02,  0x03,  0x02,  0x03, 
0x03,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x02,  0x02, 
0x02,  0x03,  0x02,  0x02,  0x02,  0x02,  0x03,  0x03,  0x02,  0x03,  0x02,  0x12,  0x12,  0x02,  0x12,  0x12, 
0x02,  0x02,  0x0d,  0x0d,  0x0d,  0x0f,  0x0d,  0x0f,  0x0d,  0x12,  0x0d,  0x12,  0x02,  0x02,  0x03,  0x02, 
0x02,  0x02,  0x06,  0x06,  0x06,  0x07,  0x06,  0x02,  0x12,  0x02,  0x03,  0x02,  0x03,  0x03,  0x06,  0x06, 
0x06,  0x03,  0x03,  0x03,  0x03,  0x03,  0x06,  0x02,  0x02,  0x12,  0x12,  0x12,  0x02,  0x02,  0x02,  0x02, 
0x03,  0x03,  0x03,  0x12,  0x0d,  0x12,  0x12,  0x02,  0x02,  0x03,  0x03,  0x02,  0x02,  0x12,  0x0d,  0x0e, 
0x12,  0x02,  0x0d,  0x0e,  0x0f,  0x11,  0x0f,  0x10,  0x0e,  0x0d,  0x0e,  0x0d,  0x12,  0x02,  0x12,  0x02, 
0x02,  0x03,  0x06,  0x07,  0x06,  0x06,  0x02,  0x11,  0x02,  0x03,  0x02,  0x02,  0x03,  0x03,  0x03,  0x06, 
0x06,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x02,  0x12,  0x12,  0x12,  0x02,  0x12,  0x02,  0x02,  0x03, 
0x03,  0x06,  0x03,  0x02,  0x12,  0x02,  0x0d,  0x12,  0x12,  0x03,  0x03,  0x0d,  0x12,  0x12,  0x12,  0x0d, 
0x0d,  0x12,  0x0d,  0x0d,  0x0d,  0x0f,  0x0f,  0x11,  0x0f,  0x0e,  0x0d,  0x0d,  0x0d,  0x12,  0x12,  0x02, 
0x03,  0x02,  0x06,  0x06,  0x06,  0x03,  0x12,  0x02,  0x03,  0x02,  0x12,  0x12,  0x03,  0x02,  0x06,  0x06, 
0x06,  0x03,  0x02,  0x02,  0x03,  0x02,  0x03,  0x02,  0x02,  0x12,  0x03,  0x02,  0x02,  0x02,  0x03,  0x03, 
0x03,  0x03,  0x03,  0x12,  0x02,  0x12,  0x12,  0x03,  0x06,  0x03,  0x12,  0x0e,  0x0d,  0x12,  0x02,  0x12, 

0x0d,  0x0d,  0x0d,  0x0d,  0x0e,  0x0f,  0x10,  0x11,  0x10,  0x0f,  0x0e,  0x0d,  0x0d,  0x12,  0x12,  0x03, 
0x03,  0x03,  0x03,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x02,  0x03,  0x02,  0x11,  0x02,  0x06, 
0x06,  0x06,  0x03,  0x03,  0x02,  0x03,  0x02,  0x02,  0x0d,  0x02,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03, 
0x02,  0x03,  0x12,  0x12,  0x12,  0x02,  0x02,  0x06,  0x03,  0x03,  0x12,  0x0d,  0x0d,  0x02,  0x12,  0x02, 
0x0d,  0x0e,  0x0d,  0x0e,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x0e,  0x0d,  0x0d,  0x12,  0x12, 
0x02,  0x02,  0x03,  0x02,  0x06,  0x03,  0x03,  0x02,  0x12,  0x12,  0x03,  0x03,  0x03,  0x12,  0x06,  0x03, 
0x06,  0x06,  0x06,  0x03,  0x03,  0x03,  0x06,  0x03,  0x12,  0x02,  0x03,  0x03,  0x13,  0x03,  0x06,  0x03, 
0x02,  0x0d,  0x0d,  0x12,  0x12,  0x12,  0x03,  0x03,  0x03,  0x02,  0x03,  0x02,  0x12,  0x02,  0x02,  0x12, 
0x0e,  0x0f,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x11,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x0f,  0x0e,  0x0d, 
0x12,  0x03,  0x02,  0x02,  0x03,  0x03,  0x03,  0x06,  0x02,  0x03,  0x02,  0x06,  0x03,  0x03,  0x02,  0x12, 
0x02,  0x06,  0x03,  0x03,  0x03,  0x06,  0x06,  0x06,  0x02,  0x03,  0x02,  0x06,  0x1c,  0x07,  0x06,  0x03, 
0x02,  0x12,  0x0d,  0x12,  0x12,  0x03,  0x03,  0x06,  0x02,  0x03,  0x02,  0x02,  0x12,  0x02,  0x12,  0x0d, 
0x0d,  0x0d,  0x0d,  0x0e,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x0e,  0x0d,  0x0d,  0x12,  0x0d, 
0x12,  0x12,  0x02,  0x12,  0x03,  0x02,  0x02,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06,  0x03,  0x03,  0x02, 
0x02,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06,  0x03,  0x06,  0x03,  0x03,  0x03,  0x1c,  0x06,  0x03,  0x02, 
0x12,  0x12,  0x12,  0x0d,  0x12,  0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x12,  0x0e,  0x12,  0x02,  0x12, 

0x12,  0x0d,  0x0d,  0x0d,  0x0d,  0x0f,  0x10,  0x11,  0x10,  0x0f,  0x0e,  0x0d,  0x0e,  0x0d,  0x0d,  0x12, 
0x0d,  0x12,  0x12,  0x02,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x03,  0x06,  0x06,  0x06,  0x03,  0x03, 
0x03,  0x03,  0x02,  0x02,  0x02,  0x03,  0x03,  0x03,  0x03,  0x06,  0x02,  0x02,  0x0f,  0x03,  0x02,  0x02, 
0x12,  0x12,  0x0d,  0x12,  0x12,  0x02,  0x03,  0x06,  0x03,  0x03,  0x03,  0x02,  0x0d,  0x03,  0x12,  0x02, 
0x02,  0x12,  0x0e,  0x0d,  0x0d,  0x0d,  0x0f,  0x11,  0x0f,  0x0e,  0x0e,  0x0e,  0x0f,  0x0e,  0x0d,  0x0d, 
0x0d,  0x0d,  0x0d,  0x0d,  0x02,  0x12,  0x12,  0x12,  0x02,  0x03,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03, 
0x06,  0x02,  0x02,  0x12,  0x02,  0x12,  0x12,  0x12,  0x03,  0x02,  0x12,  0x1c,  0x10,  0x1c,  0x03,  0x02, 
0x12,  0x12,  0x02,  0x12,  0x02,  0x03,  0x06,  0x03,  0x03,  0x03,  0x06,  0x03,  0x12,  0x12,  0x12,  0x12, 
0x12,  0x12,  0x0e,  0x0e,  0x0e,  0x0e,  0x0f,  0x10,  0x0f,  0x0d,  0x0e,  0x0f,  0x10,  0x0f,  0x0e,  0x12, 
0x12,  0x0d,  0x0e,  0x0d,  0x0e,  0x0f,  0x12,  0x02,  0x02,  0x03,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03, 
0x03,  0x02,  0x12,  0x02,  0x02,  0x03,  0x02,  0x02,  0x03,  0x02,  0x1c,  0x0f,  0x11,  0x0f,  0x1c,  0x03, 
0x02,  0x02,  0x06,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x06,  0x03,  0x06,  0x02,  0x02,  0x12,  0x02, 
0x02,  0x12,  0x0e,  0x0f,  0x0e,  0x0f,  0x0e,  0x0f,  0x0e,  0x0d,  0x0d,  0x0e,  0x0f,  0x0e,  0x0d,  0x12, 
0x02,  0x0e,  0x0d,  0x0e,  0x0f,  0x10,  0x0f,  0x02,  0x06,  0x03,  0x06,  0x12,  0x03,  0x06,  0x06,  0x03, 
0x03,  0x02,  0x02,  0x12,  0x02,  0x02,  0x13,  0x1c,  0x1c,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x10,  0x0f, 
0x1c,  0x1c,  0x13,  0x03,  0x06,  0x03,  0x06,  0x06,  0x12,  0x02,  0x06,  0x03,  0x06,  0x03,  0x02,  0x12, 

0x0d,  0x0d,  0x0f,  0x11,  0x0f,  0x0e,  0x0d,  0x0f,  0x0d,  0x0d,  0x0e,  0x0e,  0x0e,  0x0d,  0x0d,  0x12, 
0x0d,  0x12,  0x12,  0x0d,  0x0d,  0x0f,  0x02,  0x02,  0x03,  0x06,  0x03,  0x03,  0x03,  0x06,  0x06,  0x03, 
0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x02,  0x02,  0x12,  0x02,  0x1c,  0x0f,  0x11,  0x0f,  0x1c,  0x02, 
0x03,  0x03,  0x02,  0x03,  0x03,  0x06,  0x06,  0x03,  0x12,  0x03,  0x03,  0x06,  0x03,  0x03,  0x12,  0x12, 
0x12,  0x0d,  0x0e,  0x0f,  0x0e,  0x0d,  0x0d,  0x0e,  0x0d,  0x0e,  0x0d,  0x0d,  0x0d,  0x0d,  0x12,  0x0d, 
0x0d,  0x0d,  0x02,  0x02,  0x02,  0x02,  0x03,  0x02,  0x06,  0x06,  0x06,  0x06,  0x06,  0x03,  0x03,  0x12, 
0x0d,  0x12,  0x0d,  0x12,  0x02,  0x02,  0x03,  0x12,  0x02,  0x12,  0x02,  0x1c,  0x10,  0x1c,  0x12,  0x02, 
0x06,  0x03,  0x03,  0x02,  0x06,  0x06,  0x06,  0x12,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x02,  0x12, 
0x0d,  0x0d,  0x0d,  0x0e,  0x0e,  0x0d,  0x0d,  0x0d,  0x0e,  0x0f,  0x0e,  0x0d,  0x0e,  0x0d,  0x12,  0x12, 
0x13,  0x12,  0x03,  0x03,  0x02,  0x03,  0x02,  0x02,  0x03,  0x06,  0x06,  0x07,  0x03,  0x02,  0x12,  0x12, 
0x12,  0x0e,  0x0e,  0x0d,  0x12,  0x02,  0x02,  0x12,  0x12,  0x03,  0x02,  0x03,  0x0f,  0x02,  0x12,  0x02, 
0x02,  0x03,  0x03,  0x03,  0x03,  0x07,  0x06,  0x06,  0x06,  0x06,  0x03,  0x03,  0x02,  0x02,  0x12,  0x0d, 
0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0e,  0x0f,  0x10,  0x0f,  0x0e,  0x0e,  0x0f,  0x12,  0x0d, 
0x12,  0x02,  0x03,  0x02,  0x02,  0x03,  0x03,  0x03,  0x03,  0x02,  0x02,  0x06,  0x03,  0x0d,  0x12,  0x02, 
0x06,  0x02,  0x12,  0x0d,  0x02,  0x03,  0x03,  0x12,  0x13,  0x02,  0x03,  0x02,  0x1c,  0x02,  0x02,  0x12, 
0x03,  0x02,  0x02,  0x03,  0x06,  0x06,  0x07,  0x06,  0x06,  0x06,  0x03,  0x02,  0x12,  0x12,  0x12,  0x1d, 

0x0e,  0x0e,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0e,  0x0f,  0x0f,  0x0e,  0x0e,  0x0e,  0x0d,  0x12, 
0x0d,  0x02,  0x02,  0x03,  0x02,  0x03,  0x02,  0x03,  0x02,  0x0d,  0x12,  0x02,  0x02,  0x02,  0x12,  0x03, 
0x06,  0x06,  0x02,  0x02,  0x03,  0x06,  0x03,  0x02,  0x02,  0x03,  0x02,  0x02,  0x1c,  0x03,  0x02,  0x02, 
0x02,  0x03,  0x03,  0x06,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x06,  0x07,  0x16,  0x19,  0x1b,  0x1e, 
0x1d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0e,  0x0e,  0x0f,  0x0d,  0x0d,  0x12,  0x12, 
0x02,  0x02,  0x03,  0x03,  0x02,  0x12,  0x0e,  0x02,  0x02,  0x0f,  0x0d,  0x0d,  0x02,  0x02,  0x0d,  0x12, 
0x06,  0x03,  0x06,  0x03,  0x06,  0x03,  0x06,  0x03,  0x03,  0x12,  0x02,  0x12,  0x13,  0x02,  0x02,  0x0e, 
0x12,  0x03,  0x06,  0x03,  0x06,  0x03,  0x06,  0x06,  0x07,  0x07,  0x07,  0x07,  0x09,  0x19,  0x1b,  0x0c, 
0x1e,  0x0d,  0x0e,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0e,  0x0d,  0x02,  0x12,  0x02, 
0x02,  0x02,  0x02,  0x03,  0x12,  0x12,  0x0e,  0x12,  0x0f,  0x11,  0x0f,  0x12,  0x02,  0x03,  0x0d,  0x12, 
0x02,  0x12,  0x03,  0x03,  0x03,  0x06,  0x03,  0x03,  0x02,  0x0d,  0x12,  0x12,  0x02,  0x02,  0x02,  0x12, 
0x12,  0x02,  0x02,  0x03,  0x02,  0x03,  0x03,  0x07,  0x07,  0x07,  0x07,  0x07,  0x17,  0x19,  0x0b,  0x1d, 
0x0d,  0x0e,  0x0f,  0x0e,  0x0d,  0x0e,  0x0d,  0x0e,  0x12,  0x0d,  0x12,  0x12,  0x0e,  0x12,  0x02,  0x12, 
0x02,  0x12,  0x12,  0x02,  0x12,  0x12,  0x02,  0x02,  0x0d,  0x0f,  0x0d,  0x12,  0x02,  0x02,  0x02,  0x12, 
0x02,  0x03,  0x03,  0x02,  0x03,  0x03,  0x03,  0x02,  0x12,  0x0e,  0x0d,  0x12,  0x02,  0x02,  0x02,  0x12, 
0x12,  0x12,  0x02,  0x12,  0x02,  0x03,  0x06,  0x06,  0x07,  0x06,  0x07,  0x07,  0x07,  0x06,  0x12,  0x1b, 

0x0d,  0x0d,  0x0e,  0x0d,  0x0d,  0x0d,  0x0e,  0x0e,  0x0e,  0x12,  0x0d,  0x0d,  0x11,  0x0e,  0x12,  0x02, 
0x12,  0x02,  0x02,  0x03,  0x12,  0x02,  0x02,  0x03,  0x02,  0x12,  0x0d,  0x12,  0x12,  0x02,  0x02,  0x02, 
0x02,  0x03,  0x02,  0x02,  0x02,  0x02,  0x12,  0x0d,  0x0e,  0x0d,  0x12,  0x02,  0x02,  0x02,  0x12,  0x0d, 
0x0d,  0x02,  0x02,  0x12,  0x12,  0x12,  0x03,  0x06,  0x06,  0x1f,  0x06,  0x07,  0x06,  0x07,  0x03,  0x0b, 
0x0d,  0x0d,  0x0e,  0x0d,  0x12,  0x12,  0x12,  0x0d,  0x0d,  0x0d,  0x12,  0x12,  0x0e,  0x02,  0x03,  0x02, 
0x02,  0x02,  0x03,  0x02,  0x02,  0x12,  0x0d,  0x02,  0x03,  0x12,  0x0d,  0x0e,  0x12,  0x02,  0x03,  0x02, 
0x02,  0x12,  0x12,  0x12,  0x12,  0x0d,  0x12,  0x12,  0x12,  0x0d,  0x12,  0x12,  0x02,  0x12,  0x0d,  0x0e, 
0x0e,  0x12,  0x03,  0x02,  0x03,  0x02,  0x03,  0x03,  0x02,  0x12,  0x06,  0x06,  0x07,  0x1f,  0x06,  0x0b, 
0x1d,  0x0d,  0x0d,  0x12,  0x12,  0x12,  0x12,  0x12,  0x12,  0x02,  0x12,  0x02,  0x03,  0x03,  0x02,  0x03, 
0x12,  0x02,  0x02,  0x03,  0x12,  0x0e,  0x12,  0x02,  0x03,  0x02,  0x12,  0x02,  0x02,  0x02,  0x02,  0x02, 
0x12,  0x0d,  0x0d,  0x02,  0x12,  0x12,  0x12,  0x12,  0x02,  0x12,  0x12,  0x12,  0x12,  0x02,  0x12,  0x0d, 
0x0d,  0x02,  0x03,  0x06,  0x03,  0x06,  0x03,  0x12,  0x12,  0x1f,  0x06,  0x07,  0x1f,  0x13,  0x1f,  0x14, 
0x1a,  0x0d,  0x12,  0x0d,  0x02,  0x12,  0x12,  0x12,  0x02,  0x02,  0x02,  0x02,  0x03,  0x03,  0x02,  0x12, 
0x02,  0x12,  0x03,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x02,  0x02,  0x12,  0x02,  0x02,  0x12,  0x12, 
0x0d,  0x0d,  0x12,  0x12,  0x02,  0x12,  0x0d,  0x0d,  0x12,  0x0d,  0x02,  0x12,  0x12,  0x02,  0x02,  0x12, 
0x12,  0x02,  0x03,  0x03,  0x03,  0x03,  0x02,  0x13,  0x06,  0x06,  0x07,  0x07,  0x07,  0x1f,  0x12,  0x02, 

0x1a,  0x12,  0x12,  0x12,  0x12,  0x02,  0x12,  0x02,  0x02,  0x03,  0x03,  0x03,  0x06,  0x06,  0x03,  0x02, 
0x03,  0x03,  0x06,  0x03,  0x02,  0x02,  0x02,  0x03,  0x02,  0x02,  0x12,  0x02,  0x12,  0x12,  0x0d,  0x0d, 
0x0d,  0x0e,  0x0d,  0x12,  0x0e,  0x0d,  0x0e,  0x0e,  0x0d,  0x12,  0x02,  0x02,  0x02,  0x03,  0x02,  0x02, 
0x12,  0x03,  0x02,  0x02,  0x02,  0x03,  0x02,  0x03,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x03,  0x0b, 
0x12,  0x12,  0x02,  0x12,  0x0d,  0x12,  0x02,  0x02,  0x03,  0x02,  0x03,  0x06,  0x06,  0x06,  0x06,  0x03, 
0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x03,  0x03,  0x02,  0x12,  0x0d,  0x0d,  0x12,  0x0d,  0x12,  0x0d, 
0x0e,  0x0e,  0x0d,  0x0d,  0x0f,  0x0d,  0x0d,  0x0e,  0x0d,  0x12,  0x02,  0x12,  0x02,  0x02,  0x03,  0x02, 
0x02,  0x02,  0x02,  0x12,  0x02,  0x02,  0x02,  0x03,  0x06,  0x06,  0x07,  0x06,  0x06,  0x06,  0x06,  0x04, 
0x12,  0x12,  0x19,  0x0d,  0x12,  0x12,  0x02,  0x12,  0x02,  0x03,  0x03,  0x06,  0x03,  0x06,  0x03,  0x03, 
0x03,  0x06,  0x03,  0x06,  0x03,  0x02,  0x02,  0x03,  0x02,  0x0d,  0x0d,  0x0d,  0x0d,  0x12,  0x0d,  0x0e, 
0x0f,  0x0f,  0x0e,  0x0d,  0x0f,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x12,  0x0d,  0x12,  0x12,  0x12,  0x02, 
0x12,  0x02,  0x02,  0x12,  0x12,  0x02,  0x02,  0x03,  0x03,  0x06,  0x06,  0x06,  0x03,  0x03,  0x02,  0x06, 
0x04,  0x12,  0x12,  0x04,  0x02,  0x12,  0x02,  0x02,  0x06,  0x06,  0x06,  0x06,  0x06,  0x06,  0x07,  0x06, 
0x06,  0x03,  0x06,  0x03,  0x06,  0x03,  0x06,  0x02,  0x02,  0x12,  0x0d,  0x0e,  0x0d,  0x0d,  0x0e,  0x0e, 
0x0f,  0x0e,  0x0d,  0x0e,  0x10,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d,  0x12,  0x02,  0x12, 
0x12,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x03,  0x03,  0x1f,  0x06,  0x03,  0x03,  0x02,  0x02,  0x02, 

0x16,  0x04,  0x04,  0x02,  0x02,  0x02,  0x02,  0x03,  0x03,  0x06,  0x06,  0x07,  0x03,  0x12,  0x06,  0x06, 
0x03,  0x06,  0x06,  0x06,  0x03,  0x06,  0x03,  0x03,  0x02,  0x02,  0x0e,  0x0f,  0x0e,  0x0d,  0x0e,  0x0f, 
0x0e,  0x0d,  0x0d,  0x0f,  0x11,  0x0f,  0x0e,  0x0d,  0x0d,  0x12,  0x0d,  0x0d,  0x12,  0x02,  0x02,  0x02, 
0x12,  0x02,  0x02,  0x02,  0x12,  0x02,  0x02,  0x03,  0x12,  0x12,  0x06,  0x03,  0x02,  0x06,  0x02,  0x06, 
0x01,  0x01,  0x12,  0x16,  0x03,  0x02,  0x02,  0x02,  0x03,  0x06,  0x07,  0x07,  0x12,  0x0f,  0x12,  0x03, 
0x06,  0x06,  0x07,  0x03,  0x03,  0x03,  0x03,  0x03,  0x03,  0x02,  0x0d,  0x0e,  0x0d,  0x0e,  0x0d,  0x0e, 
0x0d,  0x0e,  0x0f,  0x10,  0x11,  0x10,  0x0f,  0x0e,  0x0d,  0x0d,  0x0d,  0x12,  0x12,  0x02,  0x06,  0x06, 
0x03,  0x02,  0x03,  0x12,  0x02,  0x12,  0x02,  0x12,  0x03,  0x02,  0x06,  0x03,  0x03,  0x02,  0x02,  0x02, 
0x09,  0x12,  0x12,  0x02,  0x02,  0x02,  0x02,  0x03,  0x03,  0x03,  0x03,  0x12,  0x0f,  0x11,  0x0f,  0x12, 
0x02,  0x03,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06,  0x03,  0x03,  0x12,  0x0d,  0x0d,  0x0d,  0x0d,  0x0d, 
0x0d,  0x0f,  0x10,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x0d,  0x0d,  0x0d,  0x12,  0x02,  0x06,  0x06,  0x07, 
0x06,  0x06,  0x03,  0x03,  0x02,  0x02,  0x12,  0x0e,  0x12,  0x03,  0x03,  0x03,  0x02,  0x03,  0x03,  0x06, 
0x02,  0x19,  0x12,  0x03,  0x02,  0x12,  0x02,  0x02,  0x03,  0x03,  0x06,  0x03,  0x12,  0x0f,  0x12,  0x03, 
0x03,  0x02,  0x12,  0x03,  0x03,  0x03,  0x03,  0x06,  0x06,  0x02,  0x03,  0x12,  0x0d,  0x0e,  0x0f,  0x0f, 
0x10,  0x11,  0x11,  0x11,  0x11,  0x11,  0x11,  0x11,  0x10,  0x0f,  0x0f,  0x0d,  0x0d,  0x12,  0x02,  0x03, 
0x06,  0x06,  0x06,  0x02,  0x02,  0x12,  0x0e,  0x0f,  0x0e,  0x02,  0x03,  0x02,  0x03,  0x03,  0x06,  0x02, 
};
static PROGMEM prog_uchar space_chr[] = {

0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 

0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
};
static PROGMEM prog_uchar space_pal[] = {

0x07,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x09,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x0d,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x09,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x10,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x06,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x04,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x0c,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x08,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x02,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x12,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x37,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x16,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x9b,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0xbb,  0x01,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0xdb,  0x02,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x5c,  0x43,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x12,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x12,  0x10,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x0c,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x05,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x0b,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x01,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x03,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x0e,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x14,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x16,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x9b,  0x34,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x17,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
0x79,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x08,  0x10,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
};

