/*

 Name      :  Cube
 Notes     :  wireframe cube
 
 references:
 http://www.student.kuleuven.be/~m0216922/CG/fire.html 
 
 */

// Define the screen width and height in terms of characters
#define CHAR_WIDTH 50
#define CHAR_HEIGHT 37

// Cube vertex geometry
#define CubeVertexLength 8
static char CubeVertex[8][3] =
{
	{-1, -1, 1},
	{-1, 1, 1},
	{1, 1, 1},
	{1, -1, 1},
	{-1, -1, -1},
	{-1, 1, -1},
	{1, 1, -1},
	{1, -1, -1},
};

// Cube edge cata
#define CubeEdgesLength 12
static char CubeEdges[12][2] =
{
	{0, 1},
	{1, 2},
	{2, 3},
	{3, 0},
	{4, 5},
	{5, 6},
	{6, 7},
	{7, 4},
	{0, 4},
	{1, 5},
	{2, 6},
	{3, 7},
};


class Cube
{
public:
  Cube(){}

void draw()
{


  // Total time
  //  static float TotalTime = 0.0f;
  //    TotalTime += dT;

  /*** Internal Data declaration ***/
  static float CameraRotX = 0.0f;
  static float CameraRotY = 0.0f;
  static float CameraRotZ = 0.0f;

  /*** Prepare Data ***/

  // Slightly grow the rotations
  CameraRotX += .03;
  CameraRotY += .03;
  CameraRotZ = .01;

  /*** Render Scene ***/

  // Find the center of the image
  int CenterX = CHAR_WIDTH / 2;
  int CenterY = (CHAR_HEIGHT / 2);

  // Create an on-screen point list we will be working with
  int PointList[CubeVertexLength][2];

  // For each vertex point
  for(int i = 0; i < CubeVertexLength; i++)
  {
    // The resulting vertex point we are working on
    // Note that we are creating a new object, not making a copy-reference
    float WorkingVertexX = CubeVertex[i][0];
    float WorkingVertexY = CubeVertex[i][1];
    float WorkingVertexZ = CubeVertex[i][2];

    // Apply rotation onto the vertex
    float Temp = WorkingVertexZ;
    WorkingVertexZ = -WorkingVertexX * sin(CameraRotY) - WorkingVertexZ * cos(CameraRotY);
    WorkingVertexX = -WorkingVertexX * cos(CameraRotY) + Temp * sin(CameraRotY);

    Temp = WorkingVertexZ;
    WorkingVertexZ = -WorkingVertexY * sin(CameraRotX) + WorkingVertexZ * cos(CameraRotX);
    WorkingVertexY = WorkingVertexY * cos(CameraRotX) + Temp * sin(CameraRotX);

    Temp = WorkingVertexX;
    WorkingVertexX = WorkingVertexX * cos(CameraRotZ) - WorkingVertexY * sin(CameraRotZ);
    WorkingVertexY = WorkingVertexY * cos(CameraRotZ) + Temp * sin(CameraRotZ);

    // Apply camera translation after the rotation, so we are actually just rotating the object
    /*
        WorkingVertexX -= CameraPosX;
     WorkingVertexY -= CameraPosY;
     */

    /*
        if(TotalTime <= 5)
     {
     
     float Distance = 30 - TotalTime * TotalTime * 0.2f;
     if(Distance < 2)
     Distance = 2;
     WorkingVertexZ += Distance;
     }
     else
     {
     static float NextTotalTime = 0;
     NextTotalTime += dT;
     
     // Pull distance back...
     float Distance = 2 + NextTotalTime * NextTotalTime * 0.5f;
     if(Distance > 3)
     Distance = 3;
     WorkingVertexZ += Distance;
     }
     */

    //            static float NextTotalTime = 0;
    //            NextTotalTime += dT;

    // Pull distance back...
    float Distance = 3.2;//2 + NextTotalTime * NextTotalTime * 0.5f;

    //            if(Distance > 3.5)
    //                Distance = 3.5;
    WorkingVertexZ += Distance;


    // Convert from x,y,z to x,y
    // This is called a projection transform
    // We are projecting from 3D back to 2D
    static float RatioConst = 25;
    int ScreenX = (RatioConst * (WorkingVertexX)) / WorkingVertexZ;
    int ScreenY = (RatioConst * (WorkingVertexY)) / WorkingVertexZ;

    // Save this on-screen position to render the line locations
    PointList[i][0] = int(CenterX + ScreenX);
    PointList[i][1] = int(CenterY + ScreenY);
  }


  for(int i = 0; i < CubeEdgesLength; i++)
  {
    // Find the two points we are working on
    int Point1X = PointList[CubeEdges[i][0]][0];
    int Point1Y = PointList[CubeEdges[i][0]][1];

    int Point2X = PointList[CubeEdges[i][1]][0];
    int Point2Y = PointList[CubeEdges[i][1]][1];

    // Render the edge by looking up our vertex list
    //            RenderLine(Point1X, Point1Y, Point2X, Point2Y, color(0,255,0));
    RenderLine(Point1X, Point1Y, Point2X, Point2Y, 230);            
  }

}


  
private:
float _sin(float x)
{
    //always wrap input angle to -PI..PI
    if (x < -3.14159265)
        x += 6.28318531;
    else
    if (x >  3.14159265)
        x -= 6.28318531;
    
    //compute sine
    if (x < 0)
        return 1.27323954 * x + .405284735 * x * x;
    else
        return 1.27323954 * x - 0.405284735 * x * x;

}

};



