/*

 Name        : Space Madness                                        
 Author      : Luis Gonzalez - luis@trsi.de
 Date        : 28 March, 2013                                      
 Version     : 1.0                                               
 Platform    : Arduino Mega 2560 - Atmel ATMEGA368                                   
 Environment : Arduino 1.0                                      
 Notes       : Old-school demoscene effects with MOD music playback
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
 
 */

#include "modformat.h"
#include "common.h"

#include <GD.h>
#include <SPI.h>
#include <digitalWriteFast.h>
#include <TimerOne.h>


#include <avr/pgmspace.h>

#include <PS2Keyboard.h>
#include "Arduino.h"
void fillSampleBuffer();
void setup();
void loop();
void loop2();
void runTwister(int framesToRun);
void runCopper(int framesToRun);
void runDeform(int effect, int framesToRun);
void runRotozoomer(int framesToRun);
void runJulia(int framesToRun);
void runFire(int framesToRun);
void runSubpixel(int framesToRun);
void runStars3D(int framesToRun);
void runStars2D(int framesToRun);
void runAscii(int framesToRun);
void runSmallVector(int framesToRun, const char *dotData, const char *dotColorData);
void runBigVector(int framesToRun, const char *dotData);
void showMemory();
struct active *findidle();
byte readlftsongbyte(u16 offset);
void initup(struct unpacker *up, u16 offset);
u8 readbit(struct unpacker *up);
u16 readchunk(struct unpacker *up, u8 n);
void readinstr(byte num, byte pos, byte *dest);
void runcmd(u8 ch, u8 cmd, u8 param);
void playroutine();
void initresources();
void fillLFTSampleBuffer();
void fillLFTSample();
void initSong();
void intro();
void lens();
void calculate_tfm(int lensD, int magFactor);
void metaball();
void metaPalette();
void metaball2();
void metaball3();
void plasma();
int cos256( int amplitude,  int x);
int swing(int i);
int gradient(int i);
void makePlasmaPalette(int arrlen);
void plasma2();
void star();
const int DataPin = 7;
const int IRQpin =  21;
PS2Keyboard keyboard;

static signed int sampleleft;
static signed int sampleright;

#include "textwav.h"
#include "sfx.h"
#include "modplayer.h"
#include "MemoryFree.h"
#include "meanman_32.h"

/* effects */
#include "sinescroller.h"
#include "cube.h"
#include "copper.h"
#include "fire.h"
#include "twister.h"
#include "rotozoomer.h"
#include "deform.h"
#include "julia.h"
#include "subpixel.h"
#include "plasma.h"
#include "sm_logo.h"
#include "StarField2D.h"
#include "StarField3D.h"
#include "vectorball.h"
#include "tile.h"
#include "ascii.h"


/* common audio */

#include "soundbuffer.h"
#define SOUNDBUFFERHI 0x3f00
#define SOUNDBUFFERLO 0x4000
#define SOUNDBUFFER 0x3f00
byte writepointer = 0;

static uint8_t nextPwm;

#include "talkie.h"
//Talkie voice;

/*
void fillTalkSampleBuffer(){
 byte readpointer = GD.rd(COMM+0);
 byte fullness = writepointer - readpointer;
 
 while (fullness < 254  ) {
 voice.fillTalkSample();
 GD.wr(SOUNDBUFFER  + writepointer,(nextPwm));                 
 writepointer++;
 fullness++;
 } 
 }
 */


void fillSampleBuffer(){
  byte readpointer = GD.rd(COMM+0);
  byte fullness = writepointer - readpointer;

  while (fullness < 254  ) {
    fillSample();
    writeSamples(1);
        GD.wr(SOUNDBUFFER  + writepointer, mastersample+sampleright+sampleleft);                 
//    GD.wr(SOUNDBUFFER  + writepointer, mastersample+sampleright+sampleleft);                 
    writepointer++;
    fullness++;
  } 
}

void setup(){

  GD.begin();
  //   initSong();
      GD.microcode(soundbuffer_code, sizeof(soundbuffer_code));
      modload();
  
      uint32_t  addr = GET_FAR_ADDRESS(subi_songdata);
   //   modload(addr);
    Timer1.initialize(20000);         // for subi song  
    Timer1.attachInterrupt(fillSampleBuffer);  // attaches callback() as a timer overflow interrupt

  //  Serial.begin(115200);

  keyboard.begin(DataPin, IRQpin);

/* for end
      ending();
      hideSprites();
      runAscii((256*8) + 3*(8*38) ); // ascii is 256 lines long x 8 pixel tiles
*/


 //     runAscii((256*8) + 3*(8*38) ); // ascii is 256 lines long x 8 pixel tiles
//runCopper(600);
}

//void loop(){}

void loop(){
  
  //intro();

  map_load_Outside();   
  draw_sprite(player_x, player_y, PLAYER_DOWN_1, 0, 0,0);  // player starting pos 

  int item=0;
  while (1) 
  {
    if (keyboard.available()) {

      char key = keyboard.read();
      // read the next key
     movePlayer(key,1);

      switch (key)
      {
      case PS2_ESC:
        item = runmenu(5,5, mainmenu, item,true);

        switch (item) {
        case 1:  
          saySomething();
          break;
        case 2:
          break;
        case 3:
          menu1();   // effects menu
          break;

        case 4:  // battle
          hideSprites();
          fadeAll();
          map_load_Battle();          
                      // item = runmenu(5,5, submenu1, item,true);        
          break;
        case 5:  
          break;
        case 6:   
          break;
        case 7:   
          break;
        }
        break;  
      }

    }   
  }
}


/* goal get all effects running without memory loss */
void loop2() {


  delay(1200);
  // runBigVector(300,vector_largebox_dots);
  //  runSmallVector(200,vector_dna_dots,vector_dna_color);
  //  runSmallVector(200,vector_chicken_dots,vector_chicken_color);
  //  runSmallVector(200,vector_rings_dots,vector_rings_color);


  //  runStars2D(600); 
  //  runAscii( (256*8)  ); // 35 seconds - ascii is 256 lines long x 8 pixel tiles

  // clearScreen();  

  runSubpixel(200);  
  //  runJulia(200); 
  //  runFire(500);
  runTwister(200);
  //  runCopper(200);

  //  runDeform(4,200);    
  //  runDeform(5,200);    
  //  runDeform(7,200);    
  //  runDeform(9,200);    

  //  runRotozoomer(200);
  //  plasma();
  //  metaball2();  
  //  intro();
}


void runTwister(int framesToRun){
  Twister* twister;
  twister = new Twister();
  twister->draw(framesToRun);
  delete(twister);
}

void runCopper(int framesToRun){


//  copper->draw(framesToRun);
//  delete(copper);


  Deform* deform;
  deform = new Deform();


  
  deform->init(5);  // 4,5,7,9
  //deform->draw(framesToRun);  
  //delete(deform);


 Copper* copper;
 copper = new Copper();



/*
  Copper* copper;
  copper = new Copper();

  Deform* rotozoomer;
  deform = new Deform();
*/

  do{  
    deform->render(framesToRun);
    copper->render(framesToRun);

    fb();
  }
  while(--framesToRun);

  delete(copper);
  delete(deform);


}

void runDeform(int effect, int framesToRun){
  Deform* deform;
  deform = new Deform();
  deform->init(effect);
  deform->draw(framesToRun);  
  delete(deform);
}

void runRotozoomer(int framesToRun){

  Subpixel* subpixel;
  subpixel = new Subpixel();

  Rotozoomer* rotozoomer;
  rotozoomer = new Rotozoomer();


  do{  
    rotozoomer->render1(framesToRun);
    subpixel->render();

    fb();
  }
  while(--framesToRun);

  delete(subpixel);
  delete(rotozoomer);
  //  fade();
}

void runJulia(int framesToRun){
  Julia* julia;
  julia = new Julia();
  do{  
    julia->draw();  
    fb();
  }
  while(--framesToRun);


// end transition
  framesToRun=50;  // number of frames to run
   do
   { 
   julia->draw();  
   crossFade(framesToRun);
   fb();
   screenshot();
   }
   while(framesToRun -= 2);


  delete(julia);
}


void runFire(int framesToRun){

  Cube* cube; // declare variable
  cube = new Cube(); // create instance of class C

  Fire* fire; // declare variable
  fire = new Fire(); // create instance of class C

  Sinescroller* sinescroller; // declare variable
  sinescroller = new Sinescroller(); // create instance of class C

  do{

    sinescroller->render(); // call method M          
    cube->draw();    
    fire->render(); // call method M  
    fb();
    screenshot();
  }
  while(--framesToRun);

  fire->draw(15);  
  delete(cube); // free memory        
  delete(fire); // free memory      
  delete(sinescroller); // free memory       
}


void runSubpixel(int framesToRun){
  Plasma* plasma;
  plasma = new Plasma();

  Subpixel* subpixel;
  subpixel = new Subpixel();

  do{
    plasma->render();
    subpixel->render();
    fb();
    //    screenshot();
  }
  while(--framesToRun);

  delete(plasma);  
  delete(subpixel);
}

void runStars3D(int framesToRun){
  StarField3D* stars;
  stars = new StarField3D();

  do{
    stars->draw();
    //    screenshot();
  }
  while(--framesToRun);
  delete(stars);  
  clearScreen();
}

void runStars2D(int framesToRun){
  StarField2D* stars;
  stars = new StarField2D();

  do{
    stars->draw();
    //    screenshot();
  }
  while(--framesToRun);
  fade();
  delete(stars);  
  clearScreen();
}

void runAscii(int framesToRun){
  Ascii* ascii;
  ascii = new Ascii(framesToRun/8);
  do{
    ascii->draw();
    //    screenshot();
  }
  while(--framesToRun);
  delete(ascii);  

  fadeAll();
  clearScreen();
  GD.wr16(SCROLL_Y, 0);   
}

void runSmallVector(int framesToRun, const char *dotData, const char *dotColorData){

  Vectorball* vectorball;
  vectorball = new Vectorball();

  vectorball->LoadObject(dotData,dotColorData);
  vectorball->loadSmallDots();

  do{
    
    vectorball->UpdateBalls();
  }
  while(--framesToRun);

  delete(vectorball);   
}

void runBigVector(int framesToRun, const char *dotData){
  Vectorball* vectorball;
  vectorball = new Vectorball();

  vectorball->LoadObject(dotData);
  vectorball->loadLargeDots();

  do{
    vectorball->UpdateBalls();
    //    screenshot();
  }
  while(--framesToRun);

  delete(vectorball);   
}

void showMemory(){
  char msg[30];
  int mem = freeMemory();
  sprintf(msg, "free mem %d", mem);
  GD.putstr(0, 0, msg);
}

// intro voice
/*
void runIntroSpeech(){
 // initialize timer1, and set the period in microseconds.     
 Timer1.initialize(28000);         
 
 // attaches callback() as a timer overflow interrupt  
 Timer1.attachInterrupt(fillTalkSampleBuffer);  
 
 Talkie* voice;
 voice = new Talkie();
 
 //  voice->say(spDINER);
 voice->say(lpc_intro);
 voice->say(lpc_invite);
 
 // stop calling the audio buffer
 Timer1.stop();
 Timer1.detachInterrupt();
 
 // silence woman!  
 delete(voice);  
 }
 */



/*

 Name      :  LFT Chiptune player
 Notes     :  LFT's hardware chiptune project
 
 music by subi. we ended up using a MOD engine so we scrapped this engine.
 
 */

#include "songdata.h"
#define SOUNDBUFFERHI 0x3f00
#define SOUNDBUFFERLO 0x4000

static int line;

struct active {
  byte instrument;
  uint32_t addr;
  uint16_t pos, length;
  uint16_t x, y;
};
#define NACTIVE 3
static active playing[NACTIVE];

struct active *findidle()
{
  byte i;
  for (i = 0; i < NACTIVE; i++)
    if (playing[i].addr == 0)
      return &playing[i];
  return NULL;
}



#define TRACKLEN 32

// lft song
#define MAXTRACK	0x92
#define SONGLEN		0x37

// subi song
//#define MAXTRACK	0x24
//#define SONGLEN		0x84


#include <avr/pgmspace.h>

volatile u8 callbackwait;
volatile u8 lastsample;
volatile u8 timetoplay;
volatile u8 test;
volatile u8 testwait;
u8 trackwait;
u8 trackpos;
u8 playsong;
u8 songpos;
u32 noiseseed = 1;
u8 light[2];
int globalBuffer;

const u16 freqtable[] = {
  0x010b, 0x011b, 0x012c, 0x013e, 0x0151, 0x0165, 0x017a, 0x0191, 0x01a9,
  0x01c2, 0x01dd, 0x01f9, 0x0217, 0x0237, 0x0259, 0x027d, 0x02a3, 0x02cb,
  0x02f5, 0x0322, 0x0352, 0x0385, 0x03ba, 0x03f3, 0x042f, 0x046f, 0x04b2,
  0x04fa, 0x0546, 0x0596, 0x05eb, 0x0645, 0x06a5, 0x070a, 0x0775, 0x07e6,
  0x085f, 0x08de, 0x0965, 0x09f4, 0x0a8c, 0x0b2c, 0x0bd6, 0x0c8b, 0x0d4a,
  0x0e14, 0x0eea, 0x0fcd, 0x10be, 0x11bd, 0x12cb, 0x13e9, 0x1518, 0x1659,
  0x17ad, 0x1916, 0x1a94, 0x1c28, 0x1dd5, 0x1f9b, 0x217c, 0x237a, 0x2596,
  0x27d3, 0x2a31, 0x2cb3, 0x2f5b, 0x322c, 0x3528, 0x3851, 0x3bab, 0x3f37,
  0x42f9, 0x46f5, 0x4b2d, 0x4fa6, 0x5462, 0x5967, 0x5eb7, 0x6459, 0x6a51,
  0x70a3, 0x7756, 0x7e6f
};

/*
const u16 freqtable[] = {
 0x0085, 0x008d, 0x0096, 0x009f, 0x00a8, 0x00b2, 0x00bd, 0x00c8, 0x00d4,
 0x00e1, 0x00ee, 0x00fc, 0x010b, 0x011b, 0x012c, 0x013e, 0x0151, 0x0165,
 0x017a, 0x0191, 0x01a9, 0x01c2, 0x01dd, 0x01f9, 0x0217, 0x0237, 0x0259,
 0x027d, 0x02a3, 0x02cb, 0x02f5, 0x0322, 0x0352, 0x0385, 0x03ba, 0x03f3,
 0x042f, 0x046f, 0x04b2, 0x04fa, 0x0546, 0x0596, 0x05eb, 0x0645, 0x06a5,
 0x070a, 0x0775, 0x07e6, 0x085f, 0x08de, 0x0965, 0x09f4, 0x0a8c, 0x0b2c,
 0x0bd6, 0x0c8b, 0x0d4a, 0x0e14, 0x0eea, 0x0fcd, 0x10be, 0x11bd, 0x12cb,
 0x13e9, 0x1518, 0x1659, 0x17ad, 0x1916, 0x1a94, 0x1c28, 0x1dd5, 0x1f9b,
 0x217c, 0x237a, 0x2596, 0x27d3, 0x2a31, 0x2cb3, 0x2f5b, 0x322c, 0x3528,
 0x3851, 0x3bab, 0x3f37
 };
 */

const s8 sinetable[] = {
  0, 12, 25, 37, 49, 60, 71, 81, 90, 98, 106, 112, 117, 122, 125, 126,
  127, 126, 125, 122, 117, 112, 106, 98, 90, 81, 71, 60, 49, 37, 25, 12,
  0, -12, -25, -37, -49, -60, -71, -81, -90, -98, -106, -112, -117, -122,
  -125, -126, -127, -126, -125, -122, -117, -112, -106, -98, -90, -81,
  -71, -60, -49, -37, -25, -12
};

const u8 validcmds[] = "0dfijlmtvw~+=";

enum {
  WF_TRI,
  WF_SAW,
  WF_PUL,
  WF_NOI
};

volatile struct oscillator {
  u16	freq;
  u16	phase;
  u16	duty;
  u8	waveform;
  u8	volume;	// 0-255
} 
osc[4];

struct trackline {
  u8	note;
  u8	instr;
  u8	cmd[2];
  u8	param[2];
};

struct track {
  struct trackline	line[TRACKLEN];
};

struct unpacker {
  u16	nextbyte;
  u8	buffer;
  u8	bits;
};

struct lftchannel {
  struct unpacker		trackup;
  u8			tnum;
  s8			transp;
  u8			tnote;
  u8			lastinstr;
  u8			inum;
  u16			iptr;
  u8			iwait;
  u8			inote;
  s8			bendd;
  s16			bend;
  s8			volumed;
  s16			dutyd;
  u8			vdepth;
  u8			vrate;
  u8			vpos;
  s16			inertia;
  u16			slur;
} 
lftchannel[4];

u16 resources[16 + MAXTRACK];

struct unpacker songup;

byte readlftsongbyte(u16 offset)
{

  return pgm_read_byte_near(&songdata[0] + offset); 
}

void initup(struct unpacker *up, u16 offset) {
  up->nextbyte = offset;
  up->bits = 0;
}

u8 readbit(struct unpacker *up) {
  u8 val;

  if(!up->bits) {
    up->buffer = readlftsongbyte(up->nextbyte++);
    up->bits = 8;
  }

  up->bits--;
  val = up->buffer & 1;
  up->buffer >>= 1;

  return val;
}

u16 readchunk(struct unpacker *up, u8 n) {
  u16 val = 0;
  u8 i;

  for(i = 0; i < n; i++) {
    if(readbit(up)) {
      val |= (1 << i);
    }
  }

  return val;
}

void readinstr(byte num, byte pos, byte *dest) {
  dest[0] = readlftsongbyte(resources[num] + 2 * pos + 0);
  dest[1] = readlftsongbyte(resources[num] + 2 * pos + 1);
}

void runcmd(u8 ch, u8 cmd, u8 param) {
  switch(validcmds[cmd]) {
  case '0':
    lftchannel[ch].inum = 0;
    break;
  case 'd':
    osc[ch].duty = param << 8;
    break;
  case 'f':
    lftchannel[ch].volumed = param;
    break;
  case 'i':
    lftchannel[ch].inertia = param << 1;
    break;
  case 'j':
    lftchannel[ch].iptr = param;
    break;
  case 'l':
    lftchannel[ch].bendd = param;
    break;
  case 'm':
    lftchannel[ch].dutyd = param << 6;
    break;
  case 't':
    lftchannel[ch].iwait = param;
    break;
  case 'v':
    osc[ch].volume = param;
    break;
  case 'w':
    osc[ch].waveform = param;
    break;
  case '+':
    lftchannel[ch].inote = param + lftchannel[ch].tnote - 12 * 4;
    break;
  case '=':
    lftchannel[ch].inote = param;
    break;
  case '~':
    if(lftchannel[ch].vdepth != (param >> 4)) {
      lftchannel[ch].vpos = 0;
    }
    lftchannel[ch].vdepth = param >> 4;
    lftchannel[ch].vrate = param & 15;
    break;
  }
}

void playroutine() {			// called at 50 Hz
  u8 ch;
  u8 lights;

  if(playsong) {
    if(trackwait) {
      trackwait--;
    } 
    else {
      trackwait = 4;

      if(!trackpos) {
        if(playsong) {
          if(songpos >= SONGLEN) {
            playsong = 0;
          } 
          else {
            for(ch = 0; ch < 4; ch++) {
              u8 gottransp;
              u8 transp;

              gottransp = readchunk(&songup, 1);
              lftchannel[ch].tnum = readchunk(&songup, 6);
              if(gottransp) {
                transp = readchunk(&songup, 4);
                if(transp & 0x8) transp |= 0xf0;
              } 
              else {
                transp = 0;
              }
              lftchannel[ch].transp = (s8) transp;
              if(lftchannel[ch].tnum) {
                initup(&lftchannel[ch].trackup, resources[16 + lftchannel[ch].tnum - 1]);
              }
            }
            songpos++;
          }
        }
      }

      if(playsong) {
        for(ch = 0; ch < 4; ch++) {
          if(lftchannel[ch].tnum) {
            u8 note, instr, cmd, param;
            u8 fields;

            fields = readchunk(&lftchannel[ch].trackup, 3);
            note = 0;
            instr = 0;
            cmd = 0;
            param = 0;
            if(fields & 1) note = readchunk(&lftchannel[ch].trackup, 7);
            if(fields & 2) instr = readchunk(&lftchannel[ch].trackup, 4);
            if(fields & 4) {
              cmd = readchunk(&lftchannel[ch].trackup, 4);
              param = readchunk(&lftchannel[ch].trackup, 8);
            }
            if(note) {
              lftchannel[ch].tnote = note + lftchannel[ch].transp;
              if(!instr) instr = lftchannel[ch].lastinstr;
            }
            if(instr) {
              if(instr == 2) light[1] = 5;
              if(instr == 1) {
                light[0] = 5;
                if(lftchannel[ch].tnum == 4) {
                  light[0] = light[1] = 3;
                }
              }
              if(instr == 7) {
                light[0] = light[1] = 30;
              }
              lftchannel[ch].lastinstr = instr;
              lftchannel[ch].inum = instr;
              lftchannel[ch].iptr = 0;
              lftchannel[ch].iwait = 0;
              lftchannel[ch].bend = 0;
              lftchannel[ch].bendd = 0;
              lftchannel[ch].volumed = 0;
              lftchannel[ch].dutyd = 0;
              lftchannel[ch].vdepth = 0;
            }
            if(cmd) runcmd(ch, cmd, param);
          }
        }

        trackpos++;
        trackpos &= 31;
      }
    }
  }

  for(ch = 0; ch < 4; ch++) {
    s16 vol;
    u16 duty;
    u16 slur;

    while(lftchannel[ch].inum && !lftchannel[ch].iwait) {
      u8 il[2];

      readinstr(lftchannel[ch].inum, lftchannel[ch].iptr, il);
      lftchannel[ch].iptr++;

      runcmd(ch, il[0], il[1]);
    }
    if(lftchannel[ch].iwait) lftchannel[ch].iwait--;

    if(lftchannel[ch].inertia) {
      s16 diff;

      slur = lftchannel[ch].slur;
      diff = freqtable[lftchannel[ch].inote] - slur;
      //diff >>= lftchannel[ch].inertia;
      if(diff > 0) {
        if(diff > lftchannel[ch].inertia) diff = lftchannel[ch].inertia;
      } 
      else if(diff < 0) {
        if(diff < -lftchannel[ch].inertia) diff = -lftchannel[ch].inertia;
      }
      slur += diff;
      lftchannel[ch].slur = slur;
    } 
    else {
      slur = freqtable[lftchannel[ch].inote];
    }
    osc[ch].freq =
      slur +
      lftchannel[ch].bend +
      ((lftchannel[ch].vdepth * sinetable[lftchannel[ch].vpos & 63]) >> 2);
    lftchannel[ch].bend += lftchannel[ch].bendd;
    vol = osc[ch].volume + lftchannel[ch].volumed;
    if(vol < 0) vol = 0;
    if(vol > 255) vol = 255;
    osc[ch].volume = vol;

    duty = osc[ch].duty + lftchannel[ch].dutyd;
    if(duty > 0xe000) duty = 0x2000;
    if(duty < 0x2000) duty = 0xe000;
    osc[ch].duty = duty;

    lftchannel[ch].vpos += lftchannel[ch].vrate;
  }

  lights = 0;
  if(light[0]) {
    light[0]--;
    lights |= 0x02;
  }
  if(light[1]) {
    light[1]--;
    lights |= 0x10;
  }
  //	PORTC = lights;
}

void initresources() {
  u8 i;
  struct unpacker up;

  initup(&up, 0);
  for(i = 0; i < 16 + MAXTRACK; i++) {
    resources[i] = readchunk(&up, 13);
  }

  initup(&songup, resources[0]);
}


byte lftwritepointer = 0;
void fillLFTSampleBuffer(){

  playroutine();
  byte readpointer = GD.rd(COMM+0);
  byte fullness = lftwritepointer - readpointer;

  while (fullness < 254  ) {
    // if (fullness&1)
    //  {

    fillLFTSample();
    //  }      
    GD.wr(SOUNDBUFFERHI + lftwritepointer, highByte(globalBuffer));               
    lftwritepointer++;
    fullness++;
  }
}


//At the 8KHz rate, 254 samples is enough for 31 milliseconds of sound. 
//So if a game refills the sample buffer at least once during every 36 Hz frame, the audio playback will be continuous.
void fillLFTSample(){
  u8 i;
  s16 acc;
  u8 newbit;

  newbit = 0;
  if(noiseseed & 0x80000000L) newbit ^= 1;
  if(noiseseed & 0x01000000L) newbit ^= 1;
  if(noiseseed & 0x00000040L) newbit ^= 1;
  if(noiseseed & 0x00000200L) newbit ^= 1;
  noiseseed = (noiseseed << 1) | newbit;

  /*
  if(callbackwait) {
   callbackwait--;
   } 
   else {
   timetoplay++;
   callbackwait = 180 - 1;
   }
   */

  acc = 0;
  for(i = 0; i < 4; i++) {
    s8 value; // [-32,31]

    switch(osc[i].waveform) {
    case WF_TRI:
      if(osc[i].phase < 0x8000) {
        value = -32 + (osc[i].phase >> 9);
      } 
      else {
        value = 31 - ((osc[i].phase - 0x8000) >> 9);
      }
      break;
    case WF_SAW:
      value = -32 + (osc[i].phase >> 10);
      break;
    case WF_PUL:
      value = (osc[i].phase > osc[i].duty)? -32 : 31;
      break;
    case WF_NOI:
      value = (noiseseed & 63) - 32;
      break;
    default:
      value = 0;
      break;
    }
    osc[i].phase += osc[i].freq;

    acc += value * osc[i].volume; // rhs = [-8160,7905]
  }
  // acc [-32640,31620]
  // lastsample = 128 + (acc >> 8);	// [1,251]


  //noInterrupts();
  //    GD.wr16(SAMPLE_R, acc);
  //    GD.wr16(SAMPLE_L, acc);

  //  lastsample =  128+(acc >> 8);

  globalBuffer = acc;
}


void initSong(){

  //  GD.microcode(soundbuffer_code, sizeof(soundbuffer_code));

  timetoplay = 0;
  trackwait = 0;
  trackpos = 0;
  playsong = 1;
  songpos = 0;

  osc[0].volume = 0;
  lftchannel[0].inum = 0;
  osc[1].volume = 0;
  lftchannel[1].inum = 0;
  osc[2].volume = 0;
  lftchannel[2].inum = 0;
  osc[3].volume = 0;
  lftchannel[3].inum = 0;
  initresources();


  // 1 initialaze set to 83 is 8000Hz
  // multiply by 250 to get 32Hz
  // multiply by 160 to get 50Hz
  // higher numbers are slower 170

  // configure the 16 bit hardware timer on the ATmega168/328
  Timer1.initialize(63*180);         // initialize timer1, and set the period in microseconds. 

  //  Timer1.initialize(13000);         // for subi song    
  Timer1.attachInterrupt(fillLFTSampleBuffer);  // attaches callback() as a timer overflow interrupt

}

/*

 Name      :  Intro
 Notes     :  Splash screen image on startup
 
 image converter used:
 http://gameduino.com/tools/bg/ 
 
 */

#define LOGO_TRSI_FRAMES 1
static void draw_logo_trsi(int x, int y, byte anim, byte rot, byte jk = 0) {
  switch (anim) {
  case 0:
    GD.xsprite(x, y, -80, -48, 0, 4, rot, jk);
    GD.xsprite(x, y, -64, -48, 0, 6, rot, jk);
    GD.xsprite(x, y, -48, -48, 1, 4, rot, jk);
    GD.xsprite(x, y, -32, -48, 1, 6, rot, jk);
    GD.xsprite(x, y, -16, -48, 2, 4, rot, jk);
    GD.xsprite(x, y, 0, -48, 2, 6, rot, jk);
    GD.xsprite(x, y, 16, -48, 3, 4, rot, jk);
    GD.xsprite(x, y, 32, -48, 3, 6, rot, jk);
    GD.xsprite(x, y, 48, -48, 4, 4, rot, jk);
    GD.xsprite(x, y, 64, -48, 4, 6, rot, jk);
    GD.xsprite(x, y, -80, -32, 5, 4, rot, jk);
    GD.xsprite(x, y, -64, -32, 5, 6, rot, jk);
    GD.xsprite(x, y, -48, -32, 6, 4, rot, jk);
    GD.xsprite(x, y, -32, -32, 6, 6, rot, jk);
    GD.xsprite(x, y, -16, -32, 7, 4, rot, jk);
    GD.xsprite(x, y, 0, -32, 7, 6, rot, jk);
    GD.xsprite(x, y, 16, -32, 8, 4, rot, jk);
    GD.xsprite(x, y, 32, -32, 8, 6, rot, jk);
    GD.xsprite(x, y, 48, -32, 9, 4, rot, jk);
    GD.xsprite(x, y, 64, -32, 9, 6, rot, jk);
    GD.xsprite(x, y, -80, -16, 10, 4, rot, jk);
    GD.xsprite(x, y, -64, -16, 10, 6, rot, jk);
    GD.xsprite(x, y, -48, -16, 11, 4, rot, jk);
    GD.xsprite(x, y, -32, -16, 11, 6, rot, jk);
    GD.xsprite(x, y, -16, -16, 12, 4, rot, jk);
    GD.xsprite(x, y, 0, -16, 12, 6, rot, jk);
    GD.xsprite(x, y, 16, -16, 13, 4, rot, jk);
    GD.xsprite(x, y, 32, -16, 13, 6, rot, jk);
    GD.xsprite(x, y, 48, -16, 14, 4, rot, jk);
    GD.xsprite(x, y, 64, -16, 14, 6, rot, jk);
    GD.xsprite(x, y, -96, 0, 15, 4, rot, jk);
    GD.xsprite(x, y, -80, 0, 15, 6, rot, jk);
    GD.xsprite(x, y, -64, 0, 16, 4, rot, jk);
    GD.xsprite(x, y, -48, 0, 16, 6, rot, jk);
    GD.xsprite(x, y, -32, 0, 17, 4, rot, jk);
    GD.xsprite(x, y, -16, 0, 17, 6, rot, jk);
    GD.xsprite(x, y, 0, 0, 18, 4, rot, jk);
    GD.xsprite(x, y, 16, 0, 18, 6, rot, jk);
    GD.xsprite(x, y, 32, 0, 19, 4, rot, jk);
    GD.xsprite(x, y, 48, 0, 19, 6, rot, jk);
    GD.xsprite(x, y, -96, 16, 20, 4, rot, jk);
    GD.xsprite(x, y, -80, 16, 20, 6, rot, jk);
    GD.xsprite(x, y, -64, 16, 21, 4, rot, jk);
    GD.xsprite(x, y, -48, 16, 21, 6, rot, jk);
    GD.xsprite(x, y, -32, 16, 22, 4, rot, jk);
    GD.xsprite(x, y, -16, 16, 22, 6, rot, jk);
    GD.xsprite(x, y, 0, 16, 23, 4, rot, jk);
    GD.xsprite(x, y, 16, 16, 23, 6, rot, jk);
    GD.xsprite(x, y, 32, 16, 24, 4, rot, jk);
    GD.xsprite(x, y, 48, 16, 24, 6, rot, jk);
    GD.xsprite(x, y, -16, 32, 25, 4, rot, jk);
    GD.xsprite(x, y, 0, 32, 25, 6, rot, jk);
    GD.xsprite(x, y, 16, 32, 26, 4, rot, jk);
    GD.xsprite(x, y, 32, 32, 26, 6, rot, jk);
    GD.xsprite(x, y, -16, 48, 27, 4, rot, jk);
    GD.xsprite(x, y, 0, 48, 27, 6, rot, jk);
    GD.xsprite(x, y, 16, 48, 28, 4, rot, jk);
    break;
  }
}

static PROGMEM prog_uchar logo_trsi_sprimg[] = {

  0xc9,  0x41,  0x6f,  0xf8,  0x0f,  0xe0,  0x41,  0x3e,  0xa4,  0x87,  0xfd,  0x48,  0x1e,  0xed,  0x63,  0x7b, 
  0xfc,  0x4f,  0xe2,  0x29,  0x3e,  0xad,  0x67,  0x7c,  0xff,  0xf6,  0xed,  0x67,  0xff,  0x60,  0x1d,  0xfc, 
  0x83,  0x49,  0xf0,  0x0f,  0x76,  0xd1,  0x37,  0xd8,  0x87,  0xaf,  0xb0,  0x1e,  0x23,  0xf4,  0x07,  0xc8, 
  0x28,  0x1f,  0x9c,  0x83,  0x5f,  0x98,  0x0a,  0x06,  0xfa,  0xf5,  0xaa,  0x8d,  0xe2,  0x61,  0x53,  0x3d, 
  0x5f,  0xb4,  0x68,  0x23,  0x6b,  0x80,  0x54,  0x8f,  0x1e,  0x2d,  0xd8,  0x00,  0xd1,  0xff,  0x87,  0xf8, 
  0x88,  0x1f,  0xeb,  0x13,  0x7f,  0x9a,  0xcf,  0xfa,  0xb2,  0x64,  0x37,  0x80,  0x07,  0xf9,  0x90,  0x1e, 
  0xf6,  0x23,  0xb9,  0xde,  0x00,  0xa9,  0x3b,  0xc0,  0xeb,  0x0e,  0x8e,  0x01,  0x20,  0x30,  0x06,  0xa5, 
  0x80,  0xa9,  0x5b,  0xad,  0xea,  0x00,  0x11,  0x80,  0x95,  0xab,  0x0a,  0xd0,  0x68,  0xc9,  0xa2,  0x26, 
  0x4b,  0x9a,  0x34,  0x29,  0xa6,  0x04,  0x18,  0x10,  0x60,  0x14,  0x00,  0x18,  0x48,  0x42,  0x1d,  0x00, 
  0x50,  0x10,  0x27,  0x10,  0x00,  0x18,  0x05,  0x07,  0x06,  0xc1,  0x51,  0x11,  0x6a,  0xd2,  0x48,  0x09, 
  0x20,  0x20,  0x14,  0x9c,  0x15,  0x06,  0x07,  0x09,  0x92,  0x90,  0x22,  0x08,  0x0e,  0x21,  0x28,  0x24, 
  0x32,  0x08,  0x00,  0x12,  0x18,  0x00,  0x88,  0x1a,  0xc1,  0x98,  0x10,  0xa1,  0xf8,  0x18,  0x1c,  0x24, 
  0x21,  0x08,  0xb7,  0xe1,  0x4b,  0x11,  0xa3,  0xf8,  0xf1,  0xff,  0x83,  0x78,  0x88,  0x0f,  0xeb,  0x11, 
  0x3f,  0x9a,  0xc7,  0x7a,  0xff,  0x7e,  0x03,  0x78,  0x90,  0x0e,  0xde,  0x3e,  0xbc,  0xfb,  0x09,  0xed, 
  0x76,  0x23,  0xa0,  0x47,  0xcf,  0x5e,  0x23,  0xbb,  0x5d,  0xdb,  0x7e,  0x7d,  0x7b,  0x0e,  0x01,  0x3e, 
  0xfc,  0x44,  0x54,  0xdb,  0x21,  0xc0,  0xb7,  0x57,  0x8f,  0x5e,  0x63,  0x42,  0x00,  0xf4,  0x15,  0x0a, 

  0x69,  0xf1,  0x12,  0x25,  0xca,  0xb0,  0x45,  0xaf,  0xc1,  0xc1,  0xab,  0x57,  0xcf,  0x91,  0x23,  0x4e, 
  0xc0,  0x84,  0x05,  0x0b,  0x36,  0x64,  0xcc,  0x1c,  0x6d,  0x84,  0xd0,  0xe0,  0x01,  0x2d,  0x44,  0x40, 
  0x87,  0x0c,  0x11,  0x06,  0x77,  0xa3,  0xe1,  0xdb,  0x23,  0x34,  0x9c,  0xc1,  0x0b,  0x2a,  0x74,  0xa8, 
  0x5c,  0xc3,  0x55,  0x96,  0x80,  0x02,  0x25,  0x46,  0xc0,  0x80,  0x0e,  0x1b,  0x0a,  0x34,  0xa8,  0xc2, 
  0xe3,  0x93,  0x40,  0x68,  0xd8,  0x60,  0xe1,  0x82,  0x0e,  0x03,  0x06,  0x2e,  0x68,  0xd0,  0x20,  0x1b, 
  0x0e,  0x9f,  0x94,  0x71,  0xa5,  0x40,  0xc8,  0xd1,  0x03,  0x6a,  0x42,  0x80,  0x4c,  0x31,  0x39,  0x18, 
  0x84,  0xd8,  0xff,  0x87,  0xf8,  0x88,  0x1f,  0xeb,  0x13,  0x7f,  0x9a,  0xcf,  0xfa,  0xf9,  0xbe,  0xf8, 
  0x57,  0xf8,  0x9a,  0xdf,  0xe8,  0x5b,  0x7f,  0x97,  0xef,  0xfb,  0xc3,  0x2e,  0x9e,  0x7f,  0x80,  0x14, 
  0x23,  0x42,  0x78,  0x80,  0x30,  0xc1,  0x3d,  0x60,  0x0a,  0x87,  0x0f,  0x1f,  0x0e,  0x74,  0x8c,  0x18, 
  0x21,  0x52,  0xe4,  0x18,  0x08,  0x02,  0x86,  0x40,  0x8e,  0x14,  0xe1,  0x42,  0x87,  0x08,  0x31,  0x38, 
  0x04,  0xc3,  0xc0,  0x40,  0x51,  0x20,  0xc5,  0x00,  0x10,  0x1c,  0x83,  0x24,  0xc5,  0x90,  0x10,  0x36, 
  0x0c,  0x1a,  0x14,  0x28,  0xb0,  0x05,  0x04,  0xe0,  0xa8,  0x11,  0x0d,  0x82,  0x65,  0xe8,  0x8a,  0xce, 
  0x10,  0xe3,  0x63,  0xfc,  0x08,  0x58,  0xe1,  0x3a,  0x41,  0x87,  0xa4,  0x34,  0xf1,  0xff,  0x83,  0x78, 
  0x88,  0x0f,  0xeb,  0x11,  0x3f,  0x9a,  0xc7,  0xfa,  0xf8,  0x9e,  0xf8,  0x53,  0x78,  0x9a,  0xcf,  0xe8, 
  0x59,  0x3f,  0x97,  0xe7,  0xfb,  0xc2,  0x2e,  0x22,  0x80,  0x9a,  0xf0,  0x11,  0x40,  0x28,  0x53,  0x0d, 
  0xc0,  0x42,  0xbf,  0x43,  0x00,  0xa1,  0x11,  0xbc,  0xb8,  0x0a,  0x15,  0xfa,  0x2f,  0x42,  0x60,  0xc0, 

  0xa4,  0x1e,  0xac,  0x84,  0x0b,  0xb5,  0x1a,  0x5e,  0xbd,  0x52,  0xa5,  0xcc,  0x5d,  0x70,  0xb8,  0x30, 
  0x6a,  0x39,  0xf8,  0x71,  0x0e,  0x06,  0xc1,  0x31,  0xde,  0xc5,  0x02,  0xa1,  0xe1,  0x22,  0x7c,  0x46, 
  0xc7,  0xf8,  0xe5,  0xc1,  0xab,  0x77,  0xfb,  0x01,  0x50,  0x7f,  0x80,  0xd0,  0xaf,  0x9e,  0x3f,  0x78, 
  0xb4,  0x60,  0xd1,  0x06,  0x80,  0x60,  0xa2,  0x1e,  0x3d,  0x7a,  0xf0,  0xe8,  0x83,  0x43,  0x80,  0x0c, 
  0x97,  0xe2,  0xc9,  0x07,  0x8b,  0x00,  0x15,  0x2e,  0xc3,  0x67,  0x58,  0x08,  0x13,  0xe0,  0x03,  0x30, 
  0xd8,  0x00,  0x14,  0x3d,  0xf0,  0x93,  0xc7,  0xcf,  0x9e,  0x7c,  0xc0,  0x08,  0x08,  0x74,  0xc9,  0xb2, 
  0x25,  0x4b,  0x36,  0x78,  0xf0,  0x47,  0x1f,  0x3d,  0x02,  0x34,  0xda,  0x60,  0x41,  0x96,  0x74,  0x72, 
  0x50,  0x1f,  0x2e,  0x03,  0x64,  0xa0,  0x08,  0x5f,  0x6c,  0xb3,  0x62,  0x70,  0x08,  0x1b,  0x21,  0x29, 
  0x3a,  0x85,  0xcf,  0x6c,  0x11,  0x36,  0x03,  0x74,  0xf8,  0x08,  0x00,  0x45,  0x29,  0x1a,  0x85,  0xa0, 
  0xe8,  0x18,  0x3d,  0x62,  0x50,  0xf8,  0x0d,  0x8f,  0xe1,  0x2b,  0x86,  0x05,  0x80,  0xf0,  0x9b,  0x21, 
  0x02,  0xa8,  0x5c,  0xb6,  0x72,  0x9b,  0x93,  0xfe,  0xe0,  0x11,  0xae,  0x42,  0x65,  0xfc,  0x6a,  0x9f, 
  0x11,  0x44,  0x7e,  0xf0,  0x0c,  0xdf,  0x25,  0x31,  0xc6,  0xa6,  0x08,  0x04,  0x84,  0x08,  0x04,  0x08, 
  0x46,  0x84,  0x02,  0x24,  0x23,  0x49,  0x8c,  0xa4,  0x48,  0x92,  0x42,  0x28,  0x04,  0x84,  0x84,  0xe0, 
  0x1c,  0x42,  0x82,  0x46,  0x30,  0x0a,  0xce,  0x52,  0x22,  0x84,  0x24,  0x25,  0x04,  0x10,  0x10,  0x20, 
  0xc0,  0x20,  0x40,  0x30,  0x50,  0x10,  0x20,  0x02,  0xa0,  0xa2,  0x18,  0xa0,  0x4c,  0x5a,  0x80,  0x0c, 
  0x80,  0xda,  0x94,  0xe9,  0xd4,  0xa6,  0x53,  0xa5,  0x4e,  0x69,  0x21,  0x20,  0x24,  0x85,  0x40,  0x9d, 

  0x01,  0x4a,  0x97,  0xb4,  0x00,  0x10,  0xcc,  0xc3,  0x45,  0x00,  0x08,  0x1f,  0x21,  0x30,  0x06,  0x46, 
  0x07,  0x95,  0xa1,  0x48,  0x27,  0x31,  0x66,  0x01,  0x90,  0x65,  0x20,  0x60,  0x22,  0x25,  0x40,  0x19, 
  0xd1,  0x42,  0x2c,  0x04,  0x04,  0x28,  0x46,  0x4c,  0x02,  0x40,  0xc0,  0xd4,  0x62,  0x24,  0x38,  0xb2, 
  0x8c,  0x18,  0x84,  0x82,  0x63,  0x84,  0x52,  0x62,  0x25,  0x7c,  0x44,  0xc8,  0xe0,  0x10,  0x20,  0xc2, 
  0xa5,  0x94,  0xe0,  0x11,  0x3d,  0xc2,  0x65,  0x50,  0x20,  0x12,  0xae,  0x83,  0xa7,  0xb5,  0x70,  0x1f, 
  0x9c,  0xc2,  0x47,  0xfc,  0x24,  0x10,  0x15,  0xe3,  0x67,  0x34,  0x8c,  0x80,  0xd6,  0x82,  0x43,  0x8b, 
  0x08,  0x9f,  0x84,  0xba,  0x62,  0x04,  0x32,  0x02,  0x0b,  0x57,  0x80,  0x80,  0x8a,  0x19,  0x3a,  0x0e, 
  0xc8,  0x98,  0x03,  0x84,  0x45,  0xcf,  0x9c,  0x21,  0xe3,  0x84,  0x15,  0x20,  0x20,  0x43,  0xc5,  0x03, 
  0x03,  0x7a,  0x08,  0x34,  0xec,  0x11,  0x33,  0x44,  0x9c,  0x38,  0xa1,  0xe1,  0x80,  0x88,  0x15,  0x0e, 
  0x0e,  0xe8,  0x58,  0x03,  0x85,  0x60,  0xcb,  0x10,  0x21,  0x47,  0xe7,  0xf0,  0x11,  0x1c,  0xc3,  0x86, 
  0x60,  0x03,  0x10,  0x21,  0xe2,  0xc6,  0x8b,  0x0b,  0x2b,  0x64,  0x1c,  0x01,  0x20,  0x41,  0xe0,  0xc1, 
  0x2b,  0xd6,  0x2e,  0x1e,  0xac,  0xb0,  0x61,  0xc3,  0x82,  0x08,  0x03,  0x0e,  0x9e,  0x00,  0x89,  0x17, 
  0x8e,  0x59,  0xfb,  0xf4,  0x61,  0xc0,  0x86,  0x0b,  0x0b,  0x3a,  0x74,  0xb8,  0x21,  0x20,  0x5c,  0x04, 
  0x08,  0xb3,  0x76,  0x83,  0x07,  0xee,  0x1c,  0x80,  0x03,  0x7b,  0x70,  0x0c,  0x9f,  0x78,  0xec,  0x9a, 
  0x15,  0x0b,  0x15,  0x36,  0x0e,  0x5c,  0xb8,  0x71,  0x84,  0x8d,  0xf0,  0xc5,  0xd7,  0xae,  0x5d,  0xa3, 
  0x98,  0xf1,  0xe2,  0xc6,  0x1d,  0x26,  0x03,  0x10,  0x5c,  0x31,  0x7e,  0x0c,  0x1b,  0xe6,  0x1c,  0x1c, 

  0x02,  0x44,  0xc8,  0x08,  0x5f,  0x74,  0x4c,  0x78,  0xb6,  0x1c,  0x02,  0xc2,  0x05,  0xee,  0x18,  0x10, 
  0x9e,  0x60,  0x18,  0x35,  0xec,  0xd9,  0xa0,  0x61,  0xc1,  0xe1,  0x99,  0x2e,  0x1e,  0x3a,  0xc1,  0xd1, 
  0xb2,  0x67,  0xcd,  0x86,  0x35,  0xc7,  0x8f,  0x2c,  0x18,  0x1e,  0xa0,  0x73,  0x0e,  0x0a,  0xe1,  0x23, 
  0x60,  0x08,  0xc6,  0x1b,  0x00,  0xe8,  0xa6,  0x07,  0xa8,  0x49,  0x42,  0xb3,  0x62,  0xcd,  0x96,  0x03, 
  0x86,  0xd6,  0xb8,  0x11,  0x22,  0xf0,  0xb4,  0x89,  0x60,  0x20,  0x58,  0x71,  0x0c,  0x08,  0x18,  0x82, 
  0x21,  0x62,  0x0c,  0x10,  0x18,  0x39,  0x06,  0x80,  0x40,  0x10,  0x20,  0x29,  0x06,  0x88,  0x16,  0x11, 
  0x02,  0x29,  0x06,  0x97,  0x10,  0x14,  0xbe,  0x22,  0x64,  0xb8,  0xc5,  0x48,  0x99,  0x62,  0x00,  0x0f, 
  0x0e,  0x58,  0x03,  0x00,  0xd4,  0x82,  0x60,  0x30,  0x00,  0x04,  0x8c,  0x00,  0x69,  0xd8,  0x8c,  0x19, 
  0xb3,  0x01,  0x21,  0x58,  0x06,  0x54,  0xe3,  0xc1,  0x23,  0x44,  0x88,  0x31,  0x68,  0x31,  0xc0,  0x86, 
  0x0f,  0xbd,  0x66,  0x02,  0xa4,  0xe5,  0x18,  0x18,  0x30,  0x62,  0x92,  0x46,  0xe3,  0x7a,  0xc5,  0x98, 
  0x0d,  0x1f,  0x96,  0x2d,  0x86,  0x84,  0x80,  0xa5,  0x59,  0xa3,  0x66,  0xed,  0x7a,  0xd5,  0xea,  0x8d, 
  0x0e,  0xc1,  0x31,  0x60,  0x6a,  0x0e,  0x01,  0x0a,  0x35,  0x2b,  0x55,  0xab,  0x26,  0x5c,  0x04,  0xa8, 
  0xb0,  0x11,  0x00,  0xea,  0x87,  0x62,  0xc5,  0x4a,  0xd5,  0xf0,  0x30,  0x60,  0x31,  0x1a,  0x05,  0x07, 
  0x85,  0xc1,  0x22,  0x78,  0x2a,  0xc5,  0xc3,  0x47,  0xe0,  0x20,  0x68,  0x03,  0xe1,  0x3a,  0x40,  0x48, 
  0x55,  0x2a,  0x46,  0xcc,  0x75,  0x80,  0x2c,  0x38,  0x18,  0x2b,  0x96,  0xac,  0x58,  0xba,  0x74,  0x38, 
  0xb0,  0xe1,  0xd0,  0xe1,  0xda,  0x0f,  0xb4,  0x46,  0x23,  0xe5,  0xca,  0xc3,  0x07,  0x6c,  0x18,  0x74, 

  0x78,  0xe8,  0xd1,  0x1e,  0x7e,  0x94,  0x32,  0x57,  0x2c,  0x51,  0xb0,  0x51,  0xa1,  0x01,  0xd2,  0x6b, 
  0x00,  0xf2,  0xec,  0xd5,  0x47,  0x00,  0x10,  0x1a,  0x02,  0x42,  0x42,  0xf0,  0xf2,  0xe8,  0xd9,  0x7b, 
  0x70,  0xf4,  0x11,  0x34,  0xc2,  0x87,  0x51,  0xab,  0x61,  0xc3,  0x73,  0x58,  0xf0,  0x13,  0xa0,  0x0c, 
  0x0a,  0xe8,  0x9e,  0x1e,  0x3c,  0x7a,  0x0f,  0xc6,  0x01,  0xb3,  0x13,  0xd9,  0xf1,  0x15,  0x1c,  0x02, 
  0x58,  0xf0,  0x30,  0x6d,  0xc2,  0x4e,  0xb8,  0x0a,  0x7e,  0xa6,  0xcd,  0xd8,  0xf6,  0xe3,  0xcb,  0x8f, 
  0xef,  0x00,  0x1d,  0x29,  0xfc,  0x86,  0xcb,  0x60,  0xe1,  0xc3,  0x77,  0x80,  0x89,  0x90,  0xb6,  0x6c, 
  0xfb,  0x0a,  0x40,  0xbe,  0x03,  0x44,  0x84,  0xf6,  0x6b,  0x2b,  0x38,  0xf8,  0x0c,  0xd0,  0x91,  0x32, 
  0x07,  0xf8,  0x0d,  0x1e,  0xb6,  0x7c,  0x06,  0xcd,  0x10,  0xec,  0xdb,  0x77,  0x70,  0xf0,  0x6b,  0xd3, 
  0x66,  0xf0,  0xf0,  0xe9,  0x2d,  0x40,  0x86,  0x8a,  0xe0,  0x10,  0x02,  0x6d,  0x86,  0xcf,  0x60,  0x18, 
  0xb0,  0xc2,  0xaf,  0xcd,  0x70,  0x19,  0x00,  0x42,  0x62,  0x0c,  0xf2,  0x1d,  0x3c,  0xc3,  0x65,  0x40, 
  0x89,  0x41,  0xc1,  0x3f,  0x52,  0x86,  0x2f,  0xfb,  0x01,  0x58,  0xbd,  0x7e,  0xf5,  0x01,  0x58,  0xae, 
  0xda,  0xe0,  0xad,  0x3f,  0x40,  0x04,  0xc0,  0x10,  0x1c,  0x7e,  0x42,  0x78,  0xf8,  0x0f,  0x4e,  0x11, 
  0x52,  0x7e,  0xfc,  0x89,  0x29,  0x21,  0x90,  0xbe,  0xff,  0x18,  0x1a,  0x3d,  0xf9,  0xf3,  0x27,  0x4f, 
  0x1f,  0x7c,  0xf0,  0x09,  0x57,  0xfc,  0xc1,  0x83,  0x47,  0x8f,  0x9e,  0x7c,  0xfa,  0x57,  0x1f,  0x1c, 
  0x43,  0x60,  0x74,  0x13,  0x8f,  0x3f,  0x5c,  0xa1,  0x27,  0x9b,  0xef,  0xc2,  0x46,  0x74,  0x22,  0x8b, 
  0xb6,  0xb8,  0x98,  0x4f,  0x02,  0xd1,  0x88,  0x3c,  0xda,  0xf0,  0x1d,  0x6e,  0xc1,  0x07,  0xc7,  0xf0, 

  0x93,  0x16,  0xc2,  0x55,  0x74,  0x24,  0xcb,  0xb6,  0x2a,  0x65,  0xc9,  0xb4,  0x8d,  0x36,  0x7c,  0x24, 
  0xcf,  0x2c,  0x19,  0xae,  0xa3,  0x61,  0xf8,  0x2e,  0x03,  0x99,  0x21,  0xb8,  0x91,  0x25,  0x4d,  0x88, 
  0x21,  0x83,  0x00,  0xc0,  0x88,  0x00,  0x23,  0x49,  0x92,  0x25,  0x49,  0x91,  0x24,  0x55,  0x8a,  0x36, 
  0x26,  0x41,  0x82,  0x81,  0xa4,  0x00,  0xc0,  0x48,  0x4a,  0x00,  0x96,  0xa6,  0x8c,  0x59,  0x78,  0x90, 
  0x20,  0x21,  0x04,  0x86,  0xcb,  0x90,  0xa4,  0x8d,  0x71,  0x74,  0x94,  0x14,  0x02,  0xc2,  0x47,  0x70, 
  0x96,  0xa6,  0x84,  0x81,  0xc4,  0xe8,  0x14,  0x0c,  0x95,  0x44,  0x83,  0x00,  0x21,  0x8d,  0x1a,  0xa3, 
  0xf8,  0x11,  0xcd,  0x43,  0x00,  0x35,  0x2c,  0x8c,  0x94,  0x48,  0x0a,  0x13,  0x01,  0x2a,  0x00,  0x29, 
  0xc3,  0x42,  0x2d,  0x09,  0x85,  0x84,  0x74,  0x23,  0x05,  0x09,  0x16,  0x2c,  0xb1,  0x53,  0x99,  0x92, 
  0x00,  0xa0,  0x2c,  0x5e,  0x2a,  0xe3,  0x42,  0x8d,  0x1a,  0xae,  0x48,  0x18,  0x9d,  0xa2,  0x97,  0x12, 
  0x2a,  0x58,  0x68,  0x89,  0x0c,  0x1f,  0x92,  0xc2,  0x47,  0x84,  0x0c,  0x8e,  0xda,  0xa8,  0xd0,  0xc4, 
  0xc9,  0x2b,  0x77,  0x29,  0x51,  0x14,  0x23,  0x95,  0x68,  0x11,  0x12,  0x1c,  0x02,  0x44,  0x34,  0xd6, 
  0x12,  0x0c,  0xb4,  0x58,  0xd3,  0xa2,  0x0b,  0x17,  0x2d,  0x9a,  0x34,  0x2b,  0x63,  0x0e,  0x08,  0x8e, 
  0x21,  0xc0,  0x96,  0xc8,  0xf0,  0x41,  0x2b,  0x7c,  0xa4,  0xce,  0x38,  0x11,  0x2e,  0x6d,  0x05,  0x80, 
  0xf0,  0x11,  0x02,  0xb2,  0xa0,  0x34,  0x6d,  0x01,  0xd0,  0x66,  0x08,  0xb0,  0x19,  0x1c,  0x71,  0xe2, 
  0xa2,  0x8d,  0x1b,  0x06,  0x65,  0xad,  0x86,  0x35,  0x2b,  0xd6,  0x1a,  0x20,  0x2c,  0x2b,  0x55,  0xac, 
  0x94,  0x36,  0x2e,  0xac,  0x82,  0x0d,  0x2b,  0x36,  0x2c,  0x58,  0x71,  0x50,  0x50,  0xec,  0x55,  0x70, 

  0xc0,  0xc0,  0xc5,  0xd5,  0x70,  0xb8,  0x09,  0x0e,  0x5a,  0x3d,  0x87,  0x4f,  0xa1,  0x42,  0x8d,  0x07, 
  0x87,  0x70,  0xad,  0xd4,  0x72,  0xf8,  0x54,  0x6a,  0xd8,  0xa8,  0xd1,  0x38,  0x19,  0x2e,  0x94,  0xaa, 
  0x50,  0xee,  0xd5,  0xab,  0x75,  0x15,  0x6a,  0x0c,  0x47,  0x07,  0xc1,  0x46,  0xa3,  0xa1,  0x72,  0xee, 
  0x56,  0xbd,  0x7a,  0xf7,  0x63,  0x26,  0x78,  0x18,  0x37,  0xae,  0x3d,  0x52,  0x04,  0x87,  0x80,  0xe1, 
  0xd5,  0x87,  0x1f,  0x3f,  0x76,  0xd4,  0x18,  0x15,  0x1a,  0x2e,  0x53,  0xa7,  0x52,  0xa3,  0xc1,  0x22, 
  0x40,  0x9a,  0x31,  0xaa,  0x3d,  0x7e,  0x84,  0x8d,  0x08,  0xa1,  0x23,  0x38,  0x47,  0x00,  0x33,  0xc6, 
  0x93,  0x91,  0x52,  0x1b,  0xc1,  0x25,  0x06,  0x04,  0x87,  0x14,  0xa0,  0x54,  0x7b,  0x70,  0xb7,  0x63, 
  0x34,  0x5d,  0x5a,  0x4d,  0x92,  0x7e,  0x02,  0x56,  0xf0,  0xd2,  0x6a,  0x34,  0x0b,  0x44,  0xd3,  0xf0, 
  0x6d,  0xd8,  0xaa,  0xe7,  0xe0,  0x9f,  0x3c,  0x0b,  0x85,  0x67,  0xef,  0x79,  0x2a,  0xf9,  0xda,  0x0d, 
  0x0e,  0x3e,  0x7c,  0x54,  0x81,  0x10,  0x9a,  0x30,  0xec,  0x93,  0x52,  0xad,  0x47,  0x8f,  0x1a,  0x3d, 
  0xca,  0x35,  0x6b,  0xd6,  0x1a,  0x20,  0x94,  0x6b,  0x57,  0x2c,  0xd9,  0x94,  0x2a,  0xd5,  0x6a,  0x02, 
  0xa0,  0x0a,  0xa5,  0x9a,  0xad,  0x6a,  0x0f,  0x0e,  0xca,  0xa5,  0xab,  0xd2,  0x1d,  0x20,  0x42,  0x80, 
  0x9c,  0x90,  0xa0,  0x5c,  0xab,  0x75,  0x8d,  0xc1,  0x41,  0xb6,  0xaa,  0x8e,  0x10,  0x0c,  0x64,  0x04, 
  0xac,  0x96,  0xa0,  0x43,  0x86,  0x6c,  0x56,  0xd1,  0x21,  0x00,  0xa8,  0x51,  0x1b,  0x12,  0x42,  0xc1, 
  0x40,  0x54,  0x2d,  0x5b,  0x56,  0x00,  0x50,  0x2d,  0x27,  0x00,  0x04,  0x2c,  0xe5,  0x32,  0x54,  0xc9, 
  0x96,  0xcd,  0x32,  0x04,  0xca,  0x96,  0x23,  0x37,  0x04,  0x04,  0x02,  0x39,  0xc1,  0x41,  0x6d,  0x08, 

  0xd0,  0xa5,  0x22,  0x3c,  0x86,  0x40,  0x59,  0xa1,  0x20,  0x5c,  0x84,  0x05,  0x56,  0xaa,  0x75,  0x27, 
  0x80,  0xa0,  0x10,  0x00,  0x22,  0x63,  0x70,  0x50,  0x1d,  0x08,  0xd4,  0xaa,  0x89,  0x1f,  0x64,  0xc3, 
  0xa7,  0xcc,  0xe0,  0x11,  0x20,  0xe3,  0x85,  0xda,  0x10,  0x10,  0x12,  0xe8,  0xb2,  0x95,  0x25,  0x33, 
  0xb8,  0x04,  0x0f,  0xb9,  0xd9,  0x50,  0x0d,  0x5e,  0x66,  0xc1,  0x38,  0x12,  0xb0,  0x0c,  0x0e,  0x11, 
  0x4a,  0x35,  0xab,  0x78,  0x19,  0x80,  0xc3,  0xa7,  0xec,  0xea,  0xa1,  0x9a,  0x51,  0xdc,  0x94,  0x19, 
  0x00,  0xc2,  0x57,  0x84,  0x62,  0x23,  0x3d,  0x03,  0xa4,  0x88,  0x00,  0x15,  0x3f,  0x63,  0x82,  0x1a, 
  0x59,  0x4c,  0xe4,  0xb3,  0x0e,  0x41,  0x32,  0x59,  0xda,  0x0c,  0x00,  0x3e,  0x03,  0x90,  0xb7,  0x00, 
  0xe9,  0xdb,  0x6f,  0x80,  0x08,  0xbe,  0xc1,  0x31,  0x00,  0xf8,  0x09,  0x81,  0xe1,  0x27,  0x38,  0x7b, 
  0x0b,  0x5f,  0xd6,  0x62,  0x40,  0x30,  0x09,  0x01,  0x5e,  0x7c,  0xc7,  0x40,  0x6b,  0x01,  0x20,  0xdc, 
  0x7a,  0xf3,  0x15,  0x3e,  0x02,  0x80,  0x35,  0x2b,  0x96,  0xbc,  0x84,  0xad,  0x00,  0x19,  0x3c,  0xc2, 
  0xb7,  0x27,  0x0f,  0x1e,  0xbc,  0x04,  0x03,  0x4f,  0x9e,  0xe3,  0x43,  0xfc,  0xb6,  0xe2,  0xc5,  0x53, 
  0x88,  0x08,  0x08,  0x5e,  0xc3,  0x53,  0x0c,  0x4c,  0x1e,  0xe1,  0x20,  0x40,  0xa6,  0x8b,  0x7c,  0x11, 
  0x9c,  0xf2,  0x47,  0xdc,  0x49,  0x09,  0xc1,  0x33,  0x3b,  0x16,  0x0f,  0x9f,  0xbe,  0xf2,  0x56,  0x70, 
  0x0e,  0x40,  0xc1,  0x22,  0x04,  0x06,  0xa2,  0xe8,  0x94,  0x03,  0x7d,  0x05,  0xc0,  0x10,  0xe5,  0x2b, 
  0x18,  0x85,  0xaf,  0xe0,  0x14,  0x0a,  0xe3,  0x8d,  0x5f,  0xff,  0x91,  0x23,  0xc0,  0x04,  0x97,  0xe0, 
  0x11,  0x9c,  0x7d,  0x64,  0xcd,  0xf0,  0x1b,  0xc0,  0xec,  0x07,  0xd0,  0xe0,  0x1e,  0x42,  0xf4,  0x87, 

  0xfb,  0xe0,  0x12,  0x7e,  0x83,  0xa9,  0xfe,  0x00,  0x10,  0xcf,  0xc3,  0x35,  0x7f,  0xf2,  0x49,  0x3f, 
  0x5c,  0x91,  0x27,  0x8f,  0x3e,  0xad,  0x47,  0x43,  0xf5,  0xc1,  0x23,  0x7c,  0xc6,  0xfd,  0xe0,  0x80, 
  0x3e,  0x7c,  0x86,  0xfb,  0x70,  0xc9,  0x3e,  0x20,  0x04,  0xf7,  0x68,  0x14,  0x3d,  0x73,  0x9f,  0xf8, 
  0xb0,  0x11,  0x2e,  0x03,  0x46,  0x34,  0x15,  0x1f,  0xae,  0x42,  0x20,  0xd9,  0xf8,  0x62,  0x1e,  0x7f, 
  0xb8,  0x8c,  0x50,  0x6c,  0x93,  0x49,  0x70,  0x63,  0x1f,  0x7e,  0xf4,  0x83,  0xa7,  0x4f,  0x3e,  0x40, 
  0x84,  0xff,  0xe0,  0x41,  0x1e,  0x3c,  0xfa,  0x00,  0x98,  0x3f,  0xc9,  0x92,  0x66,  0xcb,  0x9a,  0x34, 
  0x11,  0x49,  0x01,  0x50,  0x8a,  0x22,  0x25,  0x92,  0x94,  0x48,  0x51,  0x16,  0x2c,  0x58,  0x13,  0x27, 
  0xc0,  0x20,  0x04,  0x32,  0x0a,  0x4e,  0xd2,  0xa4,  0xb0,  0x66,  0xcd,  0x8a,  0x51,  0x70,  0x94,  0x12, 
  0x3e,  0xa4,  0x84,  0xcb,  0xf0,  0xc9,  0x22,  0x38,  0x85,  0x30,  0x69,  0xd2,  0x88,  0x30,  0x23,  0x12, 
  0x14,  0xa2,  0x40,  0xb8,  0x54,  0x12,  0x2e,  0x42,  0x80,  0x12,  0x22,  0xc1,  0x31,  0x40,  0x07,  0xa3, 
  0x50,  0x18,  0x1c,  0x02,  0x70,  0x50,  0x0a,  0x58,  0xda,  0x92,  0x50,  0x0c,  0x0a,  0xc1,  0x4a,  0xb4, 
  0x68,  0x89,  0x84,  0x71,  0x2b,  0x18,  0x2b,  0xb1,  0xa6,  0xc5,  0x66,  0x74,  0x0f,  0x9b,  0x01,  0xc2, 
  0xa6,  0xce,  0x78,  0x93,  0x36,  0xe2,  0x84,  0x96,  0x70,  0x19,  0x00,  0x4a,  0x44,  0xf4,  0x64,  0x12, 
  0xee,  0x42,  0x40,  0x45,  0x0c,  0x90,  0xe1,  0x2e,  0x7c,  0x45,  0x28,  0x65,  0x5a,  0xa2,  0x43,  0xb8, 
  0x0a,  0x9e,  0xe1,  0xc7,  0x4a,  0xb8,  0x09,  0x3f,  0xd1,  0x51,  0x9b,  0xcd,  0x00,  0x11,  0x3c,  0x7d, 
  0xfa,  0xa4,  0x49,  0x8b,  0x26,  0x4d,  0xda,  0x74,  0x69,  0x8b,  0x35,  0x1b,  0x00,  0x8d,  0x5b,  0xf5, 

  0xea,  0x35,  0x58,  0x84,  0x08,  0xda,  0x21,  0x90,  0x6f,  0x08,  0xb4,  0x1e,  0x1c,  0x3c,  0x07,  0x97, 
  0x00,  0x18,  0x02,  0xfc,  0xfa,  0x09,  0x1e,  0xe1,  0x03,  0x57,  0xb8,  0x0a,  0xc1,  0xe6,  0xfd,  0x7a, 
  0x0e,  0x1f,  0xe1,  0x36,  0x7c,  0xd2,  0xe7,  0x2f,  0x36,  0x06,  0xd2,  0xc6,  0x1d,  0x37,  0x43,  0x14, 
  0xfd,  0x00,  0x40,  0x97,  0x87,  0x61,  0xcf,  0xc1,  0x3d,  0x00,  0xf0,  0xa7,  0x4f,  0x87,  0xb2,  0x51, 
  0xba,  0x21,  0x00,  0x6f,  0x80,  0xa4,  0x1b,  0x0c,  0x42,  0x24,  0x1d,  0x1a,  0x34,  0xe2,  0x49,  0x30, 
  0x0a,  0x98,  0x7c,  0x93,  0x85,  0xda,  0x68,  0x11,  0x3e,  0x82,  0x6b,  0xf4,  0xa0,  0x43,  0x93,  0xad, 
  0xda,  0xf0,  0x1e,  0x13,  0xe8,  0xd2,  0xe1,  0x45,  0x96,  0x6d,  0xfc,  0x08,  0x6f,  0xc1,  0x93,  0xba, 
  0xd9,  0x60,  0x97,  0x02,  0x02,  0x24,  0x1d,  0x3c,  0x66,  0xf9,  0xc2,  0xc7,  0x1f,  0xdf,  0x32,  0x06, 
  0x5f,  0x3a,  0xe6,  0xcd,  0x07,  0x57,  0xf1,  0x01,  0x80,  0x3f,  0x7f,  0xbe,  0x7c,  0xcc,  0x07,  0x0e, 
  0xf8,  0xc9,  0x98,  0xbf,  0x78,  0xf5,  0xea,  0xe5,  0xcb,  0x67,  0x63,  0x3f,  0x00,  0xe9,  0x0f,  0x0a, 
  0xfa,  0x03,  0x81,  0x7e,  0xf9,  0x21,  0x40,  0xbd,  0xff,  0x10,  0x44,  0x3e,  0x48,  0x04,  0xc7,  0x00, 
  0x11,  0x21,  0xfd,  0x07,  0x80,  0xe0,  0xc9,  0x3e,  0x78,  0x84,  0x80,  0x90,  0x18,  0x13,  0x82,  0x67, 
  0x44,  0x50,  0x1f,  0x3c,  0x63,  0x50,  0xc4,  0x0c,  0x5e,  0x7e,  0xed,  0x26,  0xea,  0x08,  0x15,  0xbe, 
  0xfc,  0xfa,  0xf0,  0x98,  0xa8,  0x53,  0xa7,  0x7c,  0xf6,  0xc1,  0x69,  0x32,  0x24,  0xea,  0xcc,  0xc8, 
  0x9e,  0xbd,  0x77,  0x1f,  0xde,  0xbd,  0x7a,  0xf7,  0x93,  0x68,  0x82,  0x23,  0xfb,  0x60,  0xe0,  0xdd, 
  0x77,  0xca,  0x4a,  0x45,  0x01,  0x68,  0x23,  0x7a,  0xf5,  0x11,  0x00,  0xbc,  0xdb,  0x09,  0x0e,  0xb9, 

  0x22,  0x44,  0x90,  0x0f,  0x5e,  0x01,  0x53,  0x07,  0x9b,  0x70,  0x1d,  0x12,  0xe2,  0x85,  0x67,  0x8f, 
  0x9e,  0x3d,  0x2b,  0x27,  0x43,  0x3e,  0x00,  0x46,  0x88,  0x40,  0x11,  0x1c,  0x53,  0x82,  0x6a,  0x52, 
  0x6c,  0xa3,  0x71,  0xf8,  0xf6,  0x63,  0x4a,  0x15,  0x49,  0x56,  0xc4,  0x48,  0x85,  0xeb,  0x90,  0xe0, 
  0xdd,  0x97,  0x49,  0x95,  0xc4,  0x58,  0x91,  0x0a,  0x90,  0x64,  0x63,  0xb2,  0x4f,  0x61,  0xcc,  0x48, 
  0x05,  0x00,  0x92,  0x01,  0x23,  0x42,  0x24,  0x0c,  0xf4,  0x21,  0xc1,  0xa4,  0x2e,  0x53,  0xba,  0x4c, 
  0xea,  0x0c,  0x80,  0x36,  0x03,  0x90,  0x4f,  0x5f,  0x3a,  0x55,  0xea,  0x52,  0x65,  0x52,  0x9b,  0x4e, 
  0x6d,  0x36,  0x03,  0x84,  0xb5,  0xe0,  0xad,  0x2b,  0x98,  0x05,  0x27,  0x6f,  0xbe,  0x75,  0x29,  0xd3, 
  0x16,  0x02,  0xad,  0x85,  0x14,  0x6f,  0xbe,  0x7c,  0x85,  0xfb,  0xe0,  0x14,  0x1f,  0x7c,  0x87,  0x8b, 
  0xb0,  0x61,  0x2d,  0x00,  0x87,  0x02,  0x9f,  0x5e,  0x7c,  0x07,  0x8f,  0xe0,  0x12,  0xa1,  0xe2,  0x45, 
  0x78,  0xf0,  0x1d,  0xee,  0xac,  0x79,  0x0b,  0x00,  0x5e,  0x7c,  0x84,  0xaf,  0x60,  0x1b,  0x2e,  0xc2, 
  0x93,  0xdf,  0xe0,  0x68,  0x25,  0x2c,  0x06,  0xa4,  0x78,  0x19,  0x02,  0xac,  0x04,  0x87,  0xa8,  0x14, 
  0x90,  0xc2,  0x63,  0xf8,  0x0c,  0x01,  0x5e,  0xc2,  0x44,  0x70,  0x0e,  0x5e,  0xe1,  0x2b,  0xbc,  0xc7, 
  0x8b,  0x00,  0x61,  0x25,  0x00,  0x84,  0xaf,  0x88,  0x11,  0x1e,  0x42,  0x70,  0x74,  0x0b,  0xae,  0xe1, 
  0xd3,  0x4b,  0x08,  0xf6,  0x55,  0x35,  0x43,  0xa0,  0xdf,  0x3b,  0x23,  0x1e,  0xfd,  0x50,  0x0c,  0x50, 
  0x6c,  0x9f,  0x88,  0xfe,  0xb1,  0x10,  0x02,  0x42,  0x12,  0xd9,  0x37,  0xab,  0x3f,  0x78,  0x84,  0x90, 
  0xf0,  0xfb,  0x14,  0x42,  0xe8,  0x97,  0x7d,  0x1b,  0xfc,  0xc3,  0xcf,  0x17,  0x4e,  0x50,  0x11,  0x30, 

  0x7c,  0x47,  0xd0,  0x94,  0x12,  0xbe,  0x62,  0xb4,  0xfe,  0x94,  0x14,  0x7e,  0x12,  0xbc,  0x7a,  0xf2, 
  0xea,  0x03,  0x40,  0xf0,  0x4c,  0xb0,  0x99,  0x2a,  0x77,  0x04,  0xff,  0x50,  0x1a,  0xbe,  0x42,  0x78, 
  0xc4,  0x0c,  0xde,  0x11,  0xbe,  0x59,  0x04,  0xff,  0xe0,  0x1f,  0xe2,  0xc3,  0x7f,  0xcc,  0xef,  0x87, 
  0x52,  0xa4,  0x05,  0x00,  0x65,  0xdf,  0xf0,  0xbf,  0x04,  0x00,  0x29,  0x21,  0x40,  0xc9,  0xc3,  0xf8, 
  0x37,  0xc1,  0x3b,  0x5c,  0xbc,  0xc4,  0x7f,  0x1d,  0x9e,  0x7c,  0x4a,  0x93,  0xa2,  0x4d,  0xcb,  0xc3, 
  0x0a,  0x41,  0xc1,  0x2b,  0x1a,  0xbe,  0xdf,  0xf0,  0xec,  0x53,  0x4b,  0xb8,  0x08,  0x10,  0x2f,  0xfb, 
  0x05,  0x04,  0xef,  0xe0,  0x1e,  0x9c,  0x6e,  0x82,  0x15,  0x2d,  0x17,  0xe2,  0x61,  0xfe,  0xe0,  0x83, 
  0x64,  0x2b,  0x00,  0xd9,  0xb4,  0x75,  0x2c,  0x4e,  0xd0,  0xf9,  0xb4,  0x1d,  0x80,  0xde,  0xd0,  0x37, 
  0x7a,  0x81,  0xb6,  0xed,  0x06,  0xc8,  0x20,  0x14,  0x30,  0x03,  0x50,  0x70,  0x0f,  0x2e,  0xc1,  0xc3, 
  0x6e,  0xe0,  0x09,  0x74,  0xc1,  0x3d,  0xb4,  0x87,  0xff,  0x28,  0x1e,  0xe3,  0xf7,  0xa1,  0xf9,  0x00, 
  0x20,  0x1e,  0x3e,  0xfd,  0x00,  0xc0,  0x5f,  0x3c,  0xff,  0x9b,  0x70,  0x0b,  0xd4,  0xda,  0x0f,  0x9e, 
  0xfc,  0xf1,  0x07,  0x2f,  0xfc,  0xe2,  0x8f,  0x83,  0x7e,  0xf6,  0xe7,  0x40,  0x7c,  0x00,  0x08,  0x9f, 
  0xf4,  0x03,  0x81,  0x7a,  0xf9,  0xf2,  0x23,  0xc4,  0x35,  0x88,  0x1e,  0x11,  0x84,  0x3f,  0xfd,  0xe0, 
  0x10,  0x3c,  0xcd,  0xc7,  0x89,  0x10,  0x18,  0xbc,  0xf0,  0xab,  0x7f,  0x0d,  0xf2,  0xed,  0x07,  0xc5, 
  0x90,  0x18,  0x83,  0xc0,  0x47,  0x0b,  0xfd,  0xc9,  0x43,  0x3f,  0x7d,  0xfc,  0x69,  0x33,  0x02,  0x25, 
  0x80,  0x04,  0x91,  0x3e,  0xec,  0xab,  0x87,  0x9f,  0x3d,  0x42,  0x50,  0xf0,  0xca,  0x9e,  0x39,  0x80, 

  0x3d,  0x7c,  0xfe,  0x39,  0x33,  0x3c,  0x26,  0x0f,  0xf9,  0x72,  0xd3,  0x97,  0x7a,  0xf1,  0xa9,  0x20, 
  0x00,  0x66,  0x0f,  0xf6,  0xf2,  0xd9,  0xfe,  0xcd,  0x8a,  0xa0,  0x3f,  0x40,  0x14,  0xa3,  0x32,  0x48, 
  0xf7,  0x57,  0x99,  0x2f,  0x82,  0xc1,  0x51,  0xbd,  0x79,  0xfc,  0x49,  0xe1,  0x0b,  0xd5,  0x89,  0x00, 
  0x50,  0x37,  0x83,  0x07,  0xff,  0xe0,  0x5d,  0x25,  0xc4,  0xe7,  0x8a,  0xe0,  0x58,  0x1c,  0x62,  0xe0, 
  0x67,  0x34,  0x5f,  0x02,  0x2a,  0x43,  0x08,  0x6a,  0x5a,  0xbf,  0xcb,  0x7c,  0x15,  0x0c,  0x5e,  0x49, 
  0xa0,  0x5e,  0xa8,  0x62,  0x4d,  0x2a,  0x40,  0xb0,  0x22,  0xc5,  0x32,  0x40,  0x06,  0x00,  0xb2,  0x64, 
  0x59,  0x93,  0x0e,  0x8c,  0x24,  0x03,  0x04,  0x31,  0x52,  0x01,  0x82,  0x08,  0xb1,  0x00,  0x19,  0xbc, 
  0x42,  0x60,  0x70,  0x0c,  0x21,  0xc1,  0x18,  0x19,  0xb1,  0xe0,  0x80,  0x84,  0x58,  0x80,  0x24,  0x82, 
  0x24,  0x7a,  0x21,  0x0b,  0x50,  0xd1,  0x28,  0x60,  0x44,  0x0f,  0x54,  0x44,  0x89,  0xa2,  0x42,  0x86, 
  0x2a,  0x5c,  0x06,  0x80,  0x70,  0x11,  0x3c,  0xe2,  0x03,  0x41,  0xe2,  0x68,  0x22,  0x64,  0x30,  0x0c, 
  0x97,  0xc1,  0x13,  0x15,  0x1a,  0x34,  0x64,  0x51,  0x11,  0x03,  0x82,  0x2c,  0x7c,  0x20,  0x22,  0x12, 
  0x20,  0xe3,  0x06,  0xaa,  0x00,  0x09,  0x2c,  0x38,  0x24,  0x23,  0x22,  0x84,  0x88,  0xc4,  0x0d,  0xa4, 
  0x41,  0x01,  0x48,  0xbc,  0x08,  0x1f,  0xe1,  0x2a,  0x7c,  0x21,  0x45,  0x01,  0x22,  0xdc,  0xc6,  0x20, 
  0x84,  0x08,  0x82,  0x43,  0xba,  0x08,  0xc0,  0xe1,  0x13,  0x11,  0x21,  0x84,  0x08,  0x91,  0xa3,  0x8c, 
  0x19,  0xd9,  0x28,  0x3a,  0x11,  0x92,  0x04,  0x10,  0x30,  0x72,  0x50,  0xa8,  0x80,  0x20,  0x09,  0x10, 
  0x88,  0x02,  0x04,  0x22,  0x04,  0x92,  0x24,  0x20,  0x0d,  0x00,  0xd1,  0x31,  0x04,  0x64,  0x47,  0x42, 

  0x08,  0x18,  0x49,  0x92,  0x84,  0x06,  0x04,  0x1a,  0x50,  0x48,  0x00,  0x01,  0x09,  0xd7,  0x88,  0x82, 
  0x87,  0x04,  0x2b,  0xd6,  0xbc,  0x78,  0x0b,  0x90,  0x3e,  0x7d,  0xfa,  0xf2,  0x1d,  0x00,  0xfc,  0x06, 
  0x00,  0x2b,  0x5e,  0x02,  0x90,  0xaf,  0xe0,  0x10,  0x3c,  0xfd,  0x86,  0xc0,  0xe0,  0x19,  0x3e,  0x83, 
  0x4b,  0x00,  0x0a,  0xdf,  0xe1,  0xc1,  0x67,  0xb4,  0x0c,  0xdf,  0x71,  0x33,  0x20,  0x44,  0xc7,  0x68, 
  0x1c,  0x20,  0xc3,  0x53,  0x32,  0x8c,  0xb7,  0x7e,  0xfd,  0x07,  0xf7,  0xf0,  0x13,  0x3c,  0x93,  0x41, 
  0xbc,  0x0a,  0xae,  0xe1,  0x23,  0x2b,  0x06,  0xf7,  0xe0,  0x15,  0x2e,  0x8b,  0x6a,  0x70,  0xf2,  0x1f, 
  0x2d,  0x83,  0x7b,  0x70,  0xf4,  0x1f,  0x5e,  0xe2,  0x5d,  0xf0,  0x2a,  0x73,  0xe1,  0x39,  0x7c,  0x96, 
  0xe9,  0xe0,  0x16,  0x3c,  0xf2,  0x7c,  0x00,  0x49,  0xc6,  0x65,  0x3a,  0xfc,  0xd8,  0xb7,  0x6f,  0x57, 
  0xed,  0x6f,  0x08,  0x10,  0xa8,  0x03,  0x04,  0x2a,  0x54,  0xa8,  0xc1,  0xa2,  0x35,  0x6f,  0x5e,  0x7d, 
  0xf0,  0x40,  0x1b,  0x0c,  0xd0,  0xa2,  0x46,  0xbd,  0x38,  0x02,  0xa4,  0x79,  0xfd,  0xea,  0xc9,  0x87, 
  0x8f,  0x10,  0x80,  0x36,  0x04,  0x06,  0x0f,  0x50,  0xa8,  0xd5,  0xab,  0x97,  0x4f,  0x9e,  0x7c,  0x30, 
  0x08,  0x4e,  0xa0,  0x51,  0x83,  0x0c,  0x01,  0xf2,  0x03,  0x00,  0x7b,  0xf4,  0xe1,  0x32,  0x7c,  0xa2, 
  0x05,  0x0d,  0x1a,  0x24,  0xcc,  0x60,  0x11,  0x00,  0xd1,  0x07,  0xcf,  0xa4,  0x10,  0x1c,  0x40,  0xc3, 
  0x06,  0x19,  0x1e,  0x43,  0x04,  0xf9,  0x10,  0x06,  0x1b,  0x56,  0xf8,  0x88,  0x50,  0xe8,  0xc3,  0x05, 
  0x58,  0xb0,  0xa0,  0x03,  0x04,  0x6c,  0xd0,  0xfa,  0x43,  0x50,  0xc4,  0x80,  0x1f,  0x82,  0xc0,  0xc2, 
  0x85,  0x0d,  0x3b,  0x3b,  0x24,  0x68,  0xf2,  0xe0,  0x83,  0x13,  0xe8,  0x00,  0xa1,  0x3f,  0x02,  0x86, 

  0xe0,  0x98,  0x00,  0x37,  0x04,  0x05,  0x0f,  0xfd,  0xc9,  0x33,  0x40,  0x04,  0xc0,  0x04,  0xad,  0xdf, 
  0x7e,  0x08,  0x0a,  0xde,  0xb5,  0xb1,  0x49,  0x04,  0xc7,  0x84,  0x10,  0x7c,  0x4a,  0x63,  0x2b,  0x68, 
  0x1f,  0x11,  0xb2,  0x21,  0xc4,  0x40,  0xf4,  0x21,  0x38,  0x38,  0xc5,  0xc8,  0xe0,  0x1c,  0x00,  0xda, 
  0x80,  0x06,  0x4b,  0x01,  0xc2,  0x42,  0x80,  0xf4,  0x14,  0x00,  0xb7,  0x96,  0x06,  0x0d,  0x16,  0x02, 
  0x44,  0x08,  0xf0,  0xe0,  0x29,  0x40,  0x1c,  0x25,  0xdf,  0xc1,  0xd1,  0x62,  0xf0,  0xb0,  0x10,  0x9e, 
  0xce,  0x74,  0xb8,  0x0a,  0x01,  0x1e,  0x02,  0xa0,  0xb7,  0xb3,  0xb8,  0x0e,  0xc2,  0x65,  0x80,  0xf0, 
  0x18,  0x3c,  0xa3,  0xf5,  0x96,  0x88,  0x46,  0xe1,  0xc5,  0xa7,  0xb7,  0x3b,  0x64,  0x31,  0x7c,  0x06, 
  0x4b,  0x8f,  0xc1,  0x35,  0x7c,  0x5a,  0x8c,  0x56,  0x9e,  0xe3,  0xe5,  0x91,  0x0e,  0x1f,  0xc1,  0xd3, 
  0xa3,  0xe7,  0x70,  0x11,  0x9c,  0x03,  0x40,  0x32,  0xf2,  0x58,  0x24,  0xc2,  0xc3,  0xaf,  0xba,  0x9c, 
  0xc1,  0xc9,  0x4b,  0x34,  0xfc,  0x65,  0x69,  0xc3,  0x4b,  0x70,  0x08,  0x81,  0x9f,  0xfa,  0x0f,  0x06, 
  0xe3,  0x68,  0xf5,  0x69,  0x82,  0x47,  0x08,  0x4e,  0x3b,  0xbf,  0x2a,  0x78,  0x84,  0xaf,  0x70,  0xf7, 
  0x3f,  0x82,  0x67,  0x48,  0xf0,  0x91,  0x4c,  0x83,  0xe7,  0xff,  0x98,  0xb0,  0x33,  0xe8,  0x4b,  0x06, 
  0x97,  0x5d,  0x21,  0x7e,  0x42,  0x06,  0x80,  0x95,  0x11,  0xbe,  0x42,  0x8a,  0x7a,  0xfe,  0x33,  0x51, 
  0x7d,  0x70,  0x0f,  0x50,  0xea,  0xcf,  0x45,  0x02,  0x8e,  0x30,  0x01,  0x3a,  0xc5,  0x04,  0xf7,  0xf0, 
  0x11,  0x42,  0x03,  0x59,  0xf0,  0x0c,  0xf1,  0xc1,  0x3f,  0xe2,  0x07,  0xff,  0x10,  0x1f,  0xfc,  0xc3, 
  0x78,  0x84,  0x8f,  0xea,  0x31,  0x3f,  0x9e,  0x27,  0xfa,  0xe4,  0x9e,  0xfa,  0x33,  0x78,  0x9e,  0x01, 

  0x02,  0x0c,  0x18,  0x28,  0x50,  0x60,  0x41,  0x83,  0x0d,  0xc0,  0xe0,  0x03,  0x00,  0x10,  0x20,  0xa0, 
  0x40,  0x07,  0x80,  0x10,  0x1a,  0x3c,  0x83,  0x65,  0xb8,  0x0c,  0xee,  0xe0,  0xc3,  0x61,  0x70,  0x0f, 
  0xde,  0x71,  0x26,  0xb8,  0x81,  0x8f,  0xe6,  0xc1,  0x25,  0x78,  0x84,  0xfb,  0xe0,  0x1f,  0xf5,  0xc3, 
  0x5f,  0xb0,  0x8f,  0xde,  0xc1,  0x3e,  0xb4,  0x87,  0xff,  0x28,  0x1e,  0xe3,  0x93,  0x78,  0x8a,  0x4f, 
  0xeb,  0x19,  0x3f,  0x9b,  0xe7,  0xfa,  0xfc,  0x0e,  0x1e,  0x3c, 
};
static PROGMEM prog_uchar logo_trsi_sprpal[] = {

  0x0e,  0x7b,  0xf7,  0x5a,  0x65,  0x76,  0x94,  0x4e,  0xc0,  0x71,  0x31,  0x42,  0xe0,  0x6c,  0x26,  0x3d, 
  0xe7,  0x18,  0x20,  0x5c,  0x20,  0x3c,  0x00,  0x18,  0x63,  0x0c,  0x00,  0x04,  0x00,  0x00,  0x00,  0x80, 
};




#define LOGO_DESIRE_FRAMES 1
static void draw_logo_desire(int x, int y, byte anim, byte rot, byte jk = 0) {
  switch (anim) {
  case 0:
    GD.xsprite(x, y, 0, -64, 0, 4, rot, jk);
    GD.xsprite(x, y, 16, -64, 0, 6, rot, jk);
    GD.xsprite(x, y, -112, -48, 1, 4, rot, jk);
    GD.xsprite(x, y, -96, -48, 1, 6, rot, jk);
    GD.xsprite(x, y, -80, -48, 2, 4, rot, jk);
    GD.xsprite(x, y, -64, -48, 2, 6, rot, jk);
    GD.xsprite(x, y, -48, -48, 3, 4, rot, jk);
    GD.xsprite(x, y, -32, -48, 3, 6, rot, jk);
    GD.xsprite(x, y, -16, -48, 4, 4, rot, jk);
    GD.xsprite(x, y, 0, -48, 4, 6, rot, jk);
    GD.xsprite(x, y, 16, -48, 5, 4, rot, jk);
    GD.xsprite(x, y, 32, -48, 5, 6, rot, jk);
    GD.xsprite(x, y, 48, -48, 6, 4, rot, jk);
    GD.xsprite(x, y, 64, -48, 6, 6, rot, jk);
    GD.xsprite(x, y, -96, -32, 7, 4, rot, jk);
    GD.xsprite(x, y, -80, -32, 7, 6, rot, jk);
    GD.xsprite(x, y, -64, -32, 8, 4, rot, jk);
    GD.xsprite(x, y, -48, -32, 8, 6, rot, jk);
    GD.xsprite(x, y, -32, -32, 9, 4, rot, jk);
    GD.xsprite(x, y, -16, -32, 9, 6, rot, jk);
    GD.xsprite(x, y, 0, -32, 10, 4, rot, jk);
    GD.xsprite(x, y, 16, -32, 10, 6, rot, jk);
    GD.xsprite(x, y, 32, -32, 11, 4, rot, jk);
    GD.xsprite(x, y, 48, -32, 11, 6, rot, jk);
    GD.xsprite(x, y, 64, -32, 12, 4, rot, jk);
    GD.xsprite(x, y, -96, -16, 12, 6, rot, jk);
    GD.xsprite(x, y, -80, -16, 13, 4, rot, jk);
    GD.xsprite(x, y, -64, -16, 13, 6, rot, jk);
    GD.xsprite(x, y, -48, -16, 14, 4, rot, jk);
    GD.xsprite(x, y, -32, -16, 14, 6, rot, jk);
    GD.xsprite(x, y, -16, -16, 15, 4, rot, jk);
    GD.xsprite(x, y, 0, -16, 15, 6, rot, jk);
    GD.xsprite(x, y, 16, -16, 16, 4, rot, jk);
    GD.xsprite(x, y, 32, -16, 16, 6, rot, jk);
    GD.xsprite(x, y, 48, -16, 17, 4, rot, jk);
    GD.xsprite(x, y, 64, -16, 17, 6, rot, jk);
    GD.xsprite(x, y, 80, -16, 18, 4, rot, jk);
    GD.xsprite(x, y, -96, 0, 18, 6, rot, jk);
    GD.xsprite(x, y, -80, 0, 19, 4, rot, jk);
    GD.xsprite(x, y, -64, 0, 19, 6, rot, jk);
    GD.xsprite(x, y, -48, 0, 20, 4, rot, jk);
    GD.xsprite(x, y, -32, 0, 20, 6, rot, jk);
    GD.xsprite(x, y, -16, 0, 21, 4, rot, jk);
    GD.xsprite(x, y, 0, 0, 21, 6, rot, jk);
    GD.xsprite(x, y, 16, 0, 22, 4, rot, jk);
    GD.xsprite(x, y, 32, 0, 22, 6, rot, jk);
    GD.xsprite(x, y, 48, 0, 23, 4, rot, jk);
    GD.xsprite(x, y, 64, 0, 23, 6, rot, jk);
    GD.xsprite(x, y, 80, 0, 24, 4, rot, jk);
    GD.xsprite(x, y, -112, 16, 24, 6, rot, jk);
    GD.xsprite(x, y, -96, 16, 25, 4, rot, jk);
    GD.xsprite(x, y, -80, 16, 25, 6, rot, jk);
    GD.xsprite(x, y, -64, 16, 26, 4, rot, jk);
    GD.xsprite(x, y, -48, 16, 26, 6, rot, jk);
    GD.xsprite(x, y, -32, 16, 27, 4, rot, jk);
    GD.xsprite(x, y, -16, 16, 27, 6, rot, jk);
    GD.xsprite(x, y, 0, 16, 28, 4, rot, jk);
    GD.xsprite(x, y, 16, 16, 28, 6, rot, jk);
    GD.xsprite(x, y, 32, 16, 29, 4, rot, jk);
    GD.xsprite(x, y, 48, 16, 29, 6, rot, jk);
    GD.xsprite(x, y, 64, 16, 30, 4, rot, jk);
    GD.xsprite(x, y, 80, 16, 30, 6, rot, jk);
    GD.xsprite(x, y, -96, 32, 31, 4, rot, jk);
    GD.xsprite(x, y, -80, 32, 31, 6, rot, jk);
    GD.xsprite(x, y, -64, 32, 32, 4, rot, jk);
    GD.xsprite(x, y, -48, 32, 32, 6, rot, jk);
    GD.xsprite(x, y, -32, 32, 33, 4, rot, jk);
    GD.xsprite(x, y, -16, 32, 33, 6, rot, jk);
    GD.xsprite(x, y, 0, 32, 34, 4, rot, jk);
    GD.xsprite(x, y, 16, 32, 34, 6, rot, jk);
    GD.xsprite(x, y, 32, 32, 35, 4, rot, jk);
    GD.xsprite(x, y, 48, 32, 35, 6, rot, jk);
    GD.xsprite(x, y, 64, 32, 36, 4, rot, jk);
    GD.xsprite(x, y, 80, 32, 36, 6, rot, jk);
    break;
  }
}

static PROGMEM prog_uchar logo_desire_sprimg[] = {

  0xc9,  0x41,  0x6c,  0xfa,  0x0f,  0xe0,  0x41,  0x3e,  0xa4,  0x87,  0xfd,  0x48,  0x1e,  0xed,  0x63,  0x7b, 
  0xfc,  0x4f,  0xe2,  0x29,  0x3e,  0xad,  0x67,  0xfc,  0x6c,  0x9e,  0xab,  0x3c,  0x07,  0x20,  0xff,  0xf8, 
  0x83,  0x7d,  0x28,  0x09,  0xa1,  0xc1,  0xa6,  0x1c,  0x47,  0xa8,  0xe0,  0x16,  0x29,  0xe0,  0x87,  0xa0, 
  0x70,  0xe7,  0xd1,  0x83,  0x07,  0xf0,  0xc1,  0xbf,  0x49,  0x06,  0x07,  0x0f,  0xe1,  0xd3,  0x3f,  0xe6, 
  0x56,  0x16,  0x3e,  0xdb,  0x57,  0xf0,  0xcf,  0xb4,  0xe1,  0xbf,  0x9f,  0x0f,  0xfa,  0xe1,  0x3e,  0xfa, 
  0x27,  0xf8,  0x94,  0x9f,  0xe9,  0x73,  0x7f,  0x91,  0x2f,  0xfb,  0xd5,  0xbe,  0xfe,  0xb7,  0xf8,  0x8e, 
  0xdf,  0xeb,  0x07,  0xff,  0x98,  0x9f,  0xfa,  0xf3,  0x7e,  0xf9,  0x6f,  0xf8,  0x9d,  0x7f,  0xe8,  0x4f, 
  0xff,  0x95,  0xbf,  0x0b,  0xf4,  0xf0,  0xc1,  0x07,  0x88,  0x00,  0x08,  0x3f,  0x70,  0xc2,  0x07,  0x14, 
  0x20,  0x42,  0x69,  0xb0,  0x0a,  0xc8,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff, 
  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xdf,  0xf9,  0xa7,  0xff,  0xee, 
  0xbf,  0xff,  0xff,  0xdf,  0xf3,  0x47,  0xff,  0xdc,  0x5f,  0xff,  0x07,  0xff,  0xf2,  0x3f,  0x0d,  0x80, 
  0x47,  0x0f,  0x1e,  0x03,  0x8a,  0x07,  0x0f,  0x00,  0x02,  0x70,  0x08,  0x07,  0x18,  0x34,  0x20,  0x40, 
  0xf8,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff, 
  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff, 
  0xdf,  0x67,  0xf4,  0xff,  0x83,  0xd7,  0x5b,  0x08,  0xfe,  0xff,  0xf3,  0x7b,  0xfc,  0x04,  0xf4,  0xff, 
  0xdf,  0x3f,  0x04,  0x88,  0x00,  0x20,  0x22,  0x84,  0xd0,  0x0f,  0x03,  0xe0,  0x20,  0x82,  0x08,  0x11, 

  0x20,  0xc2,  0x10,  0x30,  0x12,  0x02,  0x74,  0xd0,  0x80,  0xf8,  0xbf,  0xf0,  0xff,  0xff,  0xff,  0x13, 
  0xfc,  0x43,  0xf8,  0xff,  0x8f,  0xa0,  0x31,  0x22,  0xe2,  0x87,  0x50,  0xf0,  0x21,  0x29,  0xb4,  0x47, 
  0x98,  0x90,  0x58,  0xe4,  0x63,  0x02,  0xfa,  0x00,  0xfd,  0xff,  0x63,  0x66,  0x80,  0xfc,  0xff,  0xf5, 
  0x3d,  0x00,  0x80,  0xff,  0xff,  0x31,  0xa3,  0x29,  0x04,  0x80,  0xe0,  0x13,  0x34,  0xbb,  0x44,  0x9f, 
  0x08,  0x5e,  0x01,  0x34,  0x2a,  0xf6,  0xc9,  0x10,  0x1c,  0x96,  0x03,  0x70,  0xd0,  0x88,  0x92,  0x21, 
  0x18,  0x3e,  0x04,  0xf4,  0x1e,  0xde,  0x47,  0xd8,  0x00,  0xe0,  0x1f,  0x83,  0x87,  0xaf,  0x88,  0xe0, 
  0x07,  0x00,  0x00,  0xf8,  0x3d,  0x02,  0x46,  0x00,  0xe8,  0x37,  0x41,  0x60,  0x68,  0x8c,  0x8b,  0xfe, 
  0xe5,  0xb9,  0xff,  0x0f,  0x91,  0xe0,  0x1f,  0xdc,  0x27,  0x78,  0xd8,  0xf4,  0x1c,  0xdd,  0xe7,  0x96, 
  0xa7,  0xe0,  0x1e,  0x77,  0x83,  0x7b,  0xd2,  0x5f,  0xf7,  0x9b,  0x7d,  0xfb,  0xef,  0xf1,  0x03,  0x5f, 
  0x29,  0x3c,  0xa6,  0x84,  0xe3,  0x7f,  0x2e,  0xae,  0x74,  0x88,  0x0f,  0xde,  0x9e,  0xae,  0x7b,  0xf8, 
  0xb9,  0xf7,  0xd1,  0xfb,  0xe9,  0xbf,  0x92,  0x9e,  0x04,  0xa9,  0x4b,  0xf4,  0x8a,  0x3e,  0x38,  0x08, 
  0x00,  0xfc,  0x0f,  0x08,  0xf0,  0x21,  0x05,  0x80,  0x3e,  0x11,  0x60,  0x01,  0xfe,  0xf5,  0x20,  0x1e, 
  0xe2,  0xc3,  0x7a,  0xc4,  0x8f,  0xe6,  0xb1,  0x3e,  0xbe,  0x27,  0xfe,  0x14,  0x9e,  0xe6,  0x33,  0x7a, 
  0xd6,  0xcf,  0xe5,  0xf9,  0xbe,  0xb0,  0x17,  0xff,  0x32,  0x7e,  0xce,  0x2f,  0xfd,  0x75,  0xfb,  0x86, 
  0xff,  0xe3,  0xee,  0x3f,  0x09,  0xf4,  0x7f,  0xf2,  0xa9,  0xbf,  0x41,  0xfc,  0xf8,  0x23,  0x64,  0xc4, 
  0x0c,  0x18,  0x7d,  0x72,  0x30,  0x04,  0x48,  0x88,  0x00,  0x82,  0x73,  0xd0,  0x0c,  0x46,  0x01,  0x22, 

  0x78,  0x24,  0xac,  0x90,  0x00,  0x1f,  0xd0,  0xe8,  0x98,  0x04,  0x21,  0x62,  0x52,  0x84,  0xa7,  0x10, 
  0x1c,  0x0c,  0x43,  0xd0,  0xdf,  0x4f,  0x1b,  0x91,  0x2b,  0xfb,  0x67,  0x83,  0x58,  0x19,  0x90,  0x32, 
  0x34,  0x7c,  0xf4,  0x1e,  0x53,  0x54,  0xc6,  0xcc,  0x19,  0x01,  0x7a,  0x35,  0x80,  0x87,  0x0c,  0x19, 
  0x70,  0x40,  0x44,  0x1f,  0x00,  0x83,  0x0f,  0xa0,  0xdd,  0x11,  0x10,  0x13,  0x45,  0x80,  0x4c,  0x1a, 
  0xc1,  0x0b,  0xe0,  0x61,  0xc8,  0x0d,  0xc1,  0x25,  0x7c,  0x01,  0x0c,  0xc1,  0x19,  0x31,  0xfc,  0xde, 
  0xcc,  0x92,  0x10,  0x3e,  0x32,  0x65,  0xf8,  0x01,  0x10,  0x82,  0xc3,  0x77,  0x2a,  0x08,  0xdf,  0x21, 
  0x28,  0x7c,  0x05,  0x9f,  0x1a,  0x14,  0xfc,  0x83,  0x77,  0x84,  0x0a,  0xff,  0xf1,  0x33,  0x04,  0x45, 
  0xff,  0xe8,  0x91,  0x2e,  0x53,  0x42,  0x6d,  0x80,  0x90,  0x93,  0x02,  0x66,  0x90,  0x88,  0x80,  0x83, 
  0x21,  0x84,  0x05,  0xeb,  0xe0,  0x89,  0x00,  0x43,  0x00,  0x68,  0x8e,  0xfe,  0x3d,  0x78,  0x0c,  0x40, 
  0x9e,  0x42,  0x50,  0x48,  0x08,  0xfe,  0x1e,  0x63,  0x24,  0x22,  0x0c,  0x88,  0xc2,  0x7f,  0xf0,  0x0a, 
  0x0e,  0xe1,  0x3f,  0x24,  0x6c,  0x4a,  0x44,  0x08,  0xc3,  0x7f,  0xf4,  0xcc,  0x94,  0x90,  0x4a,  0xc5, 
  0x49,  0xcc,  0x96,  0xf1,  0x67,  0x74,  0x40,  0x2a,  0x88,  0x00,  0x10,  0x06,  0x80,  0xe8,  0x13,  0x7c, 
  0x20,  0x04,  0x9f,  0x53,  0x08,  0x28,  0x00,  0x5c,  0xc8,  0xe0,  0x87,  0x10,  0x03,  0xa6,  0x00,  0x50, 
  0xfc,  0x3d,  0xa5,  0x0f,  0x04,  0x98,  0x10,  0x05,  0xff,  0xf8,  0xf9,  0x38,  0x82,  0x43,  0xf1,  0x0f, 
  0x5e,  0xe5,  0x23,  0x00,  0x04,  0x4f,  0x4f,  0xf9,  0xcf,  0x43,  0xf0,  0xf0,  0xe4,  0x3f,  0x0e,  0x84, 
  0x80,  0x00,  0x50,  0xbc,  0x20,  0x01,  0xd8,  0x83,  0x0f,  0x1b,  0xfe,  0x02,  0x82,  0x04,  0x3e,  0x7d, 

  0x86,  0x83,  0xf3,  0x39,  0x20,  0xce,  0xc7,  0xf4,  0x6a,  0x0e,  0xc1,  0x3b,  0x5c,  0x04,  0xa7,  0x3a, 
  0x99,  0x3d,  0xc3,  0x57,  0x70,  0x6c,  0x1c,  0xdb,  0x33,  0x7a,  0x84,  0x9f,  0x46,  0x79,  0x02,  0x11, 
  0x06,  0x8f,  0x6e,  0x11,  0xbe,  0xe0,  0xb7,  0x8a,  0xce,  0x59,  0x13,  0xa2,  0x67,  0xf0,  0x0b,  0x4e, 
  0x1e,  0xb7,  0x41,  0xf0,  0x01,  0x1f,  0x5c,  0x82,  0x47,  0xf8,  0x2d,  0xae,  0xe1,  0xb3,  0x44,  0x8c, 
  0x80,  0x62,  0x01,  0xbe,  0x57,  0x7c,  0xc7,  0xe7,  0x15,  0xbc,  0x2e,  0x22,  0x24,  0x40,  0xe9,  0xeb, 
  0x79,  0x4e,  0xdf,  0xf4,  0x59,  0xbc,  0x96,  0x40,  0x7d,  0x18,  0x9f,  0xf5,  0x31,  0x56,  0x6c,  0x0d, 
  0x00,  0x10,  0xf6,  0xc4,  0xc5,  0x48,  0x0a,  0xf5,  0xe9,  0x35,  0x83,  0x0f,  0x7e,  0xc1,  0x15,  0xfe, 
  0x19,  0x1a,  0x0a,  0xc1,  0xaf,  0x5e,  0xfc,  0x8f,  0xf0,  0x80,  0x1e,  0x01,  0x82,  0xdf,  0x74,  0x96, 
  0xfe,  0x67,  0xf0,  0x44,  0x8c,  0x38,  0x70,  0xee,  0xc9,  0x16,  0x78,  0x2e,  0x86,  0xa1,  0x67,  0x4f, 
  0x9e,  0x3d,  0x7a,  0xd8,  0x01,  0x1b,  0xa9,  0x30,  0x54,  0xad,  0x10,  0x10,  0x5c,  0x83,  0x47,  0x8a, 
  0xc0,  0x0f,  0x1f,  0x32,  0x20,  0xc0,  0x00,  0x01,  0x2c,  0x8e,  0xea,  0x9d,  0x23,  0xf1,  0x5f,  0x48, 
  0x0c,  0x18,  0x30,  0x06,  0x88,  0x00,  0x18,  0xbc,  0x11,  0xae,  0x04,  0x04,  0x18,  0x10,  0x60,  0x44, 
  0x18,  0x20,  0xbf,  0xc4,  0xd8,  0x40,  0x8f,  0x71,  0x24,  0xe4,  0x04,  0x84,  0x01,  0xe2,  0x27,  0x46, 
  0xef,  0x1b,  0x09,  0xa1,  0x20,  0x20,  0x7c,  0x1b,  0xef,  0x02,  0x42,  0xf0,  0x44,  0xbf,  0x10,  0x9a, 
  0x46,  0x71,  0xf9,  0x0f,  0x00,  0x82,  0x5f,  0x70,  0x0e,  0x9e,  0x1e,  0x8e,  0x42,  0xf8,  0x0f,  0x3e, 
  0xe1,  0x3f,  0xfc,  0x78,  0x40,  0xbf,  0x23,  0xe2,  0x7f,  0xfc,  0xf8,  0x03,  0xd1,  0x27,  0xfe,  0x0d, 

  0x0f,  0xf8,  0x27,  0x02,  0x50,  0x70,  0x85,  0xb8,  0xb3,  0xea,  0xf0,  0xb2,  0x5a,  0x01,  0xf1,  0xd2, 
  0x43,  0x93,  0x68,  0x8a,  0xc1,  0xa5,  0x55,  0x9c,  0x49,  0xc0,  0x01,  0x00,  0x40,  0xb8,  0x46,  0x78, 
  0x23,  0x30,  0xd4,  0x03,  0xff,  0x88,  0x01,  0x21,  0x0e,  0x1e,  0xe1,  0x3a,  0x16,  0x20,  0x18,  0x0c, 
  0xfe,  0x83,  0x03,  0xa0,  0xbe,  0x11,  0x5c,  0x0e,  0xc0,  0x93,  0x44,  0x0c,  0x09,  0x71,  0x70,  0xdf, 
  0x9e,  0xfe,  0x21,  0x35,  0x8b,  0xe8,  0x19,  0x5c,  0x82,  0x57,  0x70,  0x40,  0x94,  0x43,  0x82,  0x63, 
  0xf8,  0x09,  0x5e,  0xc1,  0x25,  0xf8,  0x17,  0xc7,  0x07,  0x1d,  0x7e,  0xcb,  0xe5,  0x45,  0x8a,  0x10, 
  0xd5,  0xb3,  0x70,  0x14,  0x81,  0xe2,  0x34,  0x0d,  0x11,  0xe0,  0x0f,  0xbe,  0xc1,  0xe9,  0x5e,  0x21, 
  0x08,  0x7f,  0x01,  0xb4,  0x5b,  0x04,  0xff,  0xe0,  0xe7,  0xa1,  0x16,  0x20,  0x08,  0xff,  0xcb,  0xe2, 
  0x8d,  0x36,  0x8e,  0x98,  0xb5,  0x19,  0x82,  0x7b,  0xf0,  0x7a,  0x07,  0x10,  0xca,  0x67,  0x40,  0x05, 
  0x5f,  0x1e,  0x21,  0x07,  0xff,  0xe0,  0x05,  0x39,  0x00,  0x84,  0xff,  0xc6,  0x11,  0x2e,  0x21,  0x85, 
  0xff,  0xe8,  0x1d,  0x1c,  0x20,  0x41,  0x8c,  0xff,  0xe1,  0x37,  0x4b,  0x6d,  0xcc,  0xe8,  0x0d,  0x29, 
  0x00,  0x04,  0xcf,  0x03,  0x15,  0xbc,  0xd0,  0x07,  0xff,  0x93,  0x99,  0xa8,  0x1e,  0x13,  0xc4,  0xe3, 
  0x59,  0x2e,  0x83,  0x57,  0xf2,  0x04,  0x84,  0x08,  0x61,  0x72,  0x8f,  0xbf,  0xe5,  0x3f,  0xf9,  0x64, 
  0xff,  0xe0,  0xdf,  0xc0,  0xc3,  0x7f,  0xf5,  0x0f,  0x17,  0x33,  0x60,  0x49,  0x22,  0xd8,  0x1e,  0x9e, 
  0xd0,  0x83,  0x87,  0x70,  0x00,  0xa7,  0x00,  0xc2,  0xcf,  0x19,  0xbc,  0xca,  0x45,  0xf0,  0x5c,  0x0e, 
  0x8b,  0xe9,  0x17,  0x04,  0xaf,  0xf0,  0x15,  0x42,  0xa3,  0x07,  0xa4,  0xf0,  0xf9,  0x12,  0x42,  0xf0, 

  0x92,  0x0c,  0xdf,  0xfe,  0x83,  0xcf,  0xef,  0x0c,  0xff,  0x21,  0x2c,  0x7e,  0x44,  0xcf,  0xf0,  0xbb, 
  0x87,  0xe2,  0x77,  0xf4,  0xb9,  0x4d,  0xe1,  0x3f,  0xdc,  0x14,  0x8f,  0xde,  0x01,  0x69,  0x57,  0x46, 
  0xd3,  0x8e,  0x01,  0x1e,  0xc1,  0x98,  0x48,  0xdf,  0xd1,  0xb8,  0x52,  0x22,  0xa8,  0x04,  0xa7,  0x32, 
  0x4f,  0xc5,  0x90,  0xe2,  0x39,  0x3d,  0xe3,  0x4f,  0x71,  0x6c,  0x9e,  0xd9,  0xb7,  0x6d,  0x66,  0xff, 
  0xf8,  0x1d,  0x9c,  0x83,  0x5f,  0x70,  0x69,  0x13,  0x47,  0xb3,  0x3e,  0x06,  0xb8,  0xff,  0xb5,  0xaf, 
  0x83,  0x5f,  0x08,  0xf0,  0xe4,  0x29,  0x84,  0x07,  0xdf,  0xe8,  0x1f,  0xbd,  0x3c,  0x86,  0x2f,  0xf8, 
  0xe0,  0x03,  0xe8,  0x35,  0x8c,  0x3e,  0xf8,  0x03,  0x48,  0xfc,  0x47,  0x8f,  0x3f,  0x40,  0x7f,  0xc7, 
  0xe0,  0x1b,  0x40,  0xb2,  0x67,  0xf0,  0x87,  0x1f,  0x20,  0xe3,  0x7f,  0x70,  0x7e,  0x19,  0xe1,  0x3f, 
  0x7c,  0x05,  0xc0,  0xf2,  0x10,  0xfd,  0xc3,  0x77,  0xf6,  0x08,  0xff,  0xe1,  0xbb,  0xfa,  0xc7,  0xbf, 
  0xfc,  0x1f,  0xfe,  0xf2,  0x7f,  0xf8,  0xae,  0x1d,  0xb1,  0x21,  0x6d,  0xcd,  0xe5,  0x1f,  0x18,  0x3c, 
  0xd1,  0x07,  0xaf,  0xa3,  0xf3,  0x03,  0xc2,  0x47,  0x13,  0x68,  0x5a,  0xe1,  0xe7,  0x5a,  0xd4,  0xa6, 
  0xe8,  0x85,  0x30,  0x00,  0x42,  0x0a,  0x10,  0x90,  0x20,  0x00,  0x58,  0x02,  0x00,  0x01,  0x25,  0x8a, 
  0xf0,  0x0b,  0x01,  0x72,  0x00,  0x00,  0x0c,  0x78,  0x58,  0xc0,  0xff,  0x77,  0x90,  0x21,  0x01,  0x0e, 
  0x10,  0xc1,  0x21,  0xf8,  0x3f,  0x8f,  0x10,  0x00,  0x28,  0xfc,  0x7f,  0xa0,  0xf0,  0x1f,  0x50,  0x20, 
  0xc4,  0x80,  0xf8,  0x55,  0x24,  0x42,  0x4c,  0xe6,  0x40,  0x8c,  0xc8,  0x53,  0xf6,  0x09,  0x1a,  0x90, 
  0xa2,  0xc5,  0x2d,  0xc0,  0x84,  0xc9,  0x53,  0xf1,  0x8c,  0xfe,  0x10,  0x82,  0x4f,  0x75,  0x49,  0x94, 

  0x00,  0x30,  0x45,  0xcf,  0xad,  0x11,  0xff,  0x82,  0x07,  0xa2,  0xe8,  0xb9,  0x0f,  0xfe,  0x43,  0x86, 
  0x8d,  0x1f,  0xe1,  0x3f,  0x7c,  0xdd,  0x8f,  0x2b,  0x81,  0x11,  0x13,  0xa0,  0xa2,  0x52,  0x0c,  0x83, 
  0x17,  0x42,  0x4c,  0x41,  0x22,  0xda,  0x85,  0x4f,  0xf4,  0x18,  0x11,  0x85,  0x9f,  0xe0,  0x1b,  0x3e, 
  0x01,  0x65,  0xb7,  0xf0,  0x8d,  0x30,  0x7c,  0xe6,  0xdf,  0xf0,  0xf1,  0x34,  0xe0,  0x07,  0xcf,  0xfa, 
  0x30,  0x5d,  0x00,  0x04,  0xcf,  0xf0,  0x35,  0x0e,  0xc7,  0x77,  0xf2,  0x0a,  0x5f,  0xb3,  0x20,  0x84, 
  0x44,  0xbf,  0x85,  0x11,  0x61,  0xa2,  0x47,  0xf8,  0x81,  0x10,  0x42,  0xa3,  0x5f,  0xf2,  0x8c,  0xd0, 
  0xf1,  0x3f,  0x68,  0x45,  0xff,  0xe8,  0x15,  0x42,  0xa2,  0xff,  0x73,  0x4a,  0x85,  0xc1,  0x3f,  0x7f, 
  0x5f,  0x89,  0xe8,  0x37,  0x9c,  0x72,  0x14,  0x00,  0xff,  0x01,  0xc0,  0xa3,  0x27,  0x4f,  0x95,  0x22, 
  0xc0,  0x0e,  0x8b,  0xe0,  0xed,  0x21,  0xf8,  0x87,  0xff,  0xf0,  0x1f,  0xfe,  0x63,  0x71,  0xfc,  0x8c, 
  0x0e,  0xf1,  0x3f,  0x29,  0x24,  0xcf,  0x10,  0x98,  0x32,  0x82,  0x71,  0x10,  0x0c,  0x4e,  0x19,  0x63, 
  0x13,  0x44,  0xc2,  0x54,  0x95,  0xbe,  0xc2,  0xa1,  0x87,  0x0c,  0x9e,  0xfd,  0x93,  0x7f,  0xf1,  0x0f, 
  0xfe,  0xc5,  0xbf,  0xfc,  0x64,  0xff,  0xea,  0x8f,  0xa9,  0x78,  0x60,  0x0c,  0x90,  0x01,  0x11,  0x43, 
  0xf8,  0xfa,  0x01,  0x18,  0x82,  0x55,  0x88,  0x08,  0x7e,  0x98,  0x82,  0x57,  0x84,  0x0e,  0x5f,  0x1e, 
  0xc3,  0x4f,  0x8c,  0x0c,  0x84,  0xd5,  0x2b,  0xc0,  0xc0,  0x0f,  0xc0,  0xc1,  0xff,  0x37,  0x04,  0xff, 
  0x08,  0x1e,  0xfe,  0x63,  0x78,  0xf8,  0x0f,  0xff,  0xd1,  0x3d,  0xfe,  0x67,  0x54,  0xf0,  0xe9,  0x3f, 
  0xfb,  0x67,  0xff,  0xf0,  0x1f,  0xfe,  0xa3,  0x7f,  0x05,  0x8f,  0xff,  0xc9,  0x3f,  0x04,  0x79,  0x6e, 

  0xe0,  0xc1,  0x03,  0x7c,  0x00,  0xc8,  0x1e,  0x21,  0x3c,  0xf8,  0xe7,  0xff,  0xf0,  0x5f,  0xfc,  0x53, 
  0x02,  0xc4,  0x00,  0x80,  0x00,  0x11,  0xa4,  0x8b,  0x08,  0x00,  0x22,  0x40,  0x80,  0xc1,  0x30,  0xf8, 
  0x07,  0x8b,  0xe0,  0x1a,  0xfe,  0x82,  0x5b,  0xf8,  0x44,  0x14,  0xbe,  0xc2,  0x76,  0xa0,  0x8a,  0x7f, 
  0xc1,  0x35,  0xfc,  0x07,  0x57,  0x80,  0xc1,  0x3f,  0xc1,  0x22,  0x0c,  0xdf,  0xd9,  0x33,  0xbb,  0x00, 
  0x0c,  0xbe,  0xd9,  0x2d,  0xfa,  0x26,  0xef,  0xe0,  0x7f,  0x19,  0x20,  0x04,  0xbf,  0xfc,  0x1b,  0xfe, 
  0xc3,  0x7f,  0xf2,  0x81,  0x10,  0xff,  0x2b,  0x68,  0xf8,  0xaf,  0xbe,  0xe1,  0xbf,  0x7e,  0x7b,  0xf4, 
  0xe8,  0xc1,  0x93,  0x27,  0x8f,  0xdf,  0x29,  0x00,  0x6c,  0x20,  0x80,  0x00,  0x00,  0x00,  0x82,  0x04, 
  0x09,  0xbd,  0x7f,  0x8f,  0x08,  0x10,  0x40,  0x08,  0x10,  0x01,  0x00,  0x03,  0x46,  0x0c,  0x98,  0x30, 
  0x61,  0x0c,  0x5f,  0x88,  0x11,  0x43,  0x0e,  0x80,  0x08,  0x31,  0x22,  0x08,  0x1e,  0x98,  0xa2,  0x17, 
  0x22,  0xc8,  0x01,  0x02,  0x52,  0xf0,  0x40,  0x98,  0x1e,  0x30,  0x21,  0x0c,  0xde,  0xb3,  0x21,  0x00, 
  0x22,  0x1c,  0x0e,  0x98,  0x42,  0x40,  0xf8,  0x86,  0x08,  0x29,  0x40,  0x96,  0xc0,  0xf0,  0x81,  0x28, 
  0x7d,  0xf5,  0x8e,  0xf5,  0x02,  0x21,  0xfc,  0x06,  0xd7,  0xf0,  0x39,  0x3c,  0xc2,  0x7f,  0xf0,  0x49, 
  0xde,  0xe1,  0x3f,  0xfc,  0x47,  0xff,  0xf0,  0x1f,  0xfd,  0xc3,  0xdf,  0xad,  0x0c,  0xff,  0xd9,  0x79, 
  0x35,  0x84,  0xff,  0xe6,  0x58,  0x2c,  0xd3,  0x3f,  0xc2,  0x2e,  0xb1,  0xb2,  0x00,  0x6e,  0xcb,  0xbe, 
  0xb9,  0xdd,  0x01,  0x74,  0x07,  0x4f,  0xf8,  0x03,  0x40,  0x70,  0x6b,  0x8f,  0xe1,  0x1b,  0x41,  0x17, 
  0xa8,  0x1f,  0x08,  0x3b,  0x44,  0xf8,  0x00,  0x14,  0x00,  0xc0,  0x07,  0x80,  0xfb,  0x9a,  0x1c,  0x21, 

  0x85,  0x80,  0x39,  0x78,  0xee,  0x83,  0x47,  0x08,  0x00,  0xbf,  0x38,  0xae,  0x7b,  0xf8,  0xbd,  0xef, 
  0xe1,  0x3f,  0xfa,  0x87,  0xff,  0xec,  0x5f,  0x3c,  0xb3,  0xcb,  0x42,  0x2c,  0xbe,  0xc9,  0xbf,  0xfc, 
  0x56,  0xff,  0xea,  0x51,  0x3e,  0xf1,  0xa3,  0x4f,  0xbf,  0x8d,  0x01,  0xc0,  0xbe,  0x08,  0x1f,  0x21, 
  0xa0,  0x79,  0x16,  0x94,  0xf0,  0xdb,  0x3e,  0x9e,  0xe1,  0x08,  0xee,  0x7e,  0xc1,  0x07,  0xa1,  0x07, 
  0xff,  0xf0,  0x3b,  0x04,  0xa0,  0x66,  0x1d,  0x30,  0x01,  0x7a,  0x0c,  0x9e,  0x6f,  0xa1,  0x44,  0x84, 
  0x60,  0x40,  0x4d,  0x01,  0x60,  0xf8,  0x42,  0xdf,  0x1c,  0xe0,  0x77,  0x86,  0xf7,  0x00,  0x21,  0x52, 
  0x86,  0x2f,  0x84,  0xab,  0xa0,  0x79,  0xc6,  0xa4,  0xcc,  0x10,  0x3f,  0x11,  0x46,  0x8f,  0xee,  0x1c, 
  0xbf,  0x3d,  0x06,  0xef,  0xaf,  0x92,  0x0a,  0x20,  0x24,  0xaf,  0xf0,  0x31,  0x1b,  0x42,  0x68,  0xf0, 
  0x8d,  0xde,  0x23,  0x09,  0x40,  0xf8,  0x3d,  0x53,  0xe5,  0x2b,  0x78,  0xf5,  0x8d,  0xe0,  0x5d,  0xbf, 
  0xe3,  0xdf,  0x7c,  0x0b,  0x5e,  0xe1,  0x2f,  0x01,  0x47,  0xaf,  0x0b,  0x11,  0xfe,  0x12,  0x43,  0xfa, 
  0x4f,  0x9e,  0x2d,  0x29,  0x7f,  0x85,  0xef,  0xfe,  0x18,  0x7d,  0xbb,  0x57,  0xf0,  0x03,  0xe8,  0x3f, 
  0x00,  0x78,  0x0a,  0x00,  0x1e,  0x6a,  0x43,  0x00,  0x0d,  0x9e,  0x9f,  0x12,  0x00,  0xc0,  0x00,  0x0a, 
  0x20,  0x7c,  0x7a,  0x3c,  0x10,  0x18,  0x30,  0x06,  0x50,  0x0c,  0xc1,  0x1b,  0x11,  0x06,  0x84,  0x18, 
  0x03,  0x0c,  0x82,  0xf0,  0xb5,  0x23,  0x10,  0x5c,  0xc7,  0x80,  0x1c,  0xbc,  0xc3,  0xd7,  0x1a,  0x08, 
  0xd0,  0xe1,  0x07,  0x11,  0x82,  0x7f,  0x1f,  0xfc,  0x83,  0x7f,  0xfe,  0x08,  0xfe,  0xc1,  0x3f,  0xfc, 
  0x47,  0xdf,  0x64,  0x10,  0xff,  0x93,  0x7f,  0xf2,  0x0f,  0xdf,  0x03,  0x2c,  0xfc,  0x67,  0xff,  0xf8, 

  0x3f,  0xa2,  0xd2,  0xff,  0x98,  0x38,  0x87,  0xc1,  0x27,  0x80,  0x07,  0x29,  0x0f,  0x01,  0x60,  0x68, 
  0x04,  0x04,  0x4f,  0x01,  0xc2,  0x53,  0x14,  0x08,  0xeb,  0x11,  0x7f,  0x5a,  0x4d,  0xac,  0x58,  0x1e, 
  0xef,  0xc3,  0x7f,  0x78,  0x4f,  0xeb,  0x19,  0x3f,  0x9b,  0xe7,  0xfa,  0xfc,  0x5e,  0xf8,  0x4b,  0x78, 
  0x99,  0x40,  0x1f,  0x00,  0xcb,  0x7f,  0x15,  0xc2,  0x1f,  0x00,  0xaa,  0x7f,  0xf0,  0x6f,  0xf0,  0xe1, 
  0xbf,  0xe5,  0xc7,  0xc0,  0xf6,  0x1f,  0xfc,  0x7b,  0x7c,  0x5f,  0x06,  0x0f,  0x3f,  0x00,  0x64,  0x66, 
  0xf8,  0x07,  0xf1,  0x37,  0x40,  0x04,  0x1f,  0x02,  0xc0,  0x07,  0x1f,  0x08,  0x13,  0x60,  0x62,  0x40, 
  0x08,  0xff,  0xe1,  0x73,  0x63,  0x45,  0xc7,  0xe0,  0x03,  0x3e,  0x7c,  0x2e,  0xca,  0xf8,  0x90,  0x20, 
  0xa2,  0xef,  0x4a,  0x48,  0x00,  0xe1,  0x23,  0xfe,  0xe1,  0x47,  0x18,  0x0c,  0x02,  0x64,  0x72,  0x84, 
  0x8f,  0x1f,  0x7d,  0xa0,  0x0a,  0xce,  0xe1,  0x27,  0x38,  0x81,  0x0f,  0xce,  0xe1,  0x23,  0x80,  0x01, 
  0x04,  0x10,  0x80,  0x01,  0x05,  0x10,  0xf0,  0xc1,  0x26,  0x64,  0x78,  0xf6,  0x7c,  0x04,  0x02,  0xa4, 
  0xe7,  0xcb,  0x87,  0x39,  0x00,  0x61,  0x0a,  0x00,  0x81,  0x3b,  0x18,  0x22,  0x46,  0x1c,  0xd0,  0x02, 
  0xac,  0xe7,  0x10,  0x10,  0xc0,  0xc3,  0x63,  0xfc,  0x08,  0xee,  0x71,  0x19,  0x7f,  0x04,  0x4f,  0x30, 
  0xbd,  0x10,  0x11,  0x22,  0x48,  0xff,  0xd3,  0x63,  0xb7,  0x6c,  0x51,  0x10,  0x82,  0x27,  0xa2,  0xe0, 
  0xe9,  0x21,  0x68,  0x75,  0x84,  0x56,  0x01,  0x00,  0x13,  0x22,  0x4c,  0xc1,  0x37,  0x42,  0x01,  0x80, 
  0x80,  0x08,  0x73,  0xf8,  0x69,  0x1f,  0x3d,  0x1a,  0x40,  0x70,  0xc0,  0x1c,  0xbd,  0x9a,  0xd3,  0x84, 
  0x0c,  0x1f,  0x65,  0x23,  0xfa,  0xb4,  0xef,  0xf0,  0x09,  0x19,  0x10,  0xe4,  0xfe,  0x99,  0x3a,  0xfa, 

  0x77,  0xfc,  0x0c,  0x0e,  0x80,  0x86,  0x7b,  0x2f,  0x02,  0x1c,  0x00,  0xfa,  0x7b,  0x70,  0x0b,  0xf6, 
  0xff,  0xbf,  0xff,  0x07,  0xf8,  0x8e,  0x53,  0x2c,  0x03,  0x7e,  0xd0,  0x0c,  0xe0,  0x81,  0x76,  0x7c, 
  0x20,  0x86,  0x1c,  0x90,  0x20,  0x07,  0x40,  0xf8,  0x90,  0x02,  0x40,  0x5f,  0x1f,  0x5a,  0x8b,  0xf2, 
  0x53,  0x20,  0xec,  0x1f,  0x9e,  0x9e,  0x23,  0x24,  0x88,  0x80,  0xde,  0xd7,  0xf0,  0x08,  0x7e,  0x21, 
  0xe0,  0x05,  0x06,  0x4c,  0x8c,  0xfd,  0x21,  0xfc,  0x84,  0xc0,  0x08,  0x09,  0x01,  0xe1,  0xf0,  0x08, 
  0x5f,  0x80,  0xc2,  0x67,  0xb8,  0xec,  0x1f,  0x18,  0xc2,  0x2f,  0x00,  0x48,  0xff,  0x63,  0x01,  0x02, 
  0xc0,  0x78,  0x2e,  0xa2,  0x27,  0xfc,  0xf0,  0xd1,  0xff,  0x20,  0x45,  0x85,  0xfe,  0x19,  0x5c,  0xb6, 
  0x63,  0xe6,  0xe8,  0x3f,  0xe1,  0xb7,  0x7f,  0x02,  0xea,  0x81,  0x3d,  0x1d,  0x40,  0xb0,  0x08,  0xce, 
  0x21,  0xb4,  0x48,  0xa2,  0x0f,  0x2e,  0x81,  0xe6,  0x36,  0xbc,  0x88,  0x97,  0xe1,  0x29,  0x00,  0x04, 
  0xa6,  0x77,  0xd4,  0x3f,  0xb6,  0x67,  0x00,  0x0a,  0x5c,  0xfd,  0xf3,  0x5a,  0x06,  0xfc,  0x2a,  0xd9, 
  0x2e,  0x47,  0x7e,  0x15,  0xf4,  0x34,  0x08,  0xe0,  0x37,  0x81,  0x1f,  0x19,  0xb0,  0x30,  0x79,  0x0c, 
  0x10,  0xff,  0x21,  0x69,  0xa0,  0x1f,  0x10,  0x7d,  0xf3,  0x5f,  0xbf,  0xcf,  0xe0,  0xd7,  0x17,  0x47, 
  0x50,  0xf0,  0x1f,  0xc7,  0xe1,  0xff,  0x31,  0x04,  0xe7,  0x57,  0x11,  0xfd,  0xa2,  0x41,  0x80,  0x4e, 
  0xde,  0x05,  0x03,  0x50,  0x70,  0xfd,  0x15,  0x10,  0xd2,  0x07,  0xfe,  0x05,  0x11,  0xfe,  0xe7,  0x07, 
  0xa4,  0xe0,  0x51,  0x0d,  0xe2,  0x67,  0xf8,  0x47,  0x94,  0x3d,  0x00,  0x25,  0x8f,  0x00,  0xe6,  0x09, 
  0x52,  0x85,  0x18,  0x08,  0xc1,  0xff,  0x7f,  0x77,  0x0f,  0xcf,  0x1e,  0x02,  0xb0,  0xc7,  0xa0,  0x70, 

  0x1b,  0xf1,  0xff,  0x4f,  0x8c,  0x18,  0x02,  0x00,  0xc6,  0x20,  0x11,  0x14,  0xd0,  0xe3,  0xcf,  0x8e, 
  0x98,  0x10,  0x62,  0x40,  0x80,  0x11,  0x61,  0x90,  0xc0,  0x8f,  0x18,  0x7d,  0x00,  0xfc,  0x99,  0x08, 
  0x10,  0x20,  0x0c,  0x22,  0xc1,  0x63,  0x00,  0x05,  0x97,  0x80,  0x98,  0x35,  0x83,  0x53,  0xb0,  0x82, 
  0xff,  0x21,  0x02,  0x64,  0x7a,  0x9c,  0x88,  0xdd,  0xfa,  0x5f,  0x65,  0x2f,  0x88,  0xc1,  0xe3,  0x72, 
  0xbe,  0x89,  0x10,  0x94,  0xbf,  0x16,  0xe2,  0xed,  0x0c,  0xff,  0xfe,  0x57,  0xe0,  0x93,  0x0e,  0x4e, 
  0x10,  0xc2,  0xef,  0x3b,  0xf9,  0x86,  0xe1,  0xab,  0x78,  0x04,  0xf7,  0xf0,  0x55,  0x1e,  0xc3,  0x7f, 
  0xfa,  0xba,  0x96,  0xc1,  0x3d,  0x7e,  0x56,  0x40,  0x48,  0xfd,  0x1e,  0x80,  0xc7,  0xf3,  0x01,  0x69, 
  0xd8,  0xf7,  0xbf,  0x80,  0x5f,  0x2d,  0xfa,  0x62,  0x40,  0x0f,  0xca,  0x90,  0x7e,  0x46,  0x80,  0x0f, 
  0x18,  0x88,  0xd3,  0xc1,  0xff,  0x0c,  0xe0,  0x9f,  0x31,  0x04,  0x04,  0xcc,  0xa0,  0x16,  0x0c,  0xfe, 
  0x41,  0x10,  0x08,  0x18,  0x18,  0x83,  0x83,  0x47,  0x8c,  0x01,  0x78,  0x7d,  0x5d,  0x08,  0x4f,  0x1e, 
  0x3d,  0xae,  0x8d,  0x00,  0x1c,  0xfc,  0x43,  0x6c,  0xf0,  0x8f,  0x00,  0xfb,  0x31,  0xfc,  0x47,  0xa7, 
  0x80,  0x1c,  0x7c,  0xa3,  0x46,  0xfc,  0x0a,  0x94,  0xe1,  0x3f,  0xc0,  0x06,  0xf1,  0xc0,  0x1b,  0xdc, 
  0x43,  0x78,  0x80,  0x0f,  0x2a,  0x57,  0x21,  0xc0,  0x85,  0xa5,  0x71,  0x11,  0xc0,  0x7e,  0xc5,  0x10, 
  0x7e,  0x9b,  0x9e,  0xfb,  0x8d,  0xe7,  0x00,  0xe4,  0x09,  0x33,  0x26,  0xc4,  0x01,  0x33,  0x40,  0x7a, 
  0x0a,  0x1a,  0x01,  0xd1,  0x73,  0xab,  0xf0,  0x88,  0x29,  0x60,  0x04,  0x84,  0x9a,  0xe9,  0x3f,  0x42, 
  0x8d,  0x88,  0x80,  0x19,  0x8d,  0x83,  0x6f,  0xec,  0x88,  0xbb,  0xe1,  0xd9,  0x73,  0x6a,  0x0c,  0x08, 

  0x2b,  0x3c,  0x08,  0x64,  0xa6,  0x90,  0x11,  0x60,  0x42,  0xd3,  0xbe,  0x3f,  0x22,  0x85,  0xe4,  0x94, 
  0x87,  0xfd,  0x80,  0x1f,  0xfd,  0xe3,  0x78,  0x82,  0x4f,  0xea,  0x29,  0x3f,  0x9d,  0x67,  0xfa,  0xec, 
  0x9e,  0xfb,  0x0b,  0x78,  0x91,  0x2f,  0xe9,  0x65,  0xbf,  0x92,  0x57,  0xfb,  0xda,  0x5e,  0xff,  0x9b, 
  0x78,  0x8b,  0x5f,  0xde,  0x7d,  0xbd,  0x2f,  0x06,  0xfd,  0x10,  0xbf,  0x06,  0xe6,  0x75,  0xb8,  0x19, 
  0xcd,  0x43,  0x7c,  0x92,  0x8e,  0xff,  0x20,  0x1e,  0xe2,  0xc3,  0x7a,  0xc4,  0x8f,  0xe6,  0xb1,  0x3e, 
  0xbe,  0x27,  0xfe,  0x14,  0x9e,  0xe6,  0x33,  0x7a,  0xd6,  0xcf,  0xe5,  0xf9,  0xbe,  0xb0,  0x17,  0xff, 
  0x32,  0x5e,  0xe1,  0xab,  0x7a,  0xcd,  0xef,  0xdf,  0x9d,  0x70,  0x90,  0x20,  0xee,  0x0a,  0xdd,  0xf2, 
  0xb7,  0x04,  0x27,  0xc4,  0xc1,  0xb1,  0x5f,  0x45,  0xd0,  0xe0,  0x3c,  0x2c,  0x23,  0x48,  0xb0,  0x19, 
  0x17,  0xa3,  0x28,  0x12,  0xc6,  0x0d,  0xcf,  0xfe,  0x83,  0xff,  0x28,  0x0d,  0x7e,  0x13,  0x3d,  0x24, 
  0x4d,  0xff,  0x39,  0xbe,  0xe0,  0x97,  0xfa,  0xca,  0x5f,  0xe7,  0x9b,  0x7e,  0xbb,  0xef,  0xfe,  0x03, 
  0x7e,  0xe4,  0x4f,  0xfa,  0xd9,  0xbf,  0xe4,  0xd7,  0xfe,  0xb6,  0x63,  0xc4,  0x88,  0x21,  0x40,  0xbc, 
  0x69,  0x4c,  0x83,  0xfb,  0xf5,  0x05,  0xd7,  0xe0,  0x3f,  0xe9,  0x82,  0xfb,  0x98,  0x8a,  0xfe,  0xe1, 
  0x6d,  0x3e,  0x85,  0x58,  0xff,  0x8b,  0x7b,  0x54,  0x04,  0xb7,  0x00,  0x1e,  0xe4,  0x43,  0x7a,  0xd8, 
  0x8f,  0xe4,  0xd1,  0x3e,  0xb6,  0xc7,  0xff,  0x24,  0x9e,  0xe2,  0xd3,  0x7a,  0xc6,  0xcf,  0xe6,  0xb9, 
  0x3e,  0xbf,  0x17,  0xfe,  0xff,  0x1f,  0x20,  0x21,  0x07,  0x80,  0xe6,  0x1a,  0xe8,  0x82,  0x7b,  0xc8, 
  0x79,  0x8f,  0xe1,  0x3f,  0x3c,  0x47,  0x97,  0x1f,  0x19,  0xfe,  0x03,  0x68,  0xe7,  0xef,  0xe1,  0x7d, 

  0x7e,  0xa0,  0x0f,  0xfd,  0x51,  0x3e,  0xee,  0x27,  0xfb,  0xf4,  0x9f,  0xe3,  0x0b,  0x7e,  0xa9,  0xaf, 
  0xfc,  0x75,  0xbe,  0xe9,  0xb7,  0xfb,  0xee,  0x3f,  0xe0,  0x47,  0xbe,  0xff,  0xd6,  0x59,  0x4f,  0x17, 
  0x22,  0x48,  0x35,  0xf5,  0x3b,  0xd1,  0xf2,  0x5b,  0x7b,  0xa0,  0x7f,  0xf6,  0xaf,  0xfd,  0xfd,  0x7f, 
  0xf1,  0x1f,  0xff,  0xd7,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff, 
  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xff,  0xa7, 
  0x7f, 
};
static PROGMEM prog_uchar logo_desire_sprpal[] = {

  0x9f,  0x4f,  0xb7,  0x3a,  0xcf,  0x25,  0xe8,  0x10,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00, 
  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x80, 
};


void intro(){

/* INTRO TECH SPEC */
  map_load_Text();
  textRect(6, 0+3, 35, 16,  "");

  putstr(8,2+3,  " DEMO PLATFORM SPECIFICATIONS:");
  putstr(8,5+3,  "         ARDUINO MEGA");
  putstr(8,7+3,  "CPU         : ATMega 2560 (8-BIT)");
  putstr(8,9+3,  "CPU SPEED   : 16Mhz");
  putstr(8,11+3, "MEMORY      :   8KB");
  putstr(8,13+3, "FLASH SPACE : 256KB ");

  textRect(6, 18+3, 35, 10,  "");

  putstr(8+4,20+3,"ADDITIONAL HARDWARE:");

  putstr(8,23+3,"Gameduino Shield");
  putstr(8,25+3,"PS2 Keyboard");


  //voice.say(lpc_invite);

//  delay(16000);

  fadeAll();

  hideSprites();

  /* LOGO FADING BETWEEN TRSI AND DESIRE */
  GD.uncompress(RAM_SPRIMG, logo_trsi_sprimg);
  GD.copy(PALETTE16A, logo_trsi_sprpal, sizeof(logo_trsi_sprpal));

  // For show, randomly scatter the frames on the screen
  GD.__wstartspr(0);
  draw_logo_trsi(160+35, 120, 0, 0);
  GD.__end();

  delay(2000);
  fadeAll();

  hideSprites();

  GD.uncompress(RAM_SPRIMG, logo_desire_sprimg);
  GD.copy(PALETTE16A, logo_desire_sprpal, sizeof(logo_desire_sprpal));

  // For show, randomly scatter the frames on the screen
  GD.__wstartspr(0);
  draw_logo_desire(160+35, 120, 0, 0);
  GD.__end();

  delay(2000);
  fadeAll();

  clearScreen();

  /* STORY NARRATIVE HERE */
  hideSprites();
  map_load_Text();
  textRect(6, 0+3, 35-4, 16,  "A long time ago, in a South   Western Scene far far away...                               There were three alien dudes  called Saar Star, Trired      Sectorius and Coopus Desirius.                              They always fought about the  question, who had the best    effects on the planet, so theydecided to visit their local  battle arena E-Werk and       have an epic duel about it.");
  
  
  //  There were three alien dudes called \"Saar Star\"\,\"Trired Sectorius\" and \"Coopus Desirius\"!They always fought about the question, who had the best effects on the planet\, so they decided to visit their local battle arena \"E-Werkius\" and have an epic duel about it.");
  fadeAll();

  clearScreen();
  clearSprites();
  runStars3D(600);
 //fadeAll();

  clearSprites();
  GD.wr(SPR_PAGE, flip);
//  clearSprites();
 fadeAll();

//  clearScreen();  


}







/*

 Name      :  Lens
 Notes     :  Flames rising up the screen
 
 The fire effect has been used quite often for oldskool demos. 
 First you create a palette of 256 colors ranging from red to
 yellow (including black). For every frame, calculate each row 
 of pixels based on the two rows below it: The value of each pixel,
 becomes the sum of the 3 pixels below it (one directly below, one 
 to the left, and one to the right), and one pixel directly two 
 rows below it. Then divide the sum so that the fire dies out 
 as it rises.
 
 references:
 http://www.student.kuleuven.be/~m0216922/CG/fire.html 
 
 */

void lens(){

  byte w = 50;
  byte h= 37;

  byte timeDisplacement=950;  // number of frames to run
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];

  int lensD = 128;
  int xx = 0;
  int yy = 0; 
  int dx = 1;
  int dy = 1;

  int ballSize = 64;

  int lensPixels = lensD*lensD;


  for (int i = 0; i < lensPixels; i++)  {
    //lensImage[i] = lensEffect[(ball_x + (lensArray[i] % lensD))+(ball_y + (lensArray[i] >> 7))*w]; 
    //lensImage[i] = lensEffect[(ball_x + lensArrayMod[i])+(ball_y + lensArrayDiv[i])*w]; 
  }


  /*
  do{    
   
   // Do the fire calculations for every pixel, from top to bottom
   int currentPixelIndex = 1; // start from center
   for (int y = RAM_PIC; y < bufferY;y+=64) { 
   for(byte x = 0; x < w; ) {
   // change to add 3 instead of 4 to speed this up for firecubefs
   pixel_buffer[x++] =      fire_buffer[currentPixelIndex] = (fire_buffer[currentPixelIndex]
   + fire_buffer[currentPixelIndex+widthLeft]
   + fire_buffer[currentPixelIndex+w]
   + fire_buffer[currentPixelIndex++ +widthRight])  >>2;          
   }
   GD.copy2(y,pixel_buffer,w);
   }
   }
   while(--timeDisplacement);
   */

}

// Lens algorithm (transformation array)
void calculate_tfm(int lensD, int magFactor)
{
  int lensArray[16*16];     

  int counter=0;
  int before = 0; //millis();

  int a, b;
  int r = lensD /2;
  int rSq=r*r;
  int s = (rSq - magFactor*magFactor);

  int x2;
  int y2;

  int temp;
  int temp2;

  // calculate the first quadrant (90 degrees) of the lens deformation, then extrapolate the other three quadrants
  for (int y = -r; y <= 0; y++) {
    y2=y*y;
    temp = ( y + r )*lensD;
    temp2 =(-y + r )*lensD;
    for (int x = -r ;x <= 0; x++) {
      x2=x*x;

      counter++;
      if(x2 + y2 >= s) {
        a = x;
        b = y;
      }
      else {
        float z = (float)sqrt(rSq - x2 - y2) ;
        a = (int)floor(x * magFactor / z);
        b = (int)floor(y * magFactor / z);
      }
      //  mirror quadrant of lens to make a complete circle
      lensArray[ temp2 - (-x + r)] =        ((-b + r) * lensD - (-a + r));          // bottom left
      lensArray[(temp2 + (-x + r))-lensD] = ((-b + r) * lensD + (-a + r))-lensD;    // bottom right
      lensArray[ temp + ( x + r)] =          ( b + r) * lensD + ( a + r);           // top left
      lensArray[(temp - ( x + r))+lensD] =   ( b + r) * lensD - ( a + r)+lensD;     // top right      
    }
  }
}


/*

 Name      :  Metaball
 Notes     :  organic-looking n-dimensional objects. 
 
 The technique for rendering metaballs was invented by Jim Blinn in the early 1980s
 
 references:
 http://www.benryves.com/tutorials/blobs/    
 http://www.student.kuleuven.be/~m0216922/CG/index.html
 
 */
void metaball(){

  byte w = 50;
  byte h = 37;


  byte numBlobs = 2;          // two blobby blobs
  byte blob_px[] = {
    0, 20    };   // where blobs will first show
  byte blob_py[] = {
    10, 0            };

  // Movement vector for each blob
  char blob_dx[] = {
    1, 1            };
  char blob_dy[] = {
    1, 1            };

  // build square lookup table
  unsigned int vy[w];
  for (int i = 0; i < w; i++) {
    vy[i] = sq(i);
  }

  metaPalette();

  int m = 0;
  byte timeDisplacement=140;     // number of frames to run

  do
  {
    for (byte i=0; i<numBlobs; ++i) {
      blob_px[i]+=blob_dx[i];
      blob_py[i]+=blob_dy[i];

      // bounce across screen
      if (blob_px[i] < 5) {
        blob_dx[i] = 1;
      }
      if (blob_px[i] >= w-5) {
        blob_dx[i] = -1;
      }
      if (blob_py[i] < 5) {
        blob_dy[i] = 1;
      }
      if (blob_py[i] > h-5) {
        blob_dy[i]=-1;
      }
    }

    int buffer=0;
    for (byte y = 0; y < h; y++) {
      for (byte x = 0; x < w; x++) { 
        m = 1;
        for (byte i = 0; i < numBlobs; i++) {
          // Increase 10K number to make your blobs bigger 
          m += 10000/(vy[abs(blob_px[i]-x)] + vy[abs(blob_py[i]-y)]+1);
        }
        if (m>255) m=255; // this prevents center of metballs from junking up (overflowed colors)

        //        GD.wr(RAM_PIC+x+buffer,meta_palette[m]);   //color(m,0,m)
        GD.wr(RAM_PIC+x+buffer,m);   //color(m,0,m)
      }
      buffer+=64;
    }
  }
  while(--timeDisplacement);
}

void metaPalette(){
  // byte meta_palette[256];
  // generate flame color palette in RGB. need 256 bytes available memory
  for(int i=0; i<256; i++)
  {
    //  meta_palette[i]  = color(i,0,i);      // Black to red
    GD.setpal(i*4, RGB(i,0,i));   
  }      
}

void metaball2(){
  byte w = 50;
  byte h = 37;

  byte numBlobs = 2;          // two blobby blobs
  byte blob_px[] = {0, 20};   // where blobs will first show
  byte blob_py[] = {10, 0};

  // Movement vector for each blob
  char blob_dx[] = {
    1, 1            };
  char blob_dy[] = {
    1, 1            };

  byte LUT[w][h];
  
 // byte pixel_buffer[w];

  // clear screen of character data  
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 4096); // second number is color 0-3  



//  generate_332_palette();
  metaPalette();
  
  
  for(byte i=0; i<86; i++)
  {
    GD.setpal(i*4, RGB(i*3,0,0));          // Black to red
    GD.setpal((i+85)*4, RGB(255,i*3,0));   // Red to yellow
    GD.setpal((i+170)*4, RGB(255,255,i*3));   // yellow to white
  } 
 // GD.ascii();


  for (int y = 0; y < h; y++) {
    for (int x = 0; x < w; x++) {
      // Increase 10K number to make your blobs bigger 
      int temp = 10000/((x*x) + (y*y));
      
      // this prevents center of metballs from junking up (overflowed colors)
      if (temp > 255) temp = 255;
      
      LUT[x][y] = temp;
    }
  }



  int m = 0;
  int timeDisplacement=340;     // number of frames to run

  do
  {
    

    for (byte i=0; i<numBlobs; ++i) {
      blob_px[i]+=blob_dx[i];
      blob_py[i]+=blob_dy[i];

      // bounce across screen
      if (blob_px[i] < 5) {
        blob_dx[i] = 1;
      }
      if (blob_px[i] >= w-5) {
        blob_dx[i] = -1;
      }
      if (blob_py[i] < 5) {
        blob_dy[i] = 1;
      }
      if (blob_py[i] > h-5) {
        blob_dy[i]=-1;
      }
    }


    int buffer=0;
    for (byte y = 0; y < h; y++) {
      for (byte x = 0; x < w; x++) { 
        m = 1;


        for (byte i = 0; i < numBlobs; i++) {
         m += LUT[abs(blob_px[i]-x)][abs(blob_py[i]-y)];
         }
         if (m>255) m=255; // this prevents center of metballs from junking up (overflowed colors)

     //   m += LUT[abs(blob_px[0]-x)][abs(blob_py[0]-y)];
//        m += LUT[abs(blob_px[1]-x)][abs(blob_py[1]-y)];

        //        GD.wr(RAM_PIC+x+buffer,meta_palette[m]);   //color(m,0,m)
         
       // pixel_buffer[x] = m;
        GD.wr(RAM_PIC+x+buffer,m);   //color(m,0,m)        
      }
      //GD.copy2(buffer,pixel_buffer,w);
      buffer+=64;
    }
  }
  while(--timeDisplacement);


}








void metaball3() {
  byte w = 50;
  byte h = 37;

  int centerx = (8 * 25);
  int centery = (8 * 17);

  byte numBlobs = 4;
  struct {
    int x, y;
    signed char dx, dy;
  } blobs[numBlobs];
  for (byte i=0; i<numBlobs; ++i) {
    blobs[i].x = random(2*centerx);
    blobs[i].y = random(2*centery);
    blobs[i].dx = random(32) - 16;
    blobs[i].dy = random(32) - 16;
  }

  for(byte i=0; i<86; i++)
  {
    GD.setpal(i*4, RGB(i*3,0,0));          // Black to red
    GD.setpal((i+85)*4, RGB(255,i*3,0));   // Red to yellow
    GD.setpal((i+170)*4, RGB(255,255,i*3));   // yellow to white
  } 

  GD.wr16(RAM_SPRPAL, TRANSPARENT);
  GD.__wstart(RAM_SPRIMG);
  for (byte r = 0; r < 64; r++)
    for (byte x = 0; x < 16; x++)
      for (byte y = 0; y < 16; y++)
        SPI.transfer((SQ(8-x) + SQ(8-y)) < r);
  GD.__end();

  int timeDisplacement=20*72;     // number of frames to run

  int recipsz = (w*w + h*h) / 4 + 1;
  byte recip[recipsz];
  for (int i = 0; i < recipsz; i++)
    recip[i] = min(255, 10000 / (4 * i));

  byte fadein = 255;
  do
  {
    for (byte i=0; i<numBlobs; ++i) {
      if (blobs[i].x < centerx)
        blobs[i].dx++;
      else
        blobs[i].dx--;
      if (blobs[i].y < centery)
        blobs[i].dy++;
      else
        blobs[i].dy--;
      blobs[i].x += blobs[i].dx;
      blobs[i].y += blobs[i].dy;
    }
    blobs[random(numBlobs)].dx = random(32) - 16;
    blobs[random(numBlobs)].dy = random(32) - 16;

    for (byte y = 0; y < h; y++) {
      GD.__wstart(RAM_PIC + (y << 6));
      for (byte x = 0; x < w; x++) { 
        int m = fadein;
        for (byte i = 0; i < 3; i++) {
           int dx = (blobs[i].x >> 3) - x;
           int dy = (blobs[i].y >> 3) - y;
           int d = SQ(dx) + SQ(dy);
           m += recip[min(d >> 2, recipsz - 1)];
        }
        SPI.transfer(min(255, m));
      }
      GD.__end();
    }
    
    /*
    GD.__wstart(RAM_SPR);
    for (byte i = 3; i < numBlobs; i++) {
      int x = blobs[i].x;
      int y = blobs[i].y;
      SPI.transfer(lowByte(x));
      SPI.transfer((highByte(x) & 1));
      SPI.transfer(lowByte(y));
      SPI.transfer((i & 0x7e) | (highByte(y) & 1));
    }
    GD.__end();
    */
    
    fadein = max(fadein - 3, 1);
  }
  while(1 || --timeDisplacement);
}


/*

 Name      :  Plasma 2
 Notes     :  Cycles of changing colours warped to give an illusion of liquid, organic movement.
 
 Precalculated plasma effect values stored in flash rom array.  Colors are the sum of sine functions and various formulas.   
 slightly slower that plasma 1
 
 */


void plasma(){

  int GRADIENTLEN = 256;

  // swing/wave function parameters
  int SWINGLEN = GRADIENTLEN*3;
  int SWINGMAX = GRADIENTLEN / 2 ;
  int swingCurve[SWINGLEN-1];

  // gradient & swing curve arrays
  makePlasmaPalette(GRADIENTLEN);

  // default values will be used upon first call
  int factor1=2;
  int factor2=3;
  int factor3=6;

  int halfmax = SWINGMAX/factor1;


  for( int i=0; i<SWINGLEN; i++ ) {
    float ni = i*TWO_PI/SWINGLEN; // ni goes [0..TWO_PI] -> one complete cos wave
    swingCurve[i]=(int)(
    cos( ni*factor1 ) *
      cos( ni*factor2 ) *
      cos( ni*factor3 ) *
      halfmax + halfmax );
  }  

  byte w = 50;
  byte h= 37;
  int timeDisplacement=600;  // number of frames to run
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];


//  Vectorball2 v;
//  v.init();      

  do
  {  
//    v.vectorball(timeDisplacement);                
    
    int t = timeDisplacement;
//    int swingT = swingCurve[t & (SWINGLEN - 1)]; 
//    int swingT = swing(t);
    
    int swingT = swingCurve[t & (SWINGLEN-1)];
    int yy=0;

    for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
      int swingY  = swingCurve[yy & (SWINGLEN - 1)];
      int swingYT = swingCurve[(yy+t) & (SWINGLEN - 1)];
      yy++;
      for (int x = 0; x < w;) {
        // this is where the magic happens: map x, y, t around
        // the swing curves and lookup a color from the gradient
        // the "formula" was found by a lot of experimentation

        // pixel_buffer[x++] = color(0,255,0);
//        pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) % SWINGLEN] + swingYT) % SWINGLEN] + swingCurve[(swingCurve[(x + timeDisplacement) % SWINGLEN] + swingY) % SWINGLEN]) ;
        pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) & (SWINGLEN - 1)] + swingYT) & (SWINGLEN - 1)] + swingCurve[(swingCurve[(x + timeDisplacement) & (SWINGLEN - 1)] + swingY) & (SWINGLEN - 1)])  ;
//       pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) ] + swingYT) ] + swingCurve[(swingCurve[(x + t) ] + swingY) ]) ;
      }
      GD.copy2(y,pixel_buffer,w);
    }

  }
  while(timeDisplacement+=-1);
  
//  v.clearball();        
  
}

// helper: get cosinus sample normalized to 0..255
int cos256( int amplitude,  int x) {
  return (int) (cos(x * TWO_PI / amplitude) * 127 + 127);
}

// helper: get a swing curve sample
int swing(int i) {
  //return swingCurve[i % SWINGLEN];
}

// helper: get a gradient sample
int gradient(int i) {
  //	return colorGrad[i % GRADIENTLEN];
}

// make gradient 
void makePlasmaPalette(int arrlen){
  // default values will be used upon first call
  int rf = 3;
  int gf = 3;
  int bf = 1;
  int rd = 0;
  int gd = arrlen / gf;
  int bd = arrlen / bf / 2;
  
  
  rd = 160;
  gd = 217;
  bd = 178;

  // fill gradient array
  for (int i = 0; i < arrlen; i++) {
    int r = cos256(arrlen / rf, i + rd);
    int g = cos256(arrlen / gf, i + gd);
    int b = cos256(arrlen / bf, i + bd);
    GD.setpal(i*4, RGB(r,g,b)); 
  }
  
  
  

//  showPalette();
//  delay(2000);  
}


void plasma2(){
  byte w = 50;
  byte h= 37;
  int timeDisplacement=1000;  // number of frames to run
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];
  
  float s1,s2;
  for (int i=0;i<128;i++) {
    s1=sin(i*PI/25);
    s2=sin(i*PI/50+PI/4);
    GD.setpal(i*4, RGB((int)(128+s1*128),(int)(128+s2*128),(int)(s1*128))); 
  }
  
  int cls[w*h];
  for (int x = 0; x < w; x++)
  {
    for (int y = 0; y < h; y++)
    {
      cls[x+y*w] = (int)((127.5 + +(127.5 * sin(x / 32.0)))+ (127.5 + +(127.5 * cos(y / 32.0))) + (127.5 + +(127.5 * sin(sqrt((x * x + y * y)) / 32.0)))  ) / 4;
    }
  }

  do{
  int currentPixelIndex = 0; 
     for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
      for(int x = 0; x < w; x++) {      
               pixel_buffer[x] = (cls[currentPixelIndex++] + timeDisplacement)&127;
      }
      GD.copy2(y,pixel_buffer,w);
    }
  }
  while(timeDisplacement+=-1);  
}

/*

 Name      :  Fire
 Notes     :  Flames rising up the screen
 
 The fire effect has been used quite often for oldskool demos. 
 First you create a palette of 256 colors ranging from red to
 yellow (including black). For every frame, calculate each row 
 of pixels based on the two rows below it: The value of each pixel,
 becomes the sum of the 3 pixels below it (one directly below, one 
 to the left, and one to the right), and one pixel directly two 
 rows below it. Then divide the sum so that the fire dies out 
 as it rises.
 
 references:
 http://www.student.kuleuven.be/~m0216922/CG/fire.html 
 
 */

void star(){
  // generate flame color palette in RGB. need 256 bytes available memory
  for(byte i=0; i<86; i++)
  {
    GD.setpal(i*4, RGB(i*3,0,0));          // Black to red
    GD.setpal((i+85)*4, RGB(255,i*3,0));   // Red to yellow
    GD.setpal((i+170)*4, RGB(255,255,i*3));   // yellow to white
  } 

//    showPalette();
//    delay(5000);

  byte w = 50;
  byte h= 37;

  byte timeDisplacement=950;  // number of frames to run
  byte fire_buffer[(w+1)*(h)];
  int  fire_length = w*h;
//  byte widthLeft = w-1;
//  byte widthRight = w+1;
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];
 
  

  do{    
    for(byte x = 0; x < w; x++){
      fire_buffer[x+fire_length] = (byte)random(255);
    }

    // Do the fire calculations for every pixel, from top to bottom
    int currentPixelIndex = 1; // start from center
    for (int y = RAM_PIC; y < bufferY;y+=64) { 
      for(byte x = 0; x < w; ) {
        // Add pixel values around current pixel
        // Output everything to screen using our palette colors      
        

/*
        GD.wr(x++ +y,  fire_buffer[currentPixelIndex] = (fire_buffer[currentPixelIndex]
          + fire_buffer[currentPixelIndex+widthLeft]
          + fire_buffer[currentPixelIndex+w]
          + fire_buffer[currentPixelIndex++ +widthRight])  >>2
          );
          */
          
          // change to add 3 instead of 4 to speed this up for firecubefs
        pixel_buffer[x++] =      fire_buffer[currentPixelIndex] = (fire_buffer[currentPixelIndex]
          + fire_buffer[currentPixelIndex+widthLeft]
          + fire_buffer[currentPixelIndex+w]
          + fire_buffer[currentPixelIndex++ +widthRight])  >>2;          
          

      }
            GD.copy2(y,pixel_buffer,w);
    }
  }
  while(--timeDisplacement);





}



