/*

 Name      :  Vectorballs
 Notes     :  2D circles spinning in 3D space
 
 A vector ball is a 2D Sprite represented in 3D space.
 Based on code by Peter Knight, Tinker.it
 
 references:
 http://code.google.com/p/tinkerit/
 
 implement large vectorballs from 4 sprites giving us 64 numballs
 resize first sprite or create from deform of varying sizesf
 
 */


#define NUM_BALLS 28
#include "sprites.h"

const PROGMEM char ballPosnX[NUM_BALLS] = {
  -3,-2,-1,0,1,-3,-2,1,2,-3,-2,1,2,-3,-2,1,2,-3,-2,-1,0,1,-3,-2,-3,-2,1,3};
const PROGMEM char ballPosnY[NUM_BALLS] = {
  3,3,3,3,3,2,2,2,2,1,1,1,1,0,0,0,0,-1,-1,-1,-1,-1,-2,-2,-3,-3,-3,-3};

class Vectorball2
{
public:
  Vectorball2();
  void draw_sprite(int x, int y, byte anim, byte rot, byte jk);
  void init();
  void clearball();
  void vectorball(int timeDisplacement);

private:
  int _pin;

  int x ;
  int y ;

  int w ;
  int h ;

  float ballX[NUM_BALLS];
  float ballY[NUM_BALLS];
  float ballZ[NUM_BALLS];
  float angx ;
  float angy ;
  float angz ;  

  //  int timeDisplacement;// number of frames to run
  byte page;              // double buffer counter
};



void vectorball(){  
  Vectorball2 v;
//  v.init();

  int timeDisplacement=1750;// number of frames to run

  do
  {    
    v.vectorball(timeDisplacement);
  }
  while(--timeDisplacement);


  // clear the balls
  //  for (int i=0; i<NUM_BALLS; i++)
  //  {
  //drawBall(-100,-100,0,i);
  //  }  
}


Vectorball2::Vectorball2(){
  
  init();
  
}

void Vectorball2::draw_sprite(int x, int y, byte anim, byte rot, byte jk = 0) {
  GD.xsprite(x, y, -16, -16, 0, 0, rot, jk);
  GD.xsprite(x, y, 0, -16, 1, 0, rot, jk);
  GD.xsprite(x, y, -16, 0, 2, 0, rot, jk);
  GD.xsprite(x, y, 0, 0, 3, 0, rot, jk);
}

void Vectorball2::init(){ 
  x = 32;
  y = 32;

  w = 32;
  h = 32;

  float ballX[NUM_BALLS];
  float ballY[NUM_BALLS];
  float ballZ[NUM_BALLS];
  angx = 0;
  angy = 0;
  angz = 0;  

  page=0;              // double buffer counter

  // load sprites from progmem
  GD.copy(RAM_SPRIMG, pickups2_img, sizeof(pickups2_img));
  GD.copy(RAM_SPRPAL, pickups2_pal, sizeof(pickups2_pal));
}

void Vectorball2::vectorball(int timeDisplacement){  
  page = timeDisplacement & 1; // fast modulo operation using bitwise AND (same as: timeDisplacement % 2)

  float cosangx = cos(angx);
  float sinangx = sin(angx);
  float cosangy = cos(angy);
  float sinangy = sin(angy);
  float cosangz = cos(angz);
  float sinangz = sin(angz);

  for (int n=0; n<NUM_BALLS; n++) {
    int x = (signed char)pgm_read_byte(&ballPosnX[n]);
    int y = (signed char)pgm_read_byte(&ballPosnY[n]);
    int z = 0;

    // Rotate about x axis
    float x2 = x;
    float y2 = y * cosangx - z * sinangx;
    float z2 = y * sinangx + z * cosangx;

    // Rotate about y axis
    float x3 = x2 * cosangy + z2 * sinangy;
    float y3 = y2;
    float z3 = -x2 * sinangy + z2 * cosangy;

    // Rotate about z axis
    float x4 = x3 * cosangz - y3 * sinangz;
    float y4 = x3 * sinangz + y3 * cosangz;
    float z4 = z3;

    ballX[n] = x4;
    ballY[n] = y4;
    ballZ[n] = z4;
  }

  // Depth sort
  int increment = NUM_BALLS << 2;  // same as: NUM_BALLS / 4

  while (increment > 0) {
    for (int i = increment; i < NUM_BALLS; i++)
    {
      int j = i;
      float tempX = ballX[i];
      float tempY = ballY[i];
      float tempZ = ballZ[i];
      while ((j >= increment) && (ballZ[j-increment] > tempZ))
      {
        ballX[j] = ballX[j - increment];
        ballY[j] = ballY[j - increment];
        ballZ[j] = ballZ[j - increment];
        j = j - increment;
      }
      ballX[j] = tempX;
      ballY[j] = tempY;
      ballZ[j] = tempZ;
    }
    increment = (increment == 2) ? 1: (int)(increment / 2.2); 
  }

  // Render with perspective
  GD.__wstartspr(page ? 256 : 0);      

  for (int i=0; i<NUM_BALLS; i++)
  {
    float scale = (ballZ[i] +30); //size of sprite
    float x = ballX[i] * scale + 200;
    float y = ballY[i] * scale + 150;
    //      drawBall(x,y,scale,i);
    draw_sprite(x, y, 0, 0);

  }
  GD.__end();

  //  GD.waitvblank();
  // swap frames
  GD.wr(SPR_PAGE, page);

  // For show, randomly scatter the frames on the screen
  //  for (int anim = 0; anim < 1; anim++)
  //    draw_sprite(random(400), random(300), 0, 0);

  // Rotate camera
  angx+=0.09;
  angy+=0.03;
  angz+=0.03;
}


void Vectorball2::clearball(){

  GD.__wstartspr(1 ? 256 : 0);
  for (int i=0; i<NUM_BALLS; i++)
  {
    draw_sprite(-100, -100, 0, 0);
  }
  GD.__end();


  GD.__wstartspr(0 ? 256 : 0);      
  for (int i=0; i<NUM_BALLS; i++)
  {
    draw_sprite(-100, -100, 0, 0);
  }
  GD.__end();
}

void drawBall(int x, int y, int scale, int sprintNum){
  GD.sprite(sprintNum, x , y , 0, 0, 0);
}





