/*

 Name      :  Rotozoomer
 Notes     :  A rotating and zooming bitmap.
 
 2D rotozoomers are a combination of rotation and scaling transformations
 Could possibly be the starting point for polygon texturing routines
 utilized in 3D renderers.  One of the most famous rotozoomers is the one 
 from Second Reality / Future Crew which this is based on.
 
 references:
 http://www.student.kuleuven.be/~m0216922/CG/xortexture.html
 
 */





class Rotozoomer
{
public:
  Rotozoomer(){
    clearScreen();
//    generate_332_palette();

  GD.copy(RAM_CHR, space_chr, sizeof(space_chr));
  GD.copy(RAM_PAL, space_pal, sizeof(space_pal));

    //cosine[i]=Float2Fixed();
     
    for (int i=0; i<256; i++) {
      sintab[i] = (int)(sin(i/256.0*PI*2)*255.0+255.0) >> 1;
      costab[i] = (int)(cos(i/256.0*PI*2)*255.0+255.0) >> 1;

//      sintab[i] = (int)(sin((float)i*360/256 * PI / 180.0)) >> 1;
//      costab[i] = (int)(cos((float)i*360/256 * PI / 180.0)) >> 1;
    }
    
    for (int i = 0; i < 256; i++) {
        float rad =  (float)i * 1.41176 * 0.0174532;
	float c = sin(rad);
	float d = cos(rad);
        roto[i] = (c + 0.8) * 4096.0; // rotation smaller = faster
	roto2[i] = (2.0 * d) * 4096.0;
    }
    
  }

  void render1(int timeDisplacement){

      int m = timeDisplacement & 0xFF;
      int sv = sintab[m];
      int cv = costab[m];
      int dy = sv;
      int dx = cv;
      int ty,tx;    
      int soffs=0;

      for (int y = RAM_PIC; y < bufferY;y+=64) { 
        ty=dy+=cv;
        tx=dx+=-sv;      
        for (byte x=0;x<RENDERWIDTH;x++) {
          // pixel_buffer[x++]  = pgm_read_byte(&meanman_32[( ((ty+=sv) >> 5)  + ((tx+=cv) & 0x3E0)) & 0x3FF]);     // 256 texture
//          fire_buffer[soffs++]  = pgm_read_byte(&meanman_32[( ((ty+=sv) >> 5)  + ((tx+=cv) & 0x3E0)) & 0x3FF]);     // 256 texture
         fire_buffer[soffs++]  = pgm_read_byte(&space_pic[( ((ty+=sv) >> 6)  + ((tx+=cv) & 0xFC0)) & 0xFFF]);     // 64 texture         (64x64)-64
        }
      }


//    fade();
  }
  
void render()
{
        /* draw the tile at current zoom and rotation */
	draw_tile(roto[path], roto[(path + 128) & 255], roto2[zpath]);
        path =(path - 1) & 255;
        zpath = (zpath + 1) & 255;

} 
 

void draw_tile(long stepx, long stepy, int zoom)
{
//  Uint8 *image = (Uint8 *)screen->pixels;
//  Uint8 *texture = (Uint8 *)tile->pixels;
  long x, y, sx, sy;
  int xd,yd;
  byte a,b;
    
  sx = sy = 0;
  
  xd = (stepx * zoom) >> 12;
  yd = (stepy * zoom) >> 12;
    
  /* Stepping across and down the screen, each screen row has a
   * starting coordinate in the texture: (sx, sy).  As each screen
   * row is traversed, the current texture coordinate (x, y) is
   * modified by (xd, yd), which are (sin(rot), cos(rot)) multiplied
   * by the current zoom factor.  For each vertical step, (xd, yd)
   * is rotated 90 degrees, to become (-yd, xd).
   *
   * More fun can be had by playing around with x, y, xd, and yd as
   * you move about the image.
   */

int soffs=0;
  for (int j = 0; j < RENDERHEIGHT; j++) {
    x = sx; 
    y = sy;   
    for (int i = 0; i < RENDERWIDTH; i++) {
      a = x >> 12 & 255;
      b = y >> 12 & 255;
      //fire_buffer[soffs++]  = pgm_read_byte(&meanman_32[  (b * 32 + a)& 0x3FF  ]);     // 64 texture         (64x64)-64
      fire_buffer[soffs++]  = pgm_read_byte(&space_pic[  (b * 64 + a)& 0xFFF  ]);     // 64 texture         (64x64)-64
      x += xd;
      y += yd;
    }
    sx -= yd;
    sy += xd;
  }
}

private:
  int roto[256];
  int roto2[256];
  unsigned int path = 0, zpath = 0;

  int sintab[256];  
  int costab[256];  
};



