/*

 Name      :  Vectorball
 Notes     :  spinning balls in space
 
 */

// http://openprocessing.org/sketch/89602
// http://www.dcee.net/Files/Programm/Demos/
//  http://kyucon.com/qblock/#/43408

//#include <avr/pgmspace.h>

#include "vectorball_data_qdot.h"
#include "vectorball_model_chicken.h"
#include "vectorball_model_coil.h"
//#include "battle.h"

typedef long Fixedpoint;
#define Int2Fixed(a)	(Fixedpoint)((Fixedpoint)(a) << 16)
#define Fixed2Int(a)	(int)((a) >> 16)
#define Float2Fixed(a)	((Fixedpoint)((a) * 65536.0))
#define Fixed2Float(a)	((a) / 65536.0)
#define FixedMul(x, y) (((x)>>8)*((y)>>8))
#define FixedDiv(x, y) (FixedMul((x), FixedRcp(y)))
#define FixedRcp(x) (Float2Fixed(1.0f/Fixed2Float(x)))

#define MAXDEGREES 255        // maximum # of degrees for sin and cos tables
#define EYE_DISTANCE Int2Fixed(60)  // distance from users eye to screen surface in pixels.  larger numbers spread them out
#define DEPTH Int2Fixed(4096)    // depth to push objects into the screen
#define MAXBALLS 158  // maximum number of balls an object can have

// data structures
typedef struct
{
  int sx,sy,sz;
  byte index;
}
BallTYPE;

// for sorting function
int CompareBallsStruct( const void *elem1, const void *elem2 ) {
  return ((BallTYPE*)elem1)->sz - ((BallTYPE*)elem2)->sz;
  //  return (*(BallTYPE*)elem1).sz - (*(BallTYPE*)elem2).sz;
}

int CompareBalls(const void *x, const void *y)
{
  //  return Ball[*(int*)x].sz - Ball[*(int*)y].sz;
  //    return (*(int*)x - *(int*)y);
}

class Vectorball
{
public:

  Vectorball(){
    clearSprites();
    InitSinCosTables();
    
    // point each element to an initial ball
    for(int i=0; i<MAXBALLS; i++)
    {

      Ball[i].sz=i;
      Ball[i].sx=400;
      Ball[i].sy=0;
    }    
  }

  ~Vectorball(){
    clearSprites();    
  }

  //  loads 32x32 sprites ball
  void loadLargeDots(){
    GD.copy(RAM_SPRPAL, sprite_sprpal, sizeof(sprite_sprpal));
    GD.uncompress(RAM_SPRIMG, sprite_sprimg);
    smalldotmode=false;
  }

  // load 16x16 sprite ball 
  void loadSmallDots(){
    GD.copy(RAM_SPRPAL, qdot_sprpal, sizeof(qdot_sprpal));
    GD.uncompress(RAM_SPRIMG, qdot_sprimg);    
    smalldotmode=true;
  }

  // change pointers to load different models
  void LoadObject(const char *dotData)
  {
    int count=0;
    flashdot0=dotData;

    // read number of dots in file
    NumOfBalls = fread(count++);
  }

  void LoadObject(const char *dotData, const char *dotColorData)
  {
    int count=0;
    flashdot0=dotData;
    flashdot1=dotColorData;  

    // read number of dots in file
    NumOfBalls = fread(count++);

    /*
  // read in all of the object's dots
     for(i=0;i < NumOfBalls; i++)
     {
     temp = fread(2*count++);
     Ball[i].ox=Int2Fixed(temp);
     temp = fread(2*count++);
     Ball[i].oy=Int2Fixed(temp);
     temp = fread(2*count++);
     Ball[i].oz=Int2Fixed(temp);
     }
     */
  }

  // update vector ball object
  void UpdateBalls()
  {

    XAngle+=1;

    //  117 = chicken
    XAngle=117; // x axists eye level

    if(XAngle >= MAXDEGREES)
    {
      XAngle -= MAXDEGREES;
    }

    YAngle+=1;

    if(YAngle >= MAXDEGREES)
    {
      YAngle -= MAXDEGREES;
    }

    ZAngle+=1;

    if(ZAngle >= MAXDEGREES)
    {
      ZAngle -= MAXDEGREES;
    }


    Fixedpoint nx,ny,nz;
    Fixedpoint sinxangle,cosxangle;
    Fixedpoint sinyangle,cosyangle;
    Fixedpoint sinzangle,coszangle;

    // get the sine and cosine angles to save time from table lookup
    sinxangle=sine[XAngle];
    cosxangle=cosine[XAngle];
    sinyangle=sine[YAngle];
    cosyangle=cosine[YAngle];
    sinzangle=sine[ZAngle];
    coszangle=cosine[ZAngle];

    // working copy of x,y,z coordinates of ball
    Fixedpoint wx;
    Fixedpoint wy;
    Fixedpoint wz;    

    Fixedpoint ox;
    Fixedpoint oy;
    Fixedpoint oz;    
    int count=1;

    // rotate the balls
    for(byte i=0;i<NumOfBalls;i++)
    {   

      // get origional x,y,z coordinates of ball from FLASH
      ox=Int2Fixed(fread(2*count++));
      oy=Int2Fixed(fread(2*count++));
      oz=Int2Fixed(fread(2*count++));

      // get origional x,y,z coordinates of ball from SRAM (2FPS faster)
      /*
       oz =Ball[i].oz;
       oy =Ball[i].oy;
       ox =Ball[i].ox;
      */

      // rotate around the x-axis    
      wx = FixedMul(oy,cosxangle) - FixedMul(oz,sinxangle);
      wy = FixedMul(oy,sinxangle) + FixedMul(oz,cosxangle);
      wz = ox;

      // rotate around the y-axis
      nx=FixedMul(wx,cosyangle) - FixedMul(wz,sinyangle);
      nz=FixedMul(wx,sinyangle) + FixedMul(wz,cosyangle);

      wx=nx;
      wz=nz-DEPTH;

      // rotate around the z-axis
      nx=FixedMul(wx,coszangle) - FixedMul(wy,sinzangle);
      ny=FixedMul(wx,sinzangle) + FixedMul(wy,coszangle);

      wx=nx;
      wy=ny;

      // project the 3-D coordinates to screen coordinates

      if (smalldotmode)
      {
        Ball[i].sx=Fixed2Int(FixedMul(FixedDiv(wx,wz),EYE_DISTANCE));//200;
        Ball[i].sy=Fixed2Int(FixedMul(FixedDiv(wy,wz),EYE_DISTANCE));//150;
        Ball[i].index=i;   // point each element to an initial ball      
      }
      else
      {
        Ball[i].sx=Fixed2Int(FixedDiv(wx,wz)<<5);
        Ball[i].sy=Fixed2Int(FixedDiv(wy,wz)<<5);
      }

      Ball[i].sz=Fixed2Int(wz);
    }

    // sort the balls according to their z coordinate
    qsort(Ball,NumOfBalls, 7,CompareBallsStruct);

    GD.__wstartspr(flip ? 256 : 0);      

    for(byte i=0; i<NumOfBalls; i++)
    {
      int sx = Ball[i].sx;
      int sy = Ball[i].sy;

      // for 16x16 sprite mode.  ideal for high dot count voxel model.
      if (smalldotmode)
      {
        //        uint32_t addr = GET_FAR_ADDRESS(flashdot1);
        //        draw_sprite_8(sx+200,sy+150, getByte(addr + i));        // pass color index instead of 0        
        draw_sprite_8(sx+200,sy+150, pgm_read_byte_near(&flashdot1[Ball[i].index]));        // pass color index instead of 0
      }

      // for 32x32 sprite mode
      else
      {
  
        int darkness = (Ball[i].sz + 4700) >> 6; 
        if (darkness>=15) darkness=15;
        if (darkness<=0) darkness=0;
  
//        int darkness = map (Ball[i].sz, 0,4096*6, 0, 15);
        draw_sprite_16(sx+200,sy+150, darkness);    
      }
    }
   GD.__end();
//    GD.waitvblank(); 
    GD.wr(SPR_PAGE, flip);
    flip = !flip;
  }

private:
  const char *flashdot0;
  const char *flashdot1;
  boolean smalldotmode;

  int XAngle;  // degrees to rotate object around x axis
  int YAngle;  // degrees to rotate object around y axis
  int ZAngle;  // degrees to rotate object around z axis
  int NumOfBalls; // current number of defined balls in object
  BallTYPE Ball[MAXBALLS];  // object containing MAXBALLS balls
  Fixedpoint cosine[MAXDEGREES];  // cosine lookup table
  Fixedpoint sine[MAXDEGREES];    // sine lookup table

  // create sin and cos lookup tables
  void InitSinCosTables(void)
  {
    for(int i=0; i<MAXDEGREES; i++)
    {
      cosine[i]=Float2Fixed(cos((float)i*360/MAXDEGREES * PI / 180.0));
      sine[i]  =Float2Fixed(sin((float)i*360/MAXDEGREES * PI / 180.0));
    }
  }

  void draw_sprite_8(int x, int y, byte anim) {
    GD.xsprite(x, y, -8, -8, anim, 0, 0, 0);
  }

  void draw_sprite_16(int x, int y, byte anim) {
    int a = (15-anim)*2;
    GD.xsprite(x, y, -16, -16, a, 0, 0, 0);
    GD.xsprite(x, y, 0, -16, a+1, 0, 0, 0);
    GD.xsprite(x, y, -16, 0, 32+a, 0, 0, 0);
    GD.xsprite(x, y, 0, 0, 32+(a+1), 0, 0, 0);
  }

  // load a dot object for use as a vector ball object
  int fread(unsigned int offset)
  {
    return pgm_read_word_near(&flashdot0[offset]);
  }
};





