/*

 Name      :  Plane Deformation
 Notes     :  Plane deformation effects
 
 This is too slow to include.  .05 FPS. Need large lookup table
 
 2D plane deformations using lookup tables was a great way to get
 smooth animations of complex calculations running on slower processors.
 These effects were very common in the 90s demoscene. The idea was to
 take a texture and deform it using a pre-calculated math reference table.
 Different formulas gave different effects such as fake 3d tunnels,
 wormholes and landscapes. Based on formulas by Inigo Quilez.
 
 references:
 http://iquilezles.org/www/material/deform/deform.htm
 
 */


class Deform
{
public:
  Deform(){
    clearScreen();    

  };

  void init(int effect){
    createLUT(RENDERWIDTH,RENDERHEIGHT,effect);
          generate_332_palette();    
      GD.copy(RAM_PAL, space_pal, sizeof(space_pal));


  }

  // FASTEST - this writes/reads to gameduino spi memory once every Y
  void draw(int frameCount){
    do{
      int currentPixelIndex = 0; 
      for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
        for(int x = 0; x < RENDERWIDTH; x++) {        
          //fire_buffer[currentPixelIndex] =  pgm_read_byte(&meanman_32[(LUT[currentPixelIndex] + frameCount)&0x3ff]);
          fire_buffer[currentPixelIndex] =  pgm_read_byte(&space_pic[(LUT[currentPixelIndex] + frameCount)&0xfff]);          
          currentPixelIndex++;
        }
      }
      fb();
      screenshot();
    }
    while(--frameCount);
  }  



  // FASTEST - this writes/reads to gameduino spi memory once every Y
  void render(int frameCount){

      int currentPixelIndex = 0; 
      for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
        for(int x = 0; x < RENDERWIDTH; x++) {        
          //fire_buffer[currentPixelIndex] =  pgm_read_byte(&meanman_32[(LUT[currentPixelIndex] + frameCount)&0x3ff]);
          fire_buffer[currentPixelIndex] =  pgm_read_byte(&space_pic[(LUT[currentPixelIndex] + frameCount)&0xfff]);          
          currentPixelIndex++;
        }
      }
      fb();
  }  


private:  

  byte LUT[RENDERWIDTH*RENDERHEIGHT];

  void createLUT(int w, int h, int effectStyle){
    byte timeDisplacement=1;  // number of frames to run
    byte page=0;              // double buffer counter

    // u and v are euclidean coordinates  
    float u,v = 0; 
    int buffer = 0;
    for(int j = 0; j < h; j++) 
    {
      float y = -1.00 + 2.00*(float)j/(float)h;
      for(int i = 0; i < w; i++) 
      {
        float x = -1.00 + 2.00*(float)i/(float)w;
        float d = sqrt(x*x + y*y);
        float a = atan2(y,x);
        switch(effectStyle) {
        case 1:   // stereographic projection / anamorphosis 
          u = cos(a)/d;
          v = sin(a)/d;
          break;
        case 2:  // hypnotic rainbow spiral
          v = sin(a+cos(3*d))/(pow(d,.2));
          u = cos(a+cos(3*d))/(pow(d,.2));
          break;
        case 3:  // rotating tunnel of wonder
          v = 2/(6*d + 3*x);
          u = a*3/PI;
          break;
        case 4:  // wavy star-burst
          v = (-0.4/d)+.1*sin(8*a);
          u = .5 + .5*a/PI;
          break;
        case 5:  // hyper-space travel
          u = (0.02*y+0.03)*cos(a*3)/d;
          v = (0.02*y+0.03)*sin(a*3)/d;
          break;
        case 6:  // five point magnetic flare
          u = 1/(d+0.5+0.5*sin(5*a));
          v = a*3/PI;
          break;
        case 7:  // cloud like dream scroll
          u = 0.1*x/(0.11+d*0.5);
          v = 0.1*y/(0.11+d*0.5);
          break;
        case 8:  // floor and ceiling with fade to dark horizon
          u = x/abs(y);
          v = 1/abs(y);
          break;
        case 9:  // hot magma liquid swirl
          u = 0.5*(a)/PI;
          v = sin(2*d);
          break;
        case 10:  // clockwise flush down the toilet
          v = pow(d,0.1);
          u = (1*a/PI)+d;
          break;
        case 11:  // 3D ball
          v = x*(3-sqrt(4-5*d*d))/(d*d+1);
          u = y*(3-sqrt(4-5*d*d))/(d*d+1);
          break;
        default:  // show texture with no deformation
          u = x;
          v = y;
          break;
        }
        int uu =int(0xFF*u);    // to look like its animating, add timeDisplacement
        int vv =int(0xFF*v);

        // get the R,G,B values from texture
        //        byte r = vv&0xFF^uu&0xFF;
        //        byte g = 0;
        //        byte b = 0;

        //GD.wr(RAM_SPRIMG+(buffer++), (byte)uu);   // store into sprite memory of GD
        LUT[buffer++] = (byte)uu;
        
        
      }
    }
  }

};

/*

 // SLOW - this writes and reads gameduino spi memory on every pixel
 void deform1(){
 int w = 50;
 int h = 37;
 createLUT(w,h,5);
 byte pixel_buffer[w];
 int timeDisplacement=800;  // number of frames to run
 int bufferY = (h*64)+RAM_PIC;
 
 do{
 int currentPixelIndex = 0; 
 for (int y = RAM_PIC; y < bufferY;y+=64) { 
 for(int x = 0; x < w; ) {        
 GD.wr(x++ + y, pgm_read_byte(&meanman_32[(GD.rd(RAM_SPRIMG+currentPixelIndex++) + timeDisplacement)&0x3ff]) );
 }
 }
 }
 while(timeDisplacement+=-2);
 }
 
 // FASTER - this writes to gameduino spi memory once every Y and reads memory on every pixel
 void deform2(){
 int w = 50;
 int h = 37;
 createLUT(w,h,5);
 byte pixel_buffer[w];
 int timeDisplacement=900;  // number of frames to run
 int bufferY = (h*64)+RAM_PIC;
 do{
 int currentPixelIndex = 0; 
 for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
 for(int x = 0; x < w; x++) {        
 pixel_buffer[x] = pgm_read_byte(&meanman_32[(GD.rd(RAM_SPRIMG+currentPixelIndex++) + timeDisplacement)&0x3ff]);
 }
 GD.copy2(y,pixel_buffer,w);
 }
 }
 while(timeDisplacement+=-1);
 }
 
 */









