/*

 Name        : Space Madness                                        
 Author      : Luis Gonzalez - luis@trsi.de
 Date        : 28 March, 2013                                      
 Version     : 1.0                                               
 Platform    : Arduino Mega 2560 - Atmel ATMEGA368                                   
 Environment : Arduino 1.0                                      
 Notes       : Old-school demoscene effects with MOD music playback
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
 
 */

#include "modformat.h"
#include "common.h"

#include <GD.h>
#include <SPI.h>
#include <digitalWriteFast.h>
#include <TimerOne.h>


#include <avr/pgmspace.h>

#include <PS2Keyboard.h>
const int DataPin = 7;
const int IRQpin =  21;
PS2Keyboard keyboard;

static signed int sampleleft;
static signed int sampleright;

#include "textwav.h"
#include "sfx.h"
#include "modplayer.h"
#include "MemoryFree.h"
#include "meanman_32.h"

/* effects */
#include "sinescroller.h"
#include "cube.h"
#include "copper.h"
#include "fire.h"
#include "twister.h"
#include "rotozoomer.h"
#include "deform.h"
#include "julia.h"
#include "subpixel.h"
#include "plasma.h"
#include "sm_logo.h"
#include "StarField2D.h"
#include "StarField3D.h"
#include "vectorball.h"
#include "tile.h"
#include "ascii.h"


/* common audio */

#include "soundbuffer.h"
#define SOUNDBUFFERHI 0x3f00
#define SOUNDBUFFERLO 0x4000
#define SOUNDBUFFER 0x3f00
byte writepointer = 0;

static uint8_t nextPwm;

#include "talkie.h"
//Talkie voice;

/*
void fillTalkSampleBuffer(){
 byte readpointer = GD.rd(COMM+0);
 byte fullness = writepointer - readpointer;
 
 while (fullness < 254  ) {
 voice.fillTalkSample();
 GD.wr(SOUNDBUFFER  + writepointer,(nextPwm));                 
 writepointer++;
 fullness++;
 } 
 }
 */


void fillSampleBuffer(){
  byte readpointer = GD.rd(COMM+0);
  byte fullness = writepointer - readpointer;

  while (fullness < 254  ) {
    fillSample();
    writeSamples(1);
        GD.wr(SOUNDBUFFER  + writepointer, mastersample+sampleright+sampleleft);                 
//    GD.wr(SOUNDBUFFER  + writepointer, mastersample+sampleright+sampleleft);                 
    writepointer++;
    fullness++;
  } 
}

void setup(){

  GD.begin();
  //   initSong();
      GD.microcode(soundbuffer_code, sizeof(soundbuffer_code));
      modload();
  
      uint32_t  addr = GET_FAR_ADDRESS(subi_songdata);
   //   modload(addr);
    Timer1.initialize(20000);         // for subi song  
    Timer1.attachInterrupt(fillSampleBuffer);  // attaches callback() as a timer overflow interrupt

  //  Serial.begin(115200);

  keyboard.begin(DataPin, IRQpin);

/* for end
      ending();
      hideSprites();
      runAscii((256*8) + 3*(8*38) ); // ascii is 256 lines long x 8 pixel tiles
*/


 //     runAscii((256*8) + 3*(8*38) ); // ascii is 256 lines long x 8 pixel tiles
//runCopper(600);
}

//void loop(){}

void loop(){
  
  //intro();

  map_load_Outside();   
  draw_sprite(player_x, player_y, PLAYER_DOWN_1, 0, 0,0);  // player starting pos 

  int item=0;
  while (1) 
  {
    if (keyboard.available()) {

      char key = keyboard.read();
      // read the next key
     movePlayer(key,1);

      switch (key)
      {
      case PS2_ESC:
        item = runmenu(5,5, mainmenu, item,true);

        switch (item) {
        case 1:  
          saySomething();
          break;
        case 2:
          break;
        case 3:
          menu1();   // effects menu
          break;

        case 4:  // battle
          hideSprites();
          fadeAll();
          map_load_Battle();          
                      // item = runmenu(5,5, submenu1, item,true);        
          break;
        case 5:  
          break;
        case 6:   
          break;
        case 7:   
          break;
        }
        break;  
      }

    }   
  }
}


/* goal get all effects running without memory loss */
void loop2() {


  delay(1200);
  // runBigVector(300,vector_largebox_dots);
  //  runSmallVector(200,vector_dna_dots,vector_dna_color);
  //  runSmallVector(200,vector_chicken_dots,vector_chicken_color);
  //  runSmallVector(200,vector_rings_dots,vector_rings_color);


  //  runStars2D(600); 
  //  runAscii( (256*8)  ); // 35 seconds - ascii is 256 lines long x 8 pixel tiles

  // clearScreen();  

  runSubpixel(200);  
  //  runJulia(200); 
  //  runFire(500);
  runTwister(200);
  //  runCopper(200);

  //  runDeform(4,200);    
  //  runDeform(5,200);    
  //  runDeform(7,200);    
  //  runDeform(9,200);    

  //  runRotozoomer(200);
  //  plasma();
  //  metaball2();  
  //  intro();
}


void runTwister(int framesToRun){
  Twister* twister;
  twister = new Twister();
  twister->draw(framesToRun);
  delete(twister);
}

void runCopper(int framesToRun){


//  copper->draw(framesToRun);
//  delete(copper);


  Deform* deform;
  deform = new Deform();


  
  deform->init(5);  // 4,5,7,9
  //deform->draw(framesToRun);  
  //delete(deform);


 Copper* copper;
 copper = new Copper();



/*
  Copper* copper;
  copper = new Copper();

  Deform* rotozoomer;
  deform = new Deform();
*/

  do{  
    deform->render(framesToRun);
    copper->render(framesToRun);

    fb();
  }
  while(--framesToRun);

  delete(copper);
  delete(deform);


}

void runDeform(int effect, int framesToRun){
  Deform* deform;
  deform = new Deform();
  deform->init(effect);
  deform->draw(framesToRun);  
  delete(deform);
}

void runRotozoomer(int framesToRun){

  Subpixel* subpixel;
  subpixel = new Subpixel();

  Rotozoomer* rotozoomer;
  rotozoomer = new Rotozoomer();


  do{  
    rotozoomer->render1(framesToRun);
    subpixel->render();

    fb();
  }
  while(--framesToRun);

  delete(subpixel);
  delete(rotozoomer);
  //  fade();
}

void runJulia(int framesToRun){
  Julia* julia;
  julia = new Julia();
  do{  
    julia->draw();  
    fb();
  }
  while(--framesToRun);


// end transition
  framesToRun=50;  // number of frames to run
   do
   { 
   julia->draw();  
   crossFade(framesToRun);
   fb();
   screenshot();
   }
   while(framesToRun -= 2);


  delete(julia);
}


void runFire(int framesToRun){

  Cube* cube; // declare variable
  cube = new Cube(); // create instance of class C

  Fire* fire; // declare variable
  fire = new Fire(); // create instance of class C

  Sinescroller* sinescroller; // declare variable
  sinescroller = new Sinescroller(); // create instance of class C

  do{

    sinescroller->render(); // call method M          
    cube->draw();    
    fire->render(); // call method M  
    fb();
    screenshot();
  }
  while(--framesToRun);

  fire->draw(15);  
  delete(cube); // free memory        
  delete(fire); // free memory      
  delete(sinescroller); // free memory       
}


void runSubpixel(int framesToRun){
  Plasma* plasma;
  plasma = new Plasma();

  Subpixel* subpixel;
  subpixel = new Subpixel();

  do{
    plasma->render();
    subpixel->render();
    fb();
    //    screenshot();
  }
  while(--framesToRun);

  delete(plasma);  
  delete(subpixel);
}

void runStars3D(int framesToRun){
  StarField3D* stars;
  stars = new StarField3D();

  do{
    stars->draw();
    //    screenshot();
  }
  while(--framesToRun);
  delete(stars);  
  clearScreen();
}

void runStars2D(int framesToRun){
  StarField2D* stars;
  stars = new StarField2D();

  do{
    stars->draw();
    //    screenshot();
  }
  while(--framesToRun);
  fade();
  delete(stars);  
  clearScreen();
}

void runAscii(int framesToRun){
  Ascii* ascii;
  ascii = new Ascii(framesToRun/8);
  do{
    ascii->draw();
    //    screenshot();
  }
  while(--framesToRun);
  delete(ascii);  

  fadeAll();
  clearScreen();
  GD.wr16(SCROLL_Y, 0);   
}

void runSmallVector(int framesToRun, const char *dotData, const char *dotColorData){

  Vectorball* vectorball;
  vectorball = new Vectorball();

  vectorball->LoadObject(dotData,dotColorData);
  vectorball->loadSmallDots();

  do{
    
    vectorball->UpdateBalls();
  }
  while(--framesToRun);

  delete(vectorball);   
}

void runBigVector(int framesToRun, const char *dotData){
  Vectorball* vectorball;
  vectorball = new Vectorball();

  vectorball->LoadObject(dotData);
  vectorball->loadLargeDots();

  do{
    vectorball->UpdateBalls();
    //    screenshot();
  }
  while(--framesToRun);

  delete(vectorball);   
}

void showMemory(){
  char msg[30];
  int mem = freeMemory();
  sprintf(msg, "free mem %d", mem);
  GD.putstr(0, 0, msg);
}

// intro voice
/*
void runIntroSpeech(){
 // initialize timer1, and set the period in microseconds.     
 Timer1.initialize(28000);         
 
 // attaches callback() as a timer overflow interrupt  
 Timer1.attachInterrupt(fillTalkSampleBuffer);  
 
 Talkie* voice;
 voice = new Talkie();
 
 //  voice->say(spDINER);
 voice->say(lpc_intro);
 voice->say(lpc_invite);
 
 // stop calling the audio buffer
 Timer1.stop();
 Timer1.detachInterrupt();
 
 // silence woman!  
 delete(voice);  
 }
 */



