/*

 Name      :  Metaball
 Notes     :  organic-looking n-dimensional objects. 
 
 The technique for rendering metaballs was invented by Jim Blinn in the early 1980s
 
 references:
 http://www.benryves.com/tutorials/blobs/    
 http://www.student.kuleuven.be/~m0216922/CG/index.html
 
 */
void metaball(){

  byte w = 50;
  byte h = 37;


  byte numBlobs = 2;          // two blobby blobs
  byte blob_px[] = {
    0, 20    };   // where blobs will first show
  byte blob_py[] = {
    10, 0            };

  // Movement vector for each blob
  char blob_dx[] = {
    1, 1            };
  char blob_dy[] = {
    1, 1            };

  // build square lookup table
  unsigned int vy[w];
  for (int i = 0; i < w; i++) {
    vy[i] = sq(i);
  }

  metaPalette();

  int m = 0;
  byte timeDisplacement=140;     // number of frames to run

  do
  {
    for (byte i=0; i<numBlobs; ++i) {
      blob_px[i]+=blob_dx[i];
      blob_py[i]+=blob_dy[i];

      // bounce across screen
      if (blob_px[i] < 5) {
        blob_dx[i] = 1;
      }
      if (blob_px[i] >= w-5) {
        blob_dx[i] = -1;
      }
      if (blob_py[i] < 5) {
        blob_dy[i] = 1;
      }
      if (blob_py[i] > h-5) {
        blob_dy[i]=-1;
      }
    }

    int buffer=0;
    for (byte y = 0; y < h; y++) {
      for (byte x = 0; x < w; x++) { 
        m = 1;
        for (byte i = 0; i < numBlobs; i++) {
          // Increase 10K number to make your blobs bigger 
          m += 10000/(vy[abs(blob_px[i]-x)] + vy[abs(blob_py[i]-y)]+1);
        }
        if (m>255) m=255; // this prevents center of metballs from junking up (overflowed colors)

        //        GD.wr(RAM_PIC+x+buffer,meta_palette[m]);   //color(m,0,m)
        GD.wr(RAM_PIC+x+buffer,m);   //color(m,0,m)
      }
      buffer+=64;
    }
  }
  while(--timeDisplacement);
}

void metaPalette(){
  // byte meta_palette[256];
  // generate flame color palette in RGB. need 256 bytes available memory
  for(int i=0; i<256; i++)
  {
    //  meta_palette[i]  = color(i,0,i);      // Black to red
    GD.setpal(i*4, RGB(i,0,i));   
  }      
}

void metaball2(){
  byte w = 50;
  byte h = 37;

  byte numBlobs = 2;          // two blobby blobs
  byte blob_px[] = {0, 20};   // where blobs will first show
  byte blob_py[] = {10, 0};

  // Movement vector for each blob
  char blob_dx[] = {
    1, 1            };
  char blob_dy[] = {
    1, 1            };

  byte LUT[w][h];
  
 // byte pixel_buffer[w];

  // clear screen of character data  
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 4096); // second number is color 0-3  



//  generate_332_palette();
  metaPalette();
  
  
  for(byte i=0; i<86; i++)
  {
    GD.setpal(i*4, RGB(i*3,0,0));          // Black to red
    GD.setpal((i+85)*4, RGB(255,i*3,0));   // Red to yellow
    GD.setpal((i+170)*4, RGB(255,255,i*3));   // yellow to white
  } 
 // GD.ascii();


  for (int y = 0; y < h; y++) {
    for (int x = 0; x < w; x++) {
      // Increase 10K number to make your blobs bigger 
      int temp = 10000/((x*x) + (y*y));
      
      // this prevents center of metballs from junking up (overflowed colors)
      if (temp > 255) temp = 255;
      
      LUT[x][y] = temp;
    }
  }



  int m = 0;
  int timeDisplacement=340;     // number of frames to run

  do
  {
    

    for (byte i=0; i<numBlobs; ++i) {
      blob_px[i]+=blob_dx[i];
      blob_py[i]+=blob_dy[i];

      // bounce across screen
      if (blob_px[i] < 5) {
        blob_dx[i] = 1;
      }
      if (blob_px[i] >= w-5) {
        blob_dx[i] = -1;
      }
      if (blob_py[i] < 5) {
        blob_dy[i] = 1;
      }
      if (blob_py[i] > h-5) {
        blob_dy[i]=-1;
      }
    }


    int buffer=0;
    for (byte y = 0; y < h; y++) {
      for (byte x = 0; x < w; x++) { 
        m = 1;


        for (byte i = 0; i < numBlobs; i++) {
         m += LUT[abs(blob_px[i]-x)][abs(blob_py[i]-y)];
         }
         if (m>255) m=255; // this prevents center of metballs from junking up (overflowed colors)

     //   m += LUT[abs(blob_px[0]-x)][abs(blob_py[0]-y)];
//        m += LUT[abs(blob_px[1]-x)][abs(blob_py[1]-y)];

        //        GD.wr(RAM_PIC+x+buffer,meta_palette[m]);   //color(m,0,m)
         
       // pixel_buffer[x] = m;
        GD.wr(RAM_PIC+x+buffer,m);   //color(m,0,m)        
      }
      //GD.copy2(buffer,pixel_buffer,w);
      buffer+=64;
    }
  }
  while(--timeDisplacement);


}








void metaball3() {
  byte w = 50;
  byte h = 37;

  int centerx = (8 * 25);
  int centery = (8 * 17);

  byte numBlobs = 4;
  struct {
    int x, y;
    signed char dx, dy;
  } blobs[numBlobs];
  for (byte i=0; i<numBlobs; ++i) {
    blobs[i].x = random(2*centerx);
    blobs[i].y = random(2*centery);
    blobs[i].dx = random(32) - 16;
    blobs[i].dy = random(32) - 16;
  }

  for(byte i=0; i<86; i++)
  {
    GD.setpal(i*4, RGB(i*3,0,0));          // Black to red
    GD.setpal((i+85)*4, RGB(255,i*3,0));   // Red to yellow
    GD.setpal((i+170)*4, RGB(255,255,i*3));   // yellow to white
  } 

  GD.wr16(RAM_SPRPAL, TRANSPARENT);
  GD.__wstart(RAM_SPRIMG);
  for (byte r = 0; r < 64; r++)
    for (byte x = 0; x < 16; x++)
      for (byte y = 0; y < 16; y++)
        SPI.transfer((SQ(8-x) + SQ(8-y)) < r);
  GD.__end();

  int timeDisplacement=20*72;     // number of frames to run

  int recipsz = (w*w + h*h) / 4 + 1;
  byte recip[recipsz];
  for (int i = 0; i < recipsz; i++)
    recip[i] = min(255, 10000 / (4 * i));

  byte fadein = 255;
  do
  {
    for (byte i=0; i<numBlobs; ++i) {
      if (blobs[i].x < centerx)
        blobs[i].dx++;
      else
        blobs[i].dx--;
      if (blobs[i].y < centery)
        blobs[i].dy++;
      else
        blobs[i].dy--;
      blobs[i].x += blobs[i].dx;
      blobs[i].y += blobs[i].dy;
    }
    blobs[random(numBlobs)].dx = random(32) - 16;
    blobs[random(numBlobs)].dy = random(32) - 16;

    for (byte y = 0; y < h; y++) {
      GD.__wstart(RAM_PIC + (y << 6));
      for (byte x = 0; x < w; x++) { 
        int m = fadein;
        for (byte i = 0; i < 3; i++) {
           int dx = (blobs[i].x >> 3) - x;
           int dy = (blobs[i].y >> 3) - y;
           int d = SQ(dx) + SQ(dy);
           m += recip[min(d >> 2, recipsz - 1)];
        }
        SPI.transfer(min(255, m));
      }
      GD.__end();
    }
    
    /*
    GD.__wstart(RAM_SPR);
    for (byte i = 3; i < numBlobs; i++) {
      int x = blobs[i].x;
      int y = blobs[i].y;
      SPI.transfer(lowByte(x));
      SPI.transfer((highByte(x) & 1));
      SPI.transfer(lowByte(y));
      SPI.transfer((i & 0x7e) | (highByte(y) & 1));
    }
    GD.__end();
    */
    
    fadein = max(fadein - 3, 1);
  }
  while(1 || --timeDisplacement);
}


