/*

 Name      :  Plasma 2
 Notes     :  Cycles of changing colours warped to give an illusion of liquid, organic movement.
 
 Precalculated plasma effect values stored in flash rom array.  Colors are the sum of sine functions and various formulas.   
 slightly slower that plasma 1
 
 */


void plasma(){

  int GRADIENTLEN = 256;

  // swing/wave function parameters
  int SWINGLEN = GRADIENTLEN*3;
  int SWINGMAX = GRADIENTLEN / 2 ;
  int swingCurve[SWINGLEN-1];

  // gradient & swing curve arrays
  makePlasmaPalette(GRADIENTLEN);

  // default values will be used upon first call
  int factor1=2;
  int factor2=3;
  int factor3=6;

  int halfmax = SWINGMAX/factor1;


  for( int i=0; i<SWINGLEN; i++ ) {
    float ni = i*TWO_PI/SWINGLEN; // ni goes [0..TWO_PI] -> one complete cos wave
    swingCurve[i]=(int)(
    cos( ni*factor1 ) *
      cos( ni*factor2 ) *
      cos( ni*factor3 ) *
      halfmax + halfmax );
  }  

  byte w = 50;
  byte h= 37;
  int timeDisplacement=600;  // number of frames to run
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];


//  Vectorball2 v;
//  v.init();      

  do
  {  
//    v.vectorball(timeDisplacement);                
    
    int t = timeDisplacement;
//    int swingT = swingCurve[t & (SWINGLEN - 1)]; 
//    int swingT = swing(t);
    
    int swingT = swingCurve[t & (SWINGLEN-1)];
    int yy=0;

    for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
      int swingY  = swingCurve[yy & (SWINGLEN - 1)];
      int swingYT = swingCurve[(yy+t) & (SWINGLEN - 1)];
      yy++;
      for (int x = 0; x < w;) {
        // this is where the magic happens: map x, y, t around
        // the swing curves and lookup a color from the gradient
        // the "formula" was found by a lot of experimentation

        // pixel_buffer[x++] = color(0,255,0);
//        pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) % SWINGLEN] + swingYT) % SWINGLEN] + swingCurve[(swingCurve[(x + timeDisplacement) % SWINGLEN] + swingY) % SWINGLEN]) ;
        pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) & (SWINGLEN - 1)] + swingYT) & (SWINGLEN - 1)] + swingCurve[(swingCurve[(x + timeDisplacement) & (SWINGLEN - 1)] + swingY) & (SWINGLEN - 1)])  ;
//       pixel_buffer[x++] = (byte)(swingCurve[(swingCurve[(x + swingT) ] + swingYT) ] + swingCurve[(swingCurve[(x + t) ] + swingY) ]) ;
      }
      GD.copy2(y,pixel_buffer,w);
    }

  }
  while(timeDisplacement+=-1);
  
//  v.clearball();        
  
}

// helper: get cosinus sample normalized to 0..255
int cos256( int amplitude,  int x) {
  return (int) (cos(x * TWO_PI / amplitude) * 127 + 127);
}

// helper: get a swing curve sample
int swing(int i) {
  //return swingCurve[i % SWINGLEN];
}

// helper: get a gradient sample
int gradient(int i) {
  //	return colorGrad[i % GRADIENTLEN];
}

// make gradient 
void makePlasmaPalette(int arrlen){
  // default values will be used upon first call
  int rf = 3;
  int gf = 3;
  int bf = 1;
  int rd = 0;
  int gd = arrlen / gf;
  int bd = arrlen / bf / 2;
  
  
  rd = 160;
  gd = 217;
  bd = 178;

  // fill gradient array
  for (int i = 0; i < arrlen; i++) {
    int r = cos256(arrlen / rf, i + rd);
    int g = cos256(arrlen / gf, i + gd);
    int b = cos256(arrlen / bf, i + bd);
    GD.setpal(i*4, RGB(r,g,b)); 
  }
  
  
  

//  showPalette();
//  delay(2000);  
}


void plasma2(){
  byte w = 50;
  byte h= 37;
  int timeDisplacement=1000;  // number of frames to run
//  int bufferY = (h*64)+RAM_PIC;
  byte pixel_buffer[w];
  
  float s1,s2;
  for (int i=0;i<128;i++) {
    s1=sin(i*PI/25);
    s2=sin(i*PI/50+PI/4);
    GD.setpal(i*4, RGB((int)(128+s1*128),(int)(128+s2*128),(int)(s1*128))); 
  }
  
  int cls[w*h];
  for (int x = 0; x < w; x++)
  {
    for (int y = 0; y < h; y++)
    {
      cls[x+y*w] = (int)((127.5 + +(127.5 * sin(x / 32.0)))+ (127.5 + +(127.5 * cos(y / 32.0))) + (127.5 + +(127.5 * sin(sqrt((x * x + y * y)) / 32.0)))  ) / 4;
    }
  }

  do{
  int currentPixelIndex = 0; 
     for (unsigned int y = RAM_PIC; y < bufferY;y+=64) { //divide by 64 to get vertical resolution
      for(int x = 0; x < w; x++) {      
               pixel_buffer[x] = (cls[currentPixelIndex++] + timeDisplacement)&127;
      }
      GD.copy2(y,pixel_buffer,w);
    }
  }
  while(timeDisplacement+=-1);  
}

