#include <avr/pgmspace.h>


typedef unsigned char	u8;
typedef char			s8;
typedef unsigned short	u16;
typedef short			s16;
typedef unsigned long	u32;
typedef long			s32;

#define NUM_NOTES (12*3)
extern const u16 periodTable[NUM_NOTES];// PROGMEM;

const u16 periodTable[NUM_NOTES]/* PROGMEM*/ =
{
    856,808,762,720,678,640,604,570,538,508,480,453, // C-1 to B-1
    428,404,381,360,339,320,302,285,269,254,240,226, // C-2 to B-2
    214,202,190,180,170,160,151,143,135,127,120,113  // C-3 to B-3
};

#define MOD_FLG_CLEAR		0b00000000
#define MOD_FLG_UPDATEVOL   0b00000001
#define MOD_FLG_UPDATEPITCH 0b00000010
#define MOD_FLG_UPDATEINST  0b00000100
#define MOD_FLG_UPDATEPAN   0b00001000
#define MOD_FLG_PLAYNOTE	0b00010000
#define MAX_PERIOD 856
#define MIN_PERIOD 113
#define NO_PERIOD 0xFFFF
#define NO_NOTE			0x3Fu
#define NO_INST_PATTERN 0x00u
#define NO_INST         0xFFu
#define DEFAULT_SPEED 6
#define DEFAULT_BPM   125
#define NOTE_SIZE 3
#define PATTERN_CHANNELS 4
#define PATTERN_ROWS 64
#define PATTERN_SIZE ( NOTE_SIZE * PATTERN_CHANNELS * PATTERN_ROWS )
#define MAX_VOLUME 63
#define MIN_VOLUME 0
#define NULL ((void*)0)
              
//#define F_DAC 15600UL
#define F_DAC 8000UL

#define AMIGA_DIV /*7093789UL*/ 7159091UL
#define PeriodToHz(T) (AMIGA_DIV / (T<<1))
#define HzToPeriod(h) (((u16)(AMIGA_DIV / h))>>1)

#if (F_CPU/(8*F_DAC)-1) > 255
#	define TMRDIV 64
#else
#	define TMRDIV 8
#endif

#define TMROCR (F_CPU/(TMRDIV*F_DAC)-1)
#define SREAL (F_CPU/(TMRDIV*(TMROCR+1)))
#define AMIGAF (256*AMIGA_DIV)/SREAL

#if AMIGAF <= 65535
#	define DPT u16
#else
#	define DPT u32
#endif


//#define PeriodToInc(period) (AMIGAF/period)>>1
#define ABS(n) ((n > 0) ? n : -n)
#define BPM(n) (bpmtab[n-32])

// Convenience macros
#define LO4( V ) ( ( (V) & 0x0F )      )
#define HI4( V ) ( ( (V) & 0xF0 ) >> 4 )

#define MOD_PAT_BREAK 0b00000001
#define MOD_PAT_JUMP  0b00000010
#define	SONG_STATE_STOP 0
#define	SONG_STATE_PLAY 1



/** Pattern Packing Format **
 * 	*  uses 3 bytes per note
 * 	*     -00-     -01-     -02-
 * 	*  |  BYTE  |  BYTE  |  BYTE  |
 * 	*   AAAAAABB BBCCCCCX DDDDDDDD
 * 	*
 * 	*  NOTE NUMBER = AAAAAA
 * 	*  EFFECT      = BBBB
 * 	*  INST NUMBER = CCCCC
 * 	*  EFFECT PARM = DDDDDDDD
 * 	*  UNUSED      = X
 */
struct note_s
{
  u8 number;
  u8 effect;
  u8 inst;
  u8 param;	
};

struct pattern_s
{
  u8 number;
  u16 offset;
  u16 index;
};

struct update_s
{
  struct channel_s*		chn;
  struct audiochannel_s*  sndchn;
  struct note_s			note;
  u8						flags;
};

struct sections_s
{
  u8		numinsts;
  u16		offsetinsts;
  u8		numorders;
  u16		offsetorders;
  u8		numpatterns;
  u16		offsetpatterns;
  u8		numsamples;
  u16		offsetsamples;
  u8		numnames;
  u16		offsetnames;
};

struct sections_s sections;

struct vibrato_s
{
  u8 speed;
  u8 depth;
  u8 waveform;
  s8 pos;
  s16 slide;
};

struct channel_s
{
  u8 inst;
  s8 volume;	// current volume on channel
  u8 pan;
  u8 note;	// last played note on channel
  u8 effect;
  u8 param;
  u16 freq;	// frequency (increment) on channel
  u8 arptick;
  u8 retrigtick;
  u8 lastSampleNumber;
  u8 portaspeed;
  u16 portaperiod;
  s8 looprow;
  u8 loopcount;
  struct vibrato_s vib;
  struct vibrato_s trem;
};

struct audiochannel_s
{
  u16 offset;		// current offset in file for sample. a zero means inactive channel	
  u32 pos;		// fixed point position, >>8 for real pos
  u16 inc;		// current increment
  u8  volume;
  u8 pan;
  u32 length;
  u32 looplength; 
};

struct inst_s
{
  u16 length;
  s8 finetune;
  u8 volume;
  u16 loopstart;
  u16 looplength;
};



void efx0_row(struct update_s*);
void efx1_row(struct update_s*);
void efx2_row(struct update_s*);
void efx3_row(struct update_s*);
void efx4_row(struct update_s*);
void efx5_row(struct update_s*);
void efx6_row(struct update_s*);
void efx7_row(struct update_s*);
void efx8_row(struct update_s*);
void efx9_row(struct update_s*);
void efxA_row(struct update_s*);
void efxB_row(struct update_s*);
void efxC_row(struct update_s*);
void efxD_row(struct update_s*);
void efxE_row(struct update_s*);
void efxF_row(struct update_s*);
void efx0_tick(struct update_s*);
void efx1_tick(struct update_s*);
void efx2_tick(struct update_s*);
void efx3_tick(struct update_s*);
void efx4_tick(struct update_s*);
void efx5_tick(struct update_s*);
void efx6_tick(struct update_s*);
void efx7_tick(struct update_s*);
void efx8_tick(struct update_s*);
void efx9_tick(struct update_s*);
void efxA_tick(struct update_s*);
void efxB_tick(struct update_s*);
void efxC_tick(struct update_s*);
void efxD_tick(struct update_s*);
void efxE_tick(struct update_s*);
void efxF_tick(struct update_s*);


typedef void (*pEffect)(struct update_s*);
