/*

 Name      :  Sinescroller
 Notes     :  sinewave text dypp scroller
 
 references:
 http://www.student.kuleuven.be/~m0216922/CG/fire.html 
 
 */


/* sinescroller */
#include "fontShort.h"
//volatile char scrollText[] = "  TRSI    DESIRE  "; 

const PROGMEM char scrollText[] = "TESTING ONE TWO";

Sinescroller::Sinescroller(){
  initFont();
  initScrollText();
  initSine();  
}

// loads the classic 16x16 font into font[]
// classic_16x16 has 46 glyphs:
//  !'(),-.:?0123456789abcdefghijklmnopqrstuvwxyz
//  http://www.flickr.com/photos/cumbrowski/sets/72157611369269727/
void Sinescroller::initFont(){
  // set up the font lookup table
  // fontLookup[character] = glyph number
  char sc[] = " !'(),-.:?0123456789abcdefghijklmnopqrstuvwxyz";

  int i;
  for (i = 0; i < 256; i++) fontLookup[i] = 0; // space
  for (i = 0; i < 46; i++)
    fontLookup[(char) sc[i]] = (char) i;

  char c;
  for (c = 'A'; c <= 'Z'; c++) // also do A-Z, not just a-z
    fontLookup[c] = (char)((int)(c-'A') + 20);

  int bmpcount=0;
  for (i = 0; i < sizeof(font)/2; i++) {

//    unsigned char TempA = FONTSHORT[bmpcount++]; 
//    unsigned char TempB = FONTSHORT[bmpcount++]; 
//    short arr = TempB  +  (TempA << 8) ;
//    font[i] = arr;


    unsigned char TempA = pgm_read_byte_near(&FONTSHORT[bmpcount++] ); 
    unsigned char TempB = pgm_read_byte_near(&FONTSHORT[bmpcount++] ); 
    short arr = TempB  +  (TempA << 8) ;
    font[i] = arr;
  }
}

// creates scrollText[], which is an array of glyph lookups rather
// than ascii text
void Sinescroller::initScrollText() {
  int i;
  for (i = 0; i < sizeof(scrollText); i++) {
   // scrollText[i] = fontLookup[scrollText[i]];
  }
}


// we use a lookup table for the sine wave rather than calculate it in
// realtime
void Sinescroller::initSine() {
  //  int sine[1024];
  // 0-1024 will be for us 0 to 4pi (720 degrees)
  // 1024 * x = 4pi, x=4pi/1024 = pi/(1024/4)  = pi/256
  //
  // the amplitude of the wave the screen height - 2, so there's a 1
  // pixel border top and bottom. But we also have to include the size
  // of the font, so the centre line running through the font has an
  // amplitude of screen height - 18 (16 pixel high font). 
  double ampl = ((RENDERHEIGHT) / 4)+1;
  int i;
  for (i = 0; i < sinelength; i++) {
    ssine[i] =    (int)((sin((double)i * 1.00 * (PI/128.0) ) *    ampl) + ampl+1)  ;
  }
}

void Sinescroller::sineScroller() {
  // obtain current scroll offset/text position and sine position,
  // then advance all of them.
  // scroll offset runs from 0 to 15. when it reaches 15, the
  // scroll position (the scrolltext character to be drawn) advances
  // and the scroll offset goes back to 0.
  int sco = scrollOffset, scp = scrollPos, sp = sinePos, y, ypos;
  if (++scrollOffset >= 16) {
    scrollOffset = 0;
    if (++scrollPos >= sizeof(scrollText)) scrollPos = 0;
  }

  sinePos += 4; 	// how fast text goes up and down
  if (sinePos >= sinelength) sinePos -= sinelength;

  // draw a series of 16-pixel high vertical lines


//  for (x = 0; x < RENDERWIDTH*RENDERHEIGHT; x++) {
 //   fire_buffer[x]=color(0,0,0);
//  }


  for (int x = 2; x < RENDERWIDTH; x++) {
    ypos = ssine[sp++]; 
    if (sp >= sinelength) sp = 0;




    int data = font[(fontLookup[pgm_read_byte_near(&scrollText[scp])] << 4) + sco]; // get scrolltext from flash memory 
//    int data = font[(scrollText[scp] << 4) + sco];
    
    if (++sco >= 16) {
      sco=0; 
      if (++scp >= sizeof(scrollText)) scp=0;
    }

/* this keeps the sine scroller stationary */
//  scrollPos = 21;
//  scrollOffset=16;



    for (y = 0; y < 16; y++) {
      if ((data & (1<<y)) != 0) {
//        fire_buffer[(y+ypos)*RENDERWIDTH + x] = rasters[y+ypos];




        fire_buffer[x+(y+ypos)*RENDERWIDTH ] = 220;//rasters[y+ypos];        

        
        
//        DrawChar(x,y+ypos,rasters[y+ypos]);
        
        // scroll shadow
//        if ((y+ypos+2 < RENDERHEIGHT) && (x+2 < RENDERWIDTH))
//        DrawChar(x+2,y+ypos+2,color(180,180,180));
        //	pD[(y+ypos+2)*RENDERWIDTH + x+2] = color(180,180,180);//0x000000;
      }
    }
  }
}


#define LRBLOCK (RENDERHEIGHT*64)+RAM_PIC // last visible  address on bottom right of screen
void fb(){
  unsigned int pixelCount =0;
  // character map is 64x64 but we only need to updated 50x37 so we skip every 64 Y blocks 
  for (unsigned int y = RAM_PIC; y < LRBLOCK;y+=64) { 
    byte fillWidth = RENDERWIDTH;
    GD.__wstart(y);
    while (fillWidth--)
    {
      SPI.transfer(fire_buffer[pixelCount++]);
    }
    GD.__end();
  }  
}










