#include <SPI.h>
#include <GD.h>
#include <digitalWriteFast.h>

#define RENDERWIDTH 50
#define RENDERHEIGHT 38
#include <stdlib.h> 


byte fire_buffer[RENDERWIDTH*RENDERHEIGHT];

#include "julia.h"

void setup()
{
  GD.begin();
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 0x0FFF); // second number is color 0-3  
}


void generate_332_palette()
{
  for (int c=0; c < 256; c++) {
    int r = (c&7) * 0xff/7;
    int g = ((c>>3)&7) * 0xff/7;
    int b = ((c>>6)&3) * 0xff/3;
    GD.setpal(c*4, RGB(r,g,b));
  }
} 

// uVGA palette RGB color conversion 
byte color(byte r, byte g, byte b){
  return ((b&255 & 0xC0)+((g&255 & 0xE0) >> 2)+((r & 0xE0) >> 5))&0xFF;
}

int frameCount=0;

void loop() {
  runJulia(200);
}

/* new hotness */
#define LRBLOCK (RENDERHEIGHT*64)+RAM_PIC // last visible  address on bottom right of screen

// fast version of fill buffer.   frame buffer as array. copy to screen characters
void fb(){
  unsigned int pixelCount =0;
  // character map is 64x64 but we only need to updated 50x37 so we skip every 64 Y blocks 
  for (unsigned int y = RAM_PIC; y < LRBLOCK;y+=64) { 
    byte fillWidth = RENDERWIDTH;
    GD.__wstart(y);
    while (fillWidth--)
    {
      SPI.transfer(fire_buffer[pixelCount++]);
    }
    GD.__end();
  }  
}


void DrawChar(byte x, byte y, byte newChar)
{
  fire_buffer[x+y*50]=newChar;    
}

void Screen5(int amount)
{  
  // All event rows start from the left
  for(int y = 0; y < RENDERHEIGHT-1; y += 2)
  {
    int dX = (RENDERWIDTH-amount)+1;
    if (dX<0)dX=0;
    RenderLine(0, y, dX, y, 0);        
  }
  for(int y = RENDERHEIGHT-1; y >= 0; y -= 2)
  {
    int dX = amount-2;
    if (dX<0)dX=0;
    RenderLine(RENDERWIDTH-1, y, dX, y, 0);
  }
}

void RenderLine(int x0, int y0, int x1, int y1, char lineChar)
{
  int dx = abs(x1-x0);
  int dy = abs(y1-y0);

  int sx, sy;

  if(x0 < x1)
    sx = 1;
  else
    sx = -1;
  if(y0 < y1)
    sy = 1;
  else
    sy = -1;

  int err = dx - dy;

  while(true)
  {
    DrawChar(x0, y0, lineChar);
    if(x0 == x1 && y0 == y1)
      break;
    int e2 = 2 * err;
    if(e2 > -dy)
    {
      err = err - dy;
      x0 = x0 + sx;
    }
    if(e2 < dx)
    {
      err = err + dx;
      y0 = y0 + sy;
    }
  }
}



void runJulia(int framesToRun){
  Julia* julia;
  julia = new Julia();
  do{  
    julia->draw();  
    fb();
  }
  while(--framesToRun);

  framesToRun=50;  // number of frames to run
  do
  { 
    julia->draw();  
    Screen5(framesToRun);
    fb();
  }
  while(framesToRun -= 2);

  delete(julia);
}



void* operator new(size_t size)
{
  void* mem = malloc(size);
  if (!mem) {
    //throw std::bad_alloc();
  }
  return mem;
}

void operator delete(void* ptr)
{
  free(ptr);
}

void* operator new[] (size_t size)
{
  return (operator new)(size);
}

void operator delete[](void* ptr)
{
  return (operator delete)(ptr);
}




