#include <SPI.h>
#include <GD.h>
#include <digitalWriteFast.h>
#include <TimerOne.h>

#define RENDERWIDTH 50
#define RENDERHEIGHT 28
byte fire_buffer[(RENDERWIDTH*RENDERHEIGHT)+RENDERWIDTH+1];

//#include <avr/pgmspace.h>

#include "soundbuffer.h"
byte writepointer = 0;
#define SOUNDBUFFER 0x3f00
static uint8_t nextPwm;

#include "talkie.h"

class Plasma
{
public:
  Plasma(){
    frame=0;
    calcsine();
    calcPalette();  
    sineColor = &sinePalette[0];
  }

  void render()
  {
    drawplasma(frame>>1, 256 - frame>>1);
    frame = (frame + 1) & 511;
  }

private:
  unsigned char sine[512];
  unsigned char sinePalette[256];
  unsigned char *sineColor;
  int frame;

  void calcPalette()
  {
    for(int i = 0; i < 256; i++)
    {
      int r = int((cos(i * 2.0 * PI / 256.0) + 1) * 32);
      int g = int(sin(i * 2.0 * PI / 512.0) * 255 * cos(i * 2.0 * PI / 1024.0));
      int b = int(sin(i * 2.0 * PI / 512.0) * 255);
      sinePalette[i] = color(b,g,r);
    }
  }

  void calcsine()
  {
    int c;
    for(c = 0; c < 512; c++)
    {
      sine[c] = int(sin(c * PI/ 64) * 64);
    } 
  }

  void drawplasma(int xa, int ya)
  {
    int x, y;
    int t2, t4;
    unsigned char *s1, *s2;

    for(y = 0; y < RENDERHEIGHT; y++) 
    {
      t2 = *(sine + ((xa + ya - y) & 255));
      s1 = (sine + xa);
      s2 = (sine + ((xa - ya + y ) & 255) + 256);

      for(x = 0; x < RENDERWIDTH; x++)
      {
        t4 = ((*(s1++) + *(s2--) + t2) & 254) + 1;
        fire_buffer[x+y*50] = *(sinePalette + t4);
      }
    }
  }
};


void generate_332_palette()
{
  for (int c=0; c < 256; c++) {
    int r = (c&7) * 0xff/7;
    int g = ((c>>3)&7) * 0xff/7;
    int b = ((c>>6)&3) * 0xff/3;
    GD.setpal(c*4, RGB(r,g,b));
  }
} 

//Talkie voice;
  Talkie voice;


void setup() {

  GD.begin();
  GD.fill(RAM_PIC, 0, 0x0FFF); // 255 = color index
  GD.fill(RAM_CHR, 0, 0x0FFF); // second number is color 0-3  

  GD.microcode(soundbuffer_code, sizeof(soundbuffer_code));

  generate_332_palette();

  // GD.ascii();
  //  GD.wr16(RAM_SPRPAL + 2 * 255, TRANSPARENT);

  //Timer1.initialize(28000);         // initialize timer1, and set the period in microseconds.     
  //Timer1.attachInterrupt(fillTalkSampleBuffer);  // attaches callback() as a timer overflow interrupt  

  GD.putstr(0, 6, "Toms diner");


  // initialize timer1, and set the period in microseconds.     
  Timer1.initialize(28000);         

  // attaches callback() as a timer overflow interrupt  
  Timer1.attachInterrupt(fillTalkSampleBuffer);  


 // runIntroSpeech();
    voice.say(lpc_invite);

}

void loop() {
}

void fillTalkSampleBuffer(){
  byte readpointer = GD.rd(COMM+0);
  byte fullness = writepointer - readpointer;

  while (fullness < 254  ) {
    voice.fillTalkSample();
    GD.wr(SOUNDBUFFER  + writepointer,(nextPwm));                 
    writepointer++;
    fullness++;
  } 
}

/* new hotness */
#define LRBLOCK (RENDERHEIGHT*64)+RAM_PIC // last visible  address on bottom right of screen

// fast version of fill buffer.   frame buffer as array. copy to screen characters
void fb(){
  unsigned int pixelCount =0;
  // character map is 64x64 but we only need to updated 50x37 so we skip every 64 Y blocks 
  for (unsigned int y = RAM_PIC; y < LRBLOCK;y+=64) { 
    byte fillWidth = RENDERWIDTH;
    GD.__wstart(y);
    while (fillWidth--)
    {
      SPI.transfer(fire_buffer[pixelCount++]);
    }
    GD.__end();
  }  
}

// uVGA palette RGB color conversion 
byte color(byte r, byte g, byte b){
  return ((b&255 & 0xC0)+((g&255 & 0xE0) >> 2)+((r & 0xE0) >> 5))&0xFF;
}

void runPlasma(int framesToRun){
  Plasma* plasma;
  plasma = new Plasma();
  do{
    plasma->render();

    fb();
    //       GD.screenshot(frameCounter++);  
  }
  while(--framesToRun);

  delete(plasma);  
}

// intro voice
/*
void runIntroSpeech(){
  // initialize timer1, and set the period in microseconds.     
  Timer1.initialize(28000);         

  // attaches callback() as a timer overflow interrupt  
  Timer1.attachInterrupt(fillTalkSampleBuffer);  

  voice = new Talkie();

   voice->say(spDINER);
  //voice->say(lpc_intro);
  //voice->say(lpc_invite);

  // stop calling the audio buffer
  Timer1.stop();
  Timer1.detachInterrupt();

  // silence woman!  
  delete(voice);  
}
*/

